@@ -1,7 +1,7 @@
 // qtractorAudioEngine.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -32,7 +32,7 @@
 #include "qtractorMonitor.h"
 #include "qtractorSessionCursor.h"
 #include "qtractorMidiEngine.h"
-#include "qtractorMidiBuffer.h"
+#include "qtractorMidiManager.h"
 #include "qtractorPlugin.h"
 #include "qtractorClip.h"
 
@@ -40,6 +40,12 @@
 
 #include "qtractorMainForm.h"
 
+#ifdef CONFIG_LV2
+#ifdef CONFIG_LV2_TIME
+#include "qtractorLv2Plugin.h"
+#endif
+#endif
+
 #ifdef CONFIG_JACK_SESSION
 #include <jack/session.h>
 #endif
@@ -925,27 +931,27 @@ int qtractorAudioEngine::process ( unsigned int nframes )
 				qtractorAudioMonitor *pAudioMonitor
 					= static_cast<qtractorAudioMonitor *> (pTrack->monitor());
 				// Pre-monitoring...
-				if (pAudioMonitor && pInputBus) {
-					// Record non-passthru metering...
-					if (pTrack->isRecord()) {
-						pAudioMonitor->process_meter(
-							pInputBus->in(), nframes, pInputBus->channels());
-					}
-					// Monitor passthru processing...
-					if (pSession->isTrackMonitor(pTrack)) {
-						pAudioMonitor->process(
-							pInputBus->in(), nframes, pInputBus->channels());
-						// Plugin-chain processing...
-						qtractorAudioBus *pOutputBus
-							= static_cast<qtractorAudioBus *> (pTrack->outputBus());
-						if (pOutputBus) {
-							pOutputBus->buffer_prepare(nframes, pInputBus);
-							if ((pTrack->pluginList())->activated() > 0)
-								(pTrack->pluginList())->process(
-									pOutputBus->buffer(), nframes);
-							pOutputBus->buffer_commit(nframes);
-							++iOutputBus;
-						}
+				if (pAudioMonitor == NULL)
+					continue;
+				// Record non-passthru metering...
+				if (pTrack->isRecord() && pInputBus) {
+					pAudioMonitor->process_meter(
+						pInputBus->in(), nframes, pInputBus->channels());
+				}
+				// Monitor passthru/insert processing...
+				qtractorPluginList *pPluginList = pTrack->pluginList();
+				if (pSession->isTrackMonitor(pTrack)
+					|| pPluginList->isAudioInsertActivated()) {
+					// Plugin-chain processing...
+					qtractorAudioBus *pOutputBus
+						= static_cast<qtractorAudioBus *> (pTrack->outputBus());
+					if (pOutputBus) {
+						pOutputBus->buffer_prepare(nframes, pInputBus);
+						if (pPluginList->isActivated())
+							pPluginList->process(pOutputBus->buffer(), nframes);
+						pAudioMonitor->process(pOutputBus->buffer(), nframes);
+						pOutputBus->buffer_commit(nframes);
+						++iOutputBus;
 					}
 				}
 			}
@@ -1427,8 +1433,13 @@ bool qtractorAudioEngine::fileExport (
 	jack_set_freewheel(m_pJackClient, 1);
 
 	// Wait for the export to end.
+	struct timespec ts;
+	ts.tv_sec  = 0;
+	ts.tv_nsec = 20000000L; // 20msec.
+
 	while (m_bExporting && !m_bExportDone) {
 		qtractorSession::stabilize(200);
+		::nanosleep(&ts, NULL); // Ain't that enough?
 		pProgressBar->setValue(pSession->playHead());
 	}
 
@@ -1478,8 +1489,7 @@ bool qtractorAudioEngine::fileExport (
 // Special track-immediate methods.
 void qtractorAudioEngine::trackMute ( qtractorTrack *pTrack, bool bMute )
 {
-	if (bMute)
-		(pTrack->pluginList())->resetBuffer();
+	if (bMute) (pTrack->pluginList())->resetBuffers();
 
 	sessionCursor()->updateTrackClip(pTrack);
 }
@@ -1662,12 +1672,10 @@ bool qtractorAudioEngine::openMetroBus (void)
 	}
 
 	// We got it...
-	const unsigned int iSampleRate = sampleRate();
-	m_pMetroBarBuff = new qtractorAudioBuffer(
-		m_pSyncThread, iChannels, iSampleRate);
+	m_pMetroBarBuff = new qtractorAudioBuffer(m_pSyncThread, iChannels);
 	m_pMetroBarBuff->open(m_sMetroBarFilename);
-	m_pMetroBeatBuff = new qtractorAudioBuffer(
-		m_pSyncThread, iChannels, iSampleRate);
+
+	m_pMetroBeatBuff = new qtractorAudioBuffer(m_pSyncThread, iChannels);
 	m_pMetroBeatBuff->open(m_sMetroBeatFilename);
 
 	return true;
@@ -1857,8 +1865,7 @@ bool qtractorAudioEngine::openPlayerBus (void)
 	}
 
 	// We got it...
-	m_pPlayerBuff = new qtractorAudioBuffer(
-		m_pSyncThread, iChannels, sampleRate());
+	m_pPlayerBuff = new qtractorAudioBuffer(m_pSyncThread, iChannels);
 
 	return true;
 }
@@ -2472,7 +2479,7 @@ void qtractorAudioBus::process_monitor ( unsigned int nframes )
 		= qtractorAudioBus::busMode();
 
 	if (busMode & qtractorBus::Input) {
-		if (m_pIPluginList && m_pIPluginList->activated())
+		if (m_pIPluginList && m_pIPluginList->isActivated())
 			m_pIPluginList->process(m_ppIBuffer, nframes);
 		if (m_pIAudioMonitor)
 			m_pIAudioMonitor->process(m_ppIBuffer, nframes);
@@ -2490,7 +2497,7 @@ void qtractorAudioBus::process_commit ( unsigned int nframes )
 	if (!m_bEnabled)
 		return;
 
-	if (m_pOPluginList && m_pOPluginList->activated())
+	if (m_pOPluginList && m_pOPluginList->isActivated())
 		m_pOPluginList->process(m_ppOBuffer, nframes);
 	if (m_pOAudioMonitor)
 		m_pOAudioMonitor->process(m_ppOBuffer, nframes);
@@ -2724,18 +2731,8 @@ unsigned int qtractorAudioBus::latency_out (void) const
 // Create plugin-list properly.
 qtractorPluginList *qtractorAudioBus::createPluginList ( int iFlags ) const
 {
-	qtractorSession *pSession = engine()->session();
-	if (pSession == NULL)
-		return NULL;
-
 	// Create plugin-list alright...
-	unsigned int iSampleRate = 0;
-	qtractorAudioEngine *pAudioEngine = pSession->audioEngine();
-	if (pAudioEngine)
-		iSampleRate = pAudioEngine->sampleRate();
-
-	qtractorPluginList *pPluginList
-		= new qtractorPluginList(0, 0, iSampleRate, iFlags);
+	qtractorPluginList *pPluginList	= new qtractorPluginList(0, iFlags);
 
 	// Set plugin-list title name...
 	updatePluginListName(pPluginList, iFlags);
@@ -2757,18 +2754,11 @@ void qtractorAudioBus::updatePluginListName (
 void qtractorAudioBus::updatePluginList (
 	qtractorPluginList *pPluginList, int iFlags )
 {
-	// Sanity checks...
-	qtractorAudioEngine *pAudioEngine
-		= static_cast<qtractorAudioEngine *> (engine());
-	if (pAudioEngine == NULL)
-		return;
-
 	// Set plugin-list title name...
 	updatePluginListName(pPluginList, iFlags);
 
 	// Set plugin-list buffer alright...
-	pPluginList->setBuffer(m_iChannels,
-		pAudioEngine->bufferSize(), pAudioEngine->sampleRate(), iFlags);
+	pPluginList->setChannels(m_iChannels, iFlags);
 }
 
 
