@@ -1,7 +1,7 @@
 // qtractorCurve.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -28,17 +28,16 @@
 
 
 // Possible cube root optimization.
-// (borrowed from metamerist.com)
 static inline float cbrtf2 ( float x )
 {
 #ifdef CONFIG_FLOAT32
 	// Avoid strict-aliasing optimization (gcc -O2).
 	union { float f; int i; } u;
 	u.f = x;
-	u.i = (u.i / 3) + 710235478;
+	u.i = (u.i / 3) + 710235478; // HACK: 709621077 for cbrtf2(1)=1
 	return u.f;
 #else
-	return cbrtf(x);
+	return ::cbrtf(x);
 #endif
 }
 
@@ -115,22 +114,22 @@ inline void updateNodeSpline ( qtractorCurve::Node *pNode, float y0,
 		y1 = y0;
 	}
 
-	if (fabs(y1 - y2) > fZero)
+	if (::fabsf(y1 - y2) > fZero)
 		s1 =  x1 / (y1 - y2);
 
 	if (pPrev) {
 		pPrev = pPrev->prev();
-		if (pPrev && (fabs(y1 - pPrev->value) > fZero)) {
+		if (pPrev && (::fabsf(y1 - pPrev->value) > fZero)) {
 			const float s0
 				= (x1 - float(pPrev->frame) - x0) / (y1 - pPrev->value);
-			if (s1 * s0 > 0.0f && fabs(s1 + s0) > fZero)
+			if (s1 * s0 > 0.0f && ::fabsf(s1 + s0) > fZero)
 				f1 = 2.0f / (s1 + s0);
 		}
 	}
 
-	if (pNext && (fabs(pNext->value - y2) > fZero))
+	if (pNext && (::fabsf(pNext->value - y2) > fZero))
 		s2 = (float(pNext->frame) - x0) / (pNext->value - y2);
-	if (s2 * s1 > 0.0f && fabs(s2 + s1) > fZero)
+	if (s2 * s1 > 0.0f && ::fabsf(s2 + s1) > fZero)
 		f2 = 2.0f / (s2 + s1);
 
 	const float x12 = x1 * x1;
@@ -262,12 +261,12 @@ qtractorCurve::Node *qtractorCurve::addNode (
 		float y2 = (pNext ? pNext->value : fValue);
 		if (m_mode == Hold || !m_observer.isDecimal()) {
 			const float fThreshold
-				= (m_bLogarithmic ? 0.1f * fabs(y1) : 0.01f)
+				= (m_bLogarithmic ? 0.1f * ::fabsf(y1) : 0.01f)
 				* (m_observer.maxValue() - m_observer.minValue());
-			if (fabs(y2 - y1) < fThreshold)
+			if (::fabsf(y2 - y1) < fThreshold)
 				pNode = pNext;
 			else
-			if (fabs(y1 - y0) < fThreshold)
+			if (::fabsf(y1 - y0) < fThreshold)
 				return NULL;
 		} else {
 			const float fThreshold = 0.5f; // (m_bLogarithmic ? 0.1f : 0.5f);
@@ -276,7 +275,7 @@ qtractorCurve::Node *qtractorCurve::addNode (
 			float x2 = (pNext ? float(pNext->frame) : m_tail.frame);
 			float s1 = (x1 > x0 ? (y1 - y0) / (x1 - x0) : 0.0f);
 			float y3 = (x2 > x1 ? s1 * (x2 - x1) + y1 : y1);
-			if (fabs(y3 - y2) < fThreshold * fabs(y3 - y1))
+			if (::fabsf(y3 - y2) < fThreshold * ::fabsf(y3 - y1))
 				return NULL;
 			if (pPrev) {
 				pNode = pPrev;
@@ -289,7 +288,7 @@ qtractorCurve::Node *qtractorCurve::addNode (
 				y2 = fValue;
 				s1 = (y1 - y0) / (x1 - x0);
 				y3 = s1 * (x2 - x1) + y1;
-				if (fabs(y3 - y2) > fThreshold * fabs(y3 - y1))
+				if (::fabsf(y3 - y2) > fThreshold * ::fabsf(y3 - y1))
 					pNode = NULL;
 			}
 		}
@@ -619,6 +618,27 @@ float qtractorCurve::scaleFromValue ( float fValue ) const
 }
 
 
+// Copy all events from another curve (raw-copy).
+void qtractorCurve::copyNodes ( qtractorCurve *pCurve )
+{
+	// Check whether we're the same...
+	if (pCurve == this)
+		return;
+
+	// Remove existing nodes.
+	m_nodes.clear();
+
+	// Clone new ones...
+	Node *pNode = pCurve->nodes().first();
+	for (; pNode; pNode = pNode->next())
+		m_nodes.append(new Node(pNode->frame, pNode->value));
+
+	// Done.
+	update();
+}
+
+
+
 // Convert MIDI sequence events to curve nodes.
 void qtractorCurve::readMidiSequence ( qtractorMidiSequence *pSeq,
 	qtractorMidiEvent::EventType ctype, unsigned short iChannel,
