@@ -1,7 +1,7 @@
 // qtractorCurve.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -223,6 +223,9 @@ public:
 	qtractorCurveEditList *editList() const
 		{ return m_pEditList; }
 
+	// Copy all events from another curve (raw-copy).
+	void copyNodes(qtractorCurve *pCurve);
+
 	// Convert MIDI sequence events to curve nodes.
 	void readMidiSequence(qtractorMidiSequence *pSeq,
 		qtractorMidiEvent::EventType ctype, unsigned short iChannel,
