@@ -1,7 +1,7 @@
 // qtractorEngine.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -132,6 +132,19 @@ void qtractorEngine::removeBus ( qtractorBus *pBus )
 }
 
 
+// Move bus on a device engine.
+void qtractorEngine::moveBus (
+	qtractorBus *pBus, qtractorBus *pNextBus )
+{
+	m_buses.unlink(pBus);
+
+	if (pNextBus)
+		m_buses.insertBefore(pBus, pNextBus);
+	else
+		m_buses.append(pBus);
+}
+
+
 // Find a device bus by name
 qtractorBus *qtractorEngine::findBus ( const QString& sBusName ) const
 {
