@@ -1,7 +1,7 @@
 // qtractorInstrumentForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -405,7 +405,7 @@ void qtractorInstrumentForm::exportSlot (void)
 		sPath = fileDialog.selectedFiles().first();
 #endif
 
-	if (sPath.isEmpty())
+	if (sPath.isEmpty() || sPath.at(0) == '.')
 		return;
 
 	// Enforce .ins extension...
