@@ -1,7 +1,7 @@
 // qtractorMainForm.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -44,8 +44,13 @@ class qtractorTimeSpinBox;
 class qtractorTempoSpinBox;
 class qtractorTempoCursor;
 
+class qtractorInstrumentMenu;
+
 class qtractorMidiEditorForm;
 class qtractorMidiEditor;
+class qtractorMidiManager;
+
+class qtractorPluginFactory;
 
 class qtractorActionControl;
 class qtractorMidiControlObserver;
@@ -151,6 +156,7 @@ public slots:
 
 	void trackAdd();
 	void trackRemove();
+	void trackDuplicate();
 	void trackProperties();
 	void trackInputs();
 	void trackOutputs();
@@ -197,6 +203,7 @@ public slots:
 	void clipMerge();
 	void clipNormalize();
 	void clipTempoAdjust();
+	void clipCrossFade();
 	void clipRangeSet();
 	void clipLoopSet();
 	void clipImport();
@@ -309,6 +316,8 @@ protected slots:
 	void updateZoomMenu();
 	void updateSnapMenu();
 
+	void updateTrackInstrumentMenu();
+
 	void selectAudioFile(const QString& sFilename, int iTrackChannel, bool bSelect);
 	void activateAudioFile(const QString& sFilename, int iTrackChannel = -1);
 
@@ -394,7 +403,6 @@ protected:
 	void updateMessagesLimit();
 	void updateMessagesCapture();
 	void updateDisplayFormat();
-	void updatePluginPaths();
 	void updateTransportMode();
 	void updateTimebase();
 	void updateMidiControlModes();
@@ -412,7 +420,8 @@ protected:
 	void updateDirtyCount(bool bDirtyCount);
 
 	void trackCurveSelectMenuAction(QMenu *pMenu,
-		qtractorMidiControlObserver *pObserver, qtractorSubject *pCurrentSubject) const;
+		qtractorMidiControlObserver *pObserver,
+		qtractorSubject *pCurrentSubject) const;
 
 	bool trackCurveSelectMenuReset(QMenu *pMenu) const;
 	bool trackCurveModeMenuReset(QMenu *pMenu) const;
@@ -438,6 +447,7 @@ private:
 	qtractorMixer *m_pMixer;
 	qtractorTracks *m_pTracks;
 	qtractorMessageList *m_pMessageList;
+	qtractorPluginFactory *m_pPluginFactory;
 	QString m_sFilename;
 	int m_iUntitled;
 	int m_iDirtyCount;
@@ -451,6 +461,7 @@ private:
 	QProgressBar *m_pProgressBar;
 	qtractorThumbView *m_pThumbView;
 	qtractorTempoCursor *m_pTempoCursor;
+	qtractorInstrumentMenu *m_pInstrumentMenu;
 	qtractorActionControl *m_pActionControl;
 	qtractorMidiControl *m_pMidiControl;
 	qtractorNsmClient *m_pNsmClient;
@@ -486,9 +497,10 @@ private:
 		StatusMute    = 3,   // Current session muting state.
 		StatusSolo    = 4,   // Current session soloing state.
 		StatusLoop    = 5,   // Current session looping state.
-		StatusTime    = 6,   // Current session length time.
-		StatusRate    = 7,   // Current session sample rate.
-		StatusItems   = 8    // Number of status items.
+		StatusXrun    = 6,   // Current session XRUN count.
+		StatusTime    = 7,   // Current session length time.
+		StatusRate    = 8,   // Current session sample rate.
+		StatusItems   = 9    // Number of status items.
 	};
 
 	QLabel *m_statusItems[StatusItems];
