@@ -1,7 +1,7 @@
 // qtractorMidiClip.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -28,7 +28,7 @@
 
 #include "qtractorDocument.h"
 
-#include "qtractorMidiBuffer.h"
+#include "qtractorMidiManager.h"
 #include "qtractorPlugin.h"
 
 #include "qtractorMidiEditor.h"
@@ -36,8 +36,7 @@
 
 #include "qtractorMainForm.h"
 
-
-#ifdef QTRACTOR_MIDI_EDITOR_TOOL
+#if 0//QTRACTOR_MIDI_EDITOR_TOOL
 #include "qtractorOptions.h"
 #endif
 
@@ -391,14 +390,10 @@ bool qtractorMidiClip::openMidiFile (
 	qtractorTimeScale::Cursor cursor(pSession->timeScale());
 	qtractorTimeScale::Node *pNode = cursor.seekFrame(iClipStart);
 	const unsigned long t0 = pNode->tickFromFrame(iClipStart);
-	if (iClipStart > iClipOffset) {
-		const unsigned long iFrameOffset = iClipStart - iClipOffset;
-		pNode = cursor.seekFrame(iFrameOffset);
-		pSeq->setTimeOffset(t0 - pNode->tickFromFrame(iFrameOffset));
-	} else {
-		pNode = cursor.seekFrame(iClipOffset);
-		pSeq->setTimeOffset(pNode->tickFromFrame(iClipOffset));
-	}
+
+	const unsigned long iClipOffset2 = iClipStart + iClipOffset;
+//	pNode = cursor.seekFrame(iClipOffset2);
+	pSeq->setTimeOffset(pNode->tickFromFrame(iClipOffset2) - t0);
 
 	const unsigned long iClipLength = clipLength();
 	const unsigned long iClipEnd = iClipStart + iClipLength;
@@ -480,6 +475,8 @@ bool qtractorMidiClip::openMidiFile (
 		// We should have events, otherwise this clip is of no use...
 		//if (m_pSeq->events().count() < 1)
 		//	return false;
+		// And initial clip name...
+		pSeq->setName(shortClipName(QFileInfo(m_pFile->filename()).baseName()));
 	}
 
 	// Actual track-channel is set by now...
@@ -1080,25 +1077,18 @@ bool qtractorMidiClip::startEditor ( QWidget *pParent )
 	if (m_pMidiEditorForm == NULL) {
 		// Build up the editor form...
 		// What style do we create tool childs?
-		Qt::WindowFlags wflags = Qt::Window
-			| Qt::CustomizeWindowHint
-			| Qt::WindowTitleHint
-			| Qt::WindowSystemMenuHint
-			| Qt::WindowMinMaxButtonsHint
-			| Qt::WindowCloseButtonHint;
-	#ifdef QTRACTOR_MIDI_EDITOR_TOOL
+		Qt::WindowFlags wflags = Qt::Window;
+	#if 0//QTRACTOR_MIDI_EDITOR_PARENT
 		qtractorOptions *pOptions = qtractorOptions::getInstance();
-		if (pOptions && pOptions->bKeepToolsOnTop)
+		if (pOptions && pOptions->bKeepToolsOnTop) {
 			wflags |= Qt::Tool;
+			// Make sure it has a parent...
+			if (pParent == NULL)
+				pParent = qtractorMainForm::getInstance();
+		}
 	#endif
 		// Do it...
 		m_pMidiEditorForm = new qtractorMidiEditorForm(pParent, wflags);
-		// Set its most standing properties...
-		if (!m_posEditor.isNull()
-			&& m_posEditor.x() >= 0 && m_posEditor.y() >= 0)
-			m_pMidiEditorForm->move(m_posEditor);
-		if (!m_sizeEditor.isNull() && m_sizeEditor.isValid())
-			m_pMidiEditorForm->resize(m_sizeEditor);
 		m_pMidiEditorForm->show();
 		m_pMidiEditorForm->setup(this);
 	} else {
@@ -1107,6 +1097,13 @@ bool qtractorMidiClip::startEditor ( QWidget *pParent )
 		m_pMidiEditorForm->show();
 	}
 
+	// Set its most standing properties...
+	if (!m_posEditor.isNull()
+		&& m_posEditor.x() >= 0 && m_posEditor.y() >= 0)
+		m_pMidiEditorForm->move(m_posEditor);
+	if (!m_sizeEditor.isNull() && m_sizeEditor.isValid())
+		m_pMidiEditorForm->resize(m_sizeEditor);
+
 	// Get it up any way...
 	m_pMidiEditorForm->raise();
 	m_pMidiEditorForm->activateWindow();
