@@ -1,7 +1,7 @@
 // qtractorMidiClip.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -126,7 +126,7 @@ public:
 		unsigned long iClipOffset);
 
 	// Clip editor methods.
-	bool startEditor(QWidget *pParent);
+	bool startEditor(QWidget *pParent = NULL);
 	void resetEditor(bool bSelectClear);
 	void updateEditor(bool bSelectClear);
 	void updateEditorContents();
