@@ -1,7 +1,7 @@
 // qtractorMidiControl.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
    Copyright (C) 2009, gizzmo aka Mathias Krause. 
 
    This program is free software; you can redistribute it and/or
@@ -43,14 +43,13 @@
 #include <math.h>
 
 // Possible cube root optimization.
-// (borrowed from metamerist.com)
 static inline float cbrtf2 ( float x )
 {
-#ifdef CONFIG_FLOAT32
+#if 0//ifdef CONFIG_FLOAT32
 	// Avoid strict-aliasing optimization (gcc -O2).
 	union { float f; int i; } u;
 	u.f = x;
-	u.i = (u.i / 3) + 710235478;
+	u.i = (u.i / 3) + 710235478; // HACK: 709621077 for cbrtf2(1)=1
 	return u.f;
 #else
 	return ::cbrtf(x);
@@ -115,6 +114,16 @@ void qtractorMidiControl::clear (void)
 }
 
 
+// Clear track (catch-up) map.
+void qtractorMidiControl::clearControlMap (void)
+{
+	ControlMap::Iterator it = m_controlMap.begin();
+	const ControlMap::Iterator& it_end = m_controlMap.end();
+	for ( ; it != it_end; ++it)
+		it.value().clear();
+}
+
+
 // Insert new controller mappings.
 void qtractorMidiControl::mapChannelParam (
 	ControlType ctype, unsigned short iChannel, unsigned short iParam,
@@ -289,7 +298,9 @@ bool qtractorMidiControl::processEvent ( const qtractorCtlEvent& ctle )
 	if (pTrack == NULL)
 		return bResult;
 
-	Scale scale(ctle.type());
+	ControlScale scale(ctle.type());
+
+	MapVal::Track& ctlv = val.track(iTrack);
 
 	float fValue;
 	switch (val.command()) {
@@ -297,47 +308,47 @@ bool qtractorMidiControl::processEvent ( const qtractorCtlEvent& ctle )
 		fValue = scale.valueFromMidi(ctle.value());
 		if (pTrack->trackType() == qtractorTrack::Audio)
 			fValue = ::cubef2(fValue);
-		if (val.sync(fValue, pTrack->gain())) {
+		if (ctlv.sync(fValue, pTrack->gain())) {
 			bResult = pSession->execute(
-				new qtractorTrackGainCommand(pTrack, val.value(), true));
+				new qtractorTrackGainCommand(pTrack, ctlv.value(), true));
 		}
 		break;
 	case TRACK_PANNING:
 		fValue = scale.valueSignedFromMidi(ctle.value());
-		if (val.sync(fValue, pTrack->panning())) {
+		if (ctlv.sync(fValue, pTrack->panning())) {
 			bResult = pSession->execute(
-				new qtractorTrackPanningCommand(pTrack, val.value(), true));
+				new qtractorTrackPanningCommand(pTrack, ctlv.value(), true));
 		}
 		break;
 	case TRACK_MONITOR:
 		fValue = scale.valueToggledFromMidi(ctle.value());
-		if (val.sync(fValue, (pTrack->isMonitor() ? 1.0f : 0.0f))) {
+		if (ctlv.sync(fValue, (pTrack->isMonitor() ? 1.0f : 0.0f))) {
 			bResult = pSession->execute(
-				new qtractorTrackMonitorCommand(pTrack, val.value(), true));
+				new qtractorTrackMonitorCommand(pTrack, ctlv.value(), true));
 		}
 		break;
 	case TRACK_RECORD:
 		fValue = scale.valueToggledFromMidi(ctle.value());
-		if (val.sync(fValue, (pTrack->isRecord() ? 1.0f : 0.0f))) {
+		if (ctlv.sync(fValue, (pTrack->isRecord() ? 1.0f : 0.0f))) {
 			bResult = pSession->execute(
 				new qtractorTrackStateCommand(pTrack,
-					qtractorTrack::Record, val.value(), true));
+					qtractorTrack::Record, ctlv.value(), true));
 		}
 		break;
 	case TRACK_MUTE:
 		fValue = scale.valueToggledFromMidi(ctle.value());
-		if (val.sync(fValue, (pTrack->isMute() ? 1.0f : 0.0f))) {
+		if (ctlv.sync(fValue, (pTrack->isMute() ? 1.0f : 0.0f))) {
 			bResult = pSession->execute(
 				new qtractorTrackStateCommand(pTrack,
-					qtractorTrack::Mute, val.value(), true));
+					qtractorTrack::Mute, ctlv.value(), true));
 		}
 		break;
 	case TRACK_SOLO:
 		fValue = scale.valueToggledFromMidi(ctle.value());
-		if (val.sync(fValue, (pTrack->isSolo() ? 1.0f : 0.0f))) {
+		if (ctlv.sync(fValue, (pTrack->isSolo() ? 1.0f : 0.0f))) {
 			bResult = pSession->execute(
 				new qtractorTrackStateCommand(pTrack,
-					qtractorTrack::Solo, val.value(), true));
+					qtractorTrack::Solo, ctlv.value(), true));
 		}
 		break;
 	default:
@@ -374,12 +385,12 @@ void qtractorMidiControl::sendTrackController (
 		const MapKey& key = it.key();
 		MapVal& val = it.value();
 		if (val.command() == command) {
-			val.syncReset();
+			val.syncReset(iTrack);
 			if (!val.isFeedback())
 				continue;
 			// Convert/normalize value...
 			const ControlType ctype = key.type();
-			const Scale scale(ctype);
+			const ControlScale scale(ctype);
 			unsigned short iValue = 0;
 			switch (command) {
 			case TRACK_GAIN:
@@ -417,7 +428,7 @@ void qtractorMidiControl::sendTrackController (
 	ControlType ctype, qtractorTrack *pTrack,
 	Command command, unsigned short iChannel, unsigned short iParam ) const
 {
-	const Scale scale(ctype);
+	const ControlScale scale(ctype);
 	unsigned short iValue = 0;
 
 	switch (command) {
@@ -964,4 +975,18 @@ const QString& qtractorMidiControl::nameFromCommand ( Command command )
 }
 
 
+// MIDI control non catch-up/hook global option.
+bool qtractorMidiControl::g_bSync = false;
+
+void qtractorMidiControl::setSync ( bool bSync )
+{
+	g_bSync = bSync;
+}
+
+bool qtractorMidiControl::isSync (void)
+{
+	return g_bSync;
+}
+
+
 // end of qtractorMidiControl.cpp
