@@ -1,7 +1,7 @@
 // qtractorMidiControlObserver.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -26,17 +26,16 @@
 #include <math.h>
 
 // Possible cube root optimization.
-// (borrowed from metamerist.com)
 static inline float cbrtf2 ( float x )
 {
-#ifdef CONFIG_FLOAT32
+#if 0//ifdef CONFIG_FLOAT32
 	// Avoid strict-aliasing optimization (gcc -O2).
 	union { float f; int i; } u;
 	u.f = x;
-	u.i = (u.i / 3) + 710235478;
+	u.i = (u.i / 3) + 710235478; // HACK: 709621077 for cbrtf2(1)=1
 	return u.f;
 #else
-	return cbrtf(x);
+	return ::cbrtf(x);
 #endif
 }
 
@@ -103,7 +102,7 @@ void qtractorMidiControlObserver::setMidiValue ( unsigned short iValue )
 		if (fValue > vmid)
 			fValue = (m_fMidiValue > vmid ? vmin : vmax);
 		else
-			fValue = (m_fMidiValue > vmid ? vmax : vmin);
+			fValue = (m_fMidiValue < vmid ? vmax : vmin);
 	}
 
 	bool bSync = (m_bHook || !qtractorObserver::isDecimal());
@@ -120,14 +119,14 @@ void qtractorMidiControlObserver::setMidiValue ( unsigned short iValue )
 		const float d2 = (m_bMidiSync ? (v1 - v0) : d1) * d1;
 		bSync = (d2 < 0.001f);
 	#endif
-		if (bSync) {
-			m_fMidiValue = fValue;
+		if (bSync)
 			m_bMidiSync = true;
-		}
 	}
 
-	if (bSync)
-		subject()->setValue(fValue);
+	if (bSync) {
+		m_fMidiValue = fValue;
+		qtractorObserver::subject()->setValue(fValue);
+	}
 }
 
 unsigned short qtractorMidiControlObserver::midiValue (void) const
