@@ -1,7 +1,7 @@
 // qtractorMidiEditView.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -154,8 +154,9 @@ void qtractorMidiEditView::updateContentsWidth ( int iContentsWidth )
 			if (iContentsWidth < w0)
 				iContentsWidth = w0;
 		}
-		iContentsWidth += pTimeScale->pixelFromBeat(
-			2 * pTimeScale->beatsPerBar()) + qtractorScrollView::width();
+		iContentsWidth += pTimeScale->pixelFromBeat(pTimeScale->beatsPerBar());
+		if (iContentsWidth  < qtractorScrollView::width())
+			iContentsWidth += qtractorScrollView::width();
 	}
 
 	qtractorScrollView::resizeContents(
