@@ -1,7 +1,7 @@
 // qtractorMidiEditorForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -46,7 +46,7 @@
 
 #include "qtractorMidiThumbView.h"
 #include "qtractorMidiEventList.h"
-
+#include "qtractorInstrumentMenu.h"
 #include "qtractorFileList.h"
 
 #include "qtractorClipCommand.h"
@@ -65,8 +65,8 @@
 
 // Constructor.
 qtractorMidiEditorForm::qtractorMidiEditorForm (
-	QWidget */*pParent*/, Qt::WindowFlags wflags )
-	: QMainWindow(NULL/*pParent*/, wflags)
+	QWidget *pParent, Qt::WindowFlags wflags )
+	: QMainWindow(pParent, wflags)
 {
 	// Setup UI struct...
 	m_ui.setupUi(this);
@@ -87,6 +87,9 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 	m_pMidiEventList->setEditor(m_pMidiEditor);
 	m_pMidiEventList->hide(); // Initially hidden.
 
+	// Custom track/instrument proxy menu.
+	m_pInstrumentMenu = new qtractorInstrumentMenu(this);
+
 	// Set edit-mode action group up...
 	m_pEditModeActionGroup = new QActionGroup(this);
 	m_pEditModeActionGroup->setExclusive(true);
@@ -182,6 +185,40 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 	m_pSnapToScaleKeyComboBox->setToolTip(tr("Scale key"));
 	m_pSnapToScaleTypeComboBox->setToolTip(tr("Scale type"));
 
+	// Late view/note type menu...
+	const QString sViewTypeObjectName("viewNoteType%1");
+	const QString sViewTypeStatusTip("Set note type to %1");
+	const int iViewTypeCount = m_pViewTypeComboBox->count();
+	for (int iIndex = 0; iIndex < iViewTypeCount; ++iIndex) {
+		const QString& sViewTypeText = m_pViewTypeComboBox->itemText(iIndex);
+		QAction *pAction = new QAction(sViewTypeText, this);
+		pAction->setObjectName(sViewTypeObjectName.arg(iIndex));
+		pAction->setStatusTip(sViewTypeStatusTip.arg(sViewTypeText));
+		pAction->setCheckable(true);
+		pAction->setData(iIndex);
+		QObject::connect(pAction,
+			SIGNAL(triggered(bool)),
+			SLOT(viewNoteType()));
+		m_ui.viewNoteTypeMenu->addAction(pAction);
+	}
+
+	// Late event/type type menu...
+	const QString sEventTypeObjectName("viewValueType%1");
+	const QString sEventTypeStatusTip("Set value type to %1");
+	const int iEventTypeCount = m_pEventTypeComboBox->count();
+	for (int iIndex = 0; iIndex < iEventTypeCount; ++iIndex) {
+		const QString& sEventTypeText = m_pEventTypeComboBox->itemText(iIndex);
+		QAction *pAction = new QAction(sEventTypeText, this);
+		pAction->setObjectName(sEventTypeObjectName.arg(iIndex));
+		pAction->setStatusTip(sEventTypeStatusTip.arg(sEventTypeText));
+		pAction->setCheckable(true);
+		pAction->setData(iIndex);
+		QObject::connect(pAction,
+			SIGNAL(triggered(bool)),
+			SLOT(viewValueType()));
+		m_ui.viewValueTypeMenu->addAction(pAction);
+	}
+
 	// Add combo-boxes to toolbars...
 	m_ui.viewToolbar->addSeparator();
 	m_ui.viewToolbar->addWidget(m_pSnapPerBeatComboBox);
@@ -457,6 +494,15 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 		SIGNAL(triggered(bool)),
 		SLOT(helpAboutQt()));
 
+	QObject::connect(m_ui.fileTrackInstrumentMenu,
+		SIGNAL(aboutToShow()),
+		SLOT(updateTrackInstrumentMenu()));
+	QObject::connect(m_ui.viewNoteTypeMenu,
+		SIGNAL(aboutToShow()),
+		SLOT(updateNoteTypeMenu()));
+	QObject::connect(m_ui.viewValueTypeMenu,
+		SIGNAL(aboutToShow()),
+		SLOT(updateValueTypeMenu()));
 	QObject::connect(m_ui.viewZoomMenu,
 		SIGNAL(aboutToShow()),
 		SLOT(updateZoomMenu()));
@@ -621,7 +667,7 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 		const qtractorMidiControl::ControlType ctype
 			= m_pEventTypeGroup->controlTypeFromIndex(pOptions->iMidiEventType);
 		m_pEventTypeGroup->setControlType(ctype);
-		eventTypeChanged(ctype);
+	//	eventTypeChanged(pOptions->iMidiEventType);
 		m_pEventTypeGroup->setControlParam(pOptions->iMidiEventParam);
 		viewTypeChanged(pOptions->iMidiViewType);
 	} else {
@@ -649,6 +695,10 @@ qtractorMidiEditorForm::~qtractorMidiEditorForm (void)
 	if (m_pMidiEditor)
 		delete m_pMidiEditor;
 
+	// Destroy custom track/instrument proxy menu.
+	if (m_pInstrumentMenu)
+		delete m_pInstrumentMenu;
+
 	// Get edit-mode action group down.
 	if (m_pEditModeActionGroup)
 		delete m_pEditModeActionGroup;
@@ -864,8 +914,7 @@ void qtractorMidiEditorForm::setup ( qtractorMidiClip *pMidiClip )
 	// form initialization and first setup or else... 
 	if (pMidiClip) {
 		// Set initial MIDI clip properties has seen fit...
-		if (queryClose())
-			m_pMidiEditor->setMidiClip(pMidiClip);
+		m_pMidiEditor->setMidiClip(pMidiClip);
 		// Setup connections to main widget...
 		QObject::connect(m_pMidiEditor,
 			SIGNAL(changeNotifySignal(qtractorMidiEditor *)),
@@ -987,7 +1036,7 @@ bool qtractorMidiEditorForm::saveClipFile ( bool bPrompt )
 			sFilename.clear();
 	#endif
 		// Have we cancelled it?
-		if (sFilename.isEmpty())
+		if (sFilename.isEmpty() || sFilename.at(0) == '.')
 			return false;
 		// Enforce .mid extension...
 		if (QFileInfo(sFilename).suffix().isEmpty())
@@ -1456,6 +1505,38 @@ void qtractorMidiEditorForm::viewNoteDuration ( bool bOn )
 }
 
 
+// Change view/note type setting via menu.
+void qtractorMidiEditorForm::viewNoteType (void)
+{
+	// Retrieve view/note type index from from action data...
+	QAction *pAction = qobject_cast<QAction *> (sender());
+	if (pAction) {
+		const int iIndex = pAction->data().toInt();
+		// Update the other toolbar control...
+		m_pViewTypeComboBox->setCurrentIndex(iIndex);
+		// Commit the change as usual...
+		viewTypeChanged(iIndex);
+	}
+}
+
+
+// Change event/value type setting via menu.
+void qtractorMidiEditorForm::viewValueType (void)
+{
+	// Retrieve event/value type index from from action data...
+	QAction *pAction = qobject_cast<QAction *> (sender());
+	if (pAction) {
+		const int iIndex = pAction->data().toInt();
+		// Update the other toolbar control...
+		const qtractorMidiControl::ControlType ctype
+			= m_pEventTypeGroup->controlTypeFromIndex(iIndex);
+		m_pEventTypeGroup->setControlType(ctype);
+		// Commit the change as usual...
+		// eventTypeChanged(iIndex);
+	}
+}
+
+
 // Show/hide the events window view.
 void qtractorMidiEditorForm::viewEvents ( bool bOn )
 {
@@ -1674,6 +1755,7 @@ void qtractorMidiEditorForm::stabilizeForm (void)
 
 	m_ui.fileTrackInputsAction->setEnabled(pTrack && pTrack->inputBus() != NULL);
 	m_ui.fileTrackOutputsAction->setEnabled(pTrack && pTrack->outputBus() != NULL);
+	m_ui.fileTrackInstrumentMenu->setEnabled(pTrack != NULL);
 	m_ui.fileTrackPropertiesAction->setEnabled(pTrack != NULL);
 	m_ui.fileRangeSetAction->setEnabled(pTrack != NULL);
 	m_ui.fileLoopSetAction->setEnabled(pTrack != NULL);
@@ -1830,11 +1912,35 @@ void qtractorMidiEditorForm::updatePlayHead ( unsigned long iPlayHead )
 //-------------------------------------------------------------------------
 // qtractorMidiEditorForm -- Selection widget slots.
 
+// Note type view menu stabilizer.
+void qtractorMidiEditorForm::updateNoteTypeMenu (void)
+{
+	const int iCurrentIndex = m_pViewTypeComboBox->currentIndex();
+	QListIterator<QAction *> iter(m_ui.viewNoteTypeMenu->actions());
+	while (iter.hasNext()) {
+		QAction *pAction = iter.next();
+		pAction->setChecked(pAction->data().toInt() == iCurrentIndex);
+	}
+}
+
+
+// Event type view menu stabilizer.
+void qtractorMidiEditorForm::updateValueTypeMenu (void)
+{
+	const int iCurrentIndex = m_pEventTypeComboBox->currentIndex();
+	QListIterator<QAction *> iter(m_ui.viewValueTypeMenu->actions());
+	while (iter.hasNext()) {
+		QAction *pAction = iter.next();
+		pAction->setChecked(pAction->data().toInt() == iCurrentIndex);
+	}
+}
+
+
 // Zoom view menu stabilizer.
 void qtractorMidiEditorForm::updateZoomMenu (void)
 {
 	const int iZoomMode = m_pMidiEditor->zoomMode();
-	
+
 	m_ui.viewZoomHorizontalAction->setChecked(
 		iZoomMode == qtractorMidiEditor::ZoomHorizontal);
 	m_ui.viewZoomVerticalAction->setChecked(
@@ -1902,6 +2008,17 @@ void qtractorMidiEditorForm::updateScaleMenu (void)
 }
 
 
+// Track/Instrument sub-menu stabilizers.
+void qtractorMidiEditorForm::updateTrackInstrumentMenu (void)
+{
+	qtractorMidiClip *pMidiClip = m_pMidiEditor->midiClip();
+	if (pMidiClip) {
+		m_pInstrumentMenu->updateTrackMenu(
+			pMidiClip->track(), m_ui.fileTrackInstrumentMenu);
+	}
+}
+
+
 // Snap-per-beat spin-box change slot.
 void qtractorMidiEditorForm::snapPerBeatChanged ( int iSnap )
 {
@@ -1953,10 +2070,11 @@ void qtractorMidiEditorForm::viewTypeChanged ( int iIndex )
 }
 
 
-void qtractorMidiEditorForm::eventTypeChanged ( int etype )
+void qtractorMidiEditorForm::eventTypeChanged ( int iIndex )
 {
 	const qtractorMidiEvent::EventType eventType
-		= qtractorMidiEvent::EventType(etype);
+		= qtractorMidiEvent::EventType(
+			m_pEventTypeComboBox->itemData(iIndex).toInt());
 
 	m_pEventParamComboBox->setEnabled(
 		eventType == qtractorMidiEvent::CONTROLLER  ||
@@ -1967,6 +2085,10 @@ void qtractorMidiEditorForm::eventTypeChanged ( int etype )
 //	updateInstrumentNames();
 
 	m_pMidiEditor->editEvent()->setEventType(eventType);
+	m_pMidiEditor->updateContents();
+	m_pMidiEventList->refresh();
+
+	stabilizeForm();
 }
 
 
