@@ -1,7 +1,7 @@
 // qtractorMidiEditorForm.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -32,8 +32,8 @@ class qtractorMidiSequence;
 class qtractorTimeScale;
 
 class qtractorMidiEventList;
-
 class qtractorMidiControlTypeGroup;
+class qtractorInstrumentMenu;
 
 class QContextMenuEvent;
 class QActionGroup;
@@ -147,7 +147,9 @@ protected slots:
 	void viewToolbarThumb(bool bOn);
 	void viewNoteDuration(bool bOn);
 	void viewNoteColor(bool bOn);
+	void viewNoteType();
 	void viewValueColor(bool bOn);
+	void viewValueType();
 	void viewEvents(bool bOn);
 	void viewPreview(bool bOn);
 	void viewFollow(bool bOn);
@@ -169,10 +171,15 @@ protected slots:
 	void helpAbout();
 	void helpAboutQt();
 
+	void updateNoteTypeMenu();
+	void updateValueTypeMenu();
+
 	void updateZoomMenu();
 	void updateSnapMenu();
 	void updateScaleMenu();
 
+	void updateTrackInstrumentMenu();
+
 	void sendNote(int iNote, int iVelocity);
 
 	void selectionChanged(qtractorMidiEditor *);
@@ -215,6 +222,9 @@ private:
 	// Edit-mode action group up.
 	QActionGroup *m_pEditModeActionGroup;
 
+	// Custom track/instrument proxy menu.
+	qtractorInstrumentMenu *m_pInstrumentMenu;
+
 	// View/Snap-to-beat actions (for shortcuts access)
 	QList<QAction *> m_snapPerBeatActions;
 
