@@ -1,7 +1,7 @@
 // qtractorMidiEngine.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -40,6 +40,7 @@ class qtractorMidiInputThread;
 class qtractorMidiOutputThread;
 class qtractorMidiMonitor;
 class qtractorMidiSysexList;
+class qtractorMidiInputBuffer;
 class qtractorMidiPlayer;
 class qtractorPluginList;
 class qtractorCurveList;
@@ -130,6 +131,14 @@ public:
 	// Shut-off all MIDI tracks (panic)...
 	void shutOffAllTracks() const;
 
+	// ALSA port input registry methods.
+	void addInputBus(qtractorMidiBus *pMidiBus);
+	void removeInputBus(qtractorMidiBus *pMidiBus);
+
+	void addInputBuffer(int iAlsaPort,
+		qtractorMidiInputBuffer *pMidiInputBuffer);
+	void removeInputBuffer(int iAlsaPort);
+
 	// MIDI event capture method.
 	void capture(snd_seq_event_t *pEv);
 
@@ -262,6 +271,9 @@ public:
 	// Free overriden SysEx queued events.
 	void clearSysexCache();
 
+	// Reset ouput queue drift stats (audio vs. MIDI)...
+	void resetDrift();
+
 protected:
 
 	// Concrete device (de)activation methods.
@@ -271,9 +283,6 @@ protected:
 	void deactivate();
 	void clean();
 
-	// Reset ouput queue drift stats (audio vs. MIDI)...
-	void resetDrift();
-
 	// Metronome (de)activation methods.
 	void createMetroBus();
 	bool openMetroBus();
@@ -312,6 +321,10 @@ private:
 	qtractorMidiInputThread  *m_pInputThread;
 	qtractorMidiOutputThread *m_pOutputThread;
 
+	// ALSA port input registries.
+	QHash<int, qtractorMidiBus *> m_inputBuses;
+	QHash<int, qtractorMidiInputBuffer *> m_inputBuffers;
+
 	// Whether to check for time drift.
 	bool m_bDriftCorrect;
 
@@ -323,7 +336,6 @@ private:
 	long m_iTimeStart;
 	long m_iTimeDrift;
 	long m_iFrameStart;
-	long m_iFrameDrift;
 
 	// The absolute-time/frame when playback started.
 	unsigned long m_iTimeStartEx;
@@ -422,7 +434,7 @@ public:
 		Patch() : bankSelMethod(-1), bank(-1), prog(-1) {}
 		// Validate members.
 		bool isValid() const
-			{ return bankSelMethod >= 0 && (bank >= 0 || prog >= 0); }
+			{ return (bankSelMethod >= 0 || bank >= 0 || prog >= 0); }
 		// Payload members.
 		QString instrumentName;
 		int     bankSelMethod;
