@@ -1,7 +1,7 @@
 // qtractorMidiFile.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -472,7 +472,7 @@ bool qtractorMidiFile::readTracks ( qtractorMidiSequence **ppSeqs,
 					break;
 				}
 				if (meta == qtractorMidiEvent::TEMPO) {
-					m_pTempoMap->addNodeTempo(iTime,
+					m_pTempoMap->addNodeTempo(iTrackTime,
 						qtractorTimeScale::uroundf(
 							60000000.0f / float(readInt(len))));
 				} else {
@@ -485,19 +485,20 @@ bool qtractorMidiFile::readTracks ( qtractorMidiSequence **ppSeqs,
 					// Now, we'll deal only with some...
 					switch (meta) {
 					case qtractorMidiEvent::TRACKNAME:
-						pSeq->setName(QString((const char *) data).simplified());
+						pSeq->setName(
+							QString::fromLatin1((const char *) data).simplified());
 						break;
 					case qtractorMidiEvent::TIME:
 						// Beats per bar is the numerator of time signature...
 						if ((unsigned short) data[0] > 0) {
-							m_pTempoMap->addNodeTime(iTime,
+							m_pTempoMap->addNodeTime(iTrackTime,
 								(unsigned short) data[0],
 								(unsigned short) data[1]);
 						}
 						break;
 					case qtractorMidiEvent::MARKER:
-						m_pTempoMap->addMarker(iTime,
-							QString((const char *) data).simplified());
+						m_pTempoMap->addMarker(iTrackTime,
+							QString::fromLatin1((const char *) data).simplified());
 						break;
 					default:
 						// Ignore all others...
@@ -596,6 +597,38 @@ unsigned long qtractorMidiFile::readTrackDuration ( unsigned short iTrackChannel
 		const unsigned short iChannel = (iStatus & 0x0f);
 		if ((iChannelFilter & 0xf0) || (iChannelFilter == iChannel))
 			iTrackDuration = iTrackTime;
+
+		qtractorMidiEvent::EventType type
+			= qtractorMidiEvent::EventType(iStatus & 0xf0);
+		if (iStatus == qtractorMidiEvent::META)
+			type = qtractorMidiEvent::META;
+
+		switch (type) {
+		case qtractorMidiEvent::NOTEOFF:
+		case qtractorMidiEvent::NOTEON:
+		case qtractorMidiEvent::KEYPRESS:
+		case qtractorMidiEvent::CONTROLLER:
+		case qtractorMidiEvent::PITCHBEND:
+			readInt(2);
+			break;
+		case qtractorMidiEvent::PGMCHANGE:
+		case qtractorMidiEvent::CHANPRESS:
+			readInt(1);
+			break;
+		case qtractorMidiEvent::META:
+			readInt(1);
+			// Fall thru...
+		case qtractorMidiEvent::SYSEX:
+		{
+			int n = readInt();
+			if (n < 1 || ::fseek(m_pFile, m_iOffset + n, SEEK_SET))
+				m_iOffset = iTrackEnd; // Force EoT!
+			else
+				m_iOffset += n;
+		}	// Fall thru...
+		default:
+			break;
+		}
 	}
 
 	return iTrackDuration;
@@ -698,7 +731,7 @@ bool qtractorMidiFile::writeTracks (
 			writeInt(qtractorMidiEvent::META, 1);
 			writeInt(qtractorMidiEvent::TRACKNAME, 1);
 			writeInt(sTrackName.length());
-			const QByteArray aTrackName = sTrackName.toUtf8();
+			const QByteArray aTrackName = sTrackName.toLatin1();
 			writeData((unsigned char *) aTrackName.constData(), aTrackName.length());
 		}
 
@@ -877,17 +910,17 @@ bool qtractorMidiFile::writeTracks (
 					while (pNode && iTime >= pNode->tick) {
 						while (pMarker && pMarker->tick < pNode->tick) {
 							writeMarker(pMarker, iLastTime);
-							iLastTime = pMarker->tick;
+							iLastTime = pSeq->timep(pMarker->tick, m_iTicksPerBeat);
 							pMarker = pMarker->next();
 						}
 						writeNode(pNode, iLastTime);
-						iLastTime = pNode->tick;
+						iLastTime = pSeq->timep(pNode->tick, m_iTicksPerBeat);
 						iLastStatus = 0;
 						pNode = pNode->next();
 					}
 					while (pMarker && iTime >= pMarker->tick) {
 						writeMarker(pMarker, iLastTime);
-						iLastTime = pMarker->tick;
+						iLastTime = pSeq->timep(pMarker->tick, m_iTicksPerBeat);
 						iLastStatus = 0;
 						pMarker = pMarker->next();
 					}
@@ -1243,7 +1276,7 @@ void qtractorMidiFile::writeMarker (
 	writeInt(qtractorMidiEvent::META, 1);
 	writeInt(qtractorMidiEvent::MARKER, 1);
 	writeInt(pMarker->text.length());
-	const QByteArray aMarker = pMarker->text.toUtf8();
+	const QByteArray aMarker = pMarker->text.toLatin1();
 	writeData((unsigned char *) aMarker.constData(), aMarker.length());
 }
 
