@@ -1,7 +1,7 @@
 // qtractorMidiThumbView.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -68,10 +68,12 @@ qtractorMidiThumbView::qtractorMidiThumbView(
 
 	m_dragState   = DragNone;
 	m_pRubberBand = new qtractorRubberBand(QRubberBand::Rectangle, this, 2);
-//	QPalette pal(m_pRubberBand->palette());
-//	pal.setColor(m_pRubberBand->foregroundRole(), pal.highlight().color());
-//	m_pRubberBand->setPalette(pal);
-//	m_pRubberBand->setBackgroundRole(QPalette::NoRole);
+#if 0
+	QPalette pal(m_pRubberBand->palette());
+	pal.setColor(m_pRubberBand->foregroundRole(), pal.highlight().color());
+	m_pRubberBand->setPalette(pal);
+	m_pRubberBand->setBackgroundRole(QPalette::NoRole);
+#endif
 	m_pRubberBand->show();
 
 	QFrame::setToolTip(tr("MIDI Thumb view"));
@@ -389,13 +391,14 @@ void qtractorMidiThumbView::mousePressEvent ( QMouseEvent *pMouseEvent )
 
 	// Only expected behavior with left-button pressed...
 	if (pMouseEvent->button() == Qt::LeftButton) {
+		const QRect& rect = m_pRubberBand->geometry();
 		m_posDrag = pMouseEvent->pos();
-		if (m_pRubberBand->geometry().contains(m_posDrag)) {
+		QFrame::setCursor(QCursor(Qt::PointingHandCursor));
+		if (rect.contains(m_posDrag)) {
 			m_dragState = DragStart;
-			QFrame::setCursor(QCursor(Qt::SizeHorCursor));
 		} else {
 			m_dragState = DragClick;
-			QFrame::setCursor(QCursor(Qt::PointingHandCursor));
+			m_posDrag.setX(((rect.left() + rect.right()) >> 1));
 		}
 	}
 	else
@@ -415,12 +418,16 @@ void qtractorMidiThumbView::mouseMoveEvent ( QMouseEvent *pMouseEvent )
 	// Only expected behavior with left-button pressed...
 	if (pMouseEvent->buttons() & Qt::LeftButton) {
 		const QPoint& pos = pMouseEvent->pos();
-		if (m_dragState == DragStart
+		if ((m_dragState == DragStart || m_dragState == DragClick)
 			&& (pos - m_posDrag).manhattanLength()
-				> QApplication::startDragDistance())
+				> QApplication::startDragDistance()) {
 			m_dragState = DragMove;
-		if (m_dragState == DragMove)
-			updateThumb(pos.x() - m_posDrag.x());
+			QFrame::setCursor(QCursor(Qt::SizeHorCursor));
+		}
+		if (m_dragState == DragMove) {
+			updateView(pos.x() - m_posDrag.x());
+			m_posDrag.setX(pos.x());
+		}
 	}
 
 	QFrame::mouseMoveEvent(pMouseEvent);
@@ -437,10 +444,8 @@ void qtractorMidiThumbView::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 		if (m_dragState == DragMove)
 			updateView(pos.x() - m_posDrag.x());
 		else {
-			if (m_dragState == DragClick) {
-				const QRect& rect = m_pRubberBand->geometry();
-				updateView(pos.x() - ((rect.left() + rect.right()) >> 1));
-			}
+			if (m_dragState == DragClick)
+				updateView(pos.x() - m_posDrag.x());
 			// Make it change playhead?...
 			if (pMouseEvent->modifiers()
 				& (Qt::ShiftModifier | Qt::ControlModifier))
