@@ -1,7 +1,7 @@
 // qtractorOptions.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -209,6 +209,8 @@ void qtractorOptions::loadOptions (void)
 	iCurveMode      = m_settings.value("/CurveMode", 0).toInt();
 	iEditRangeOptions = m_settings.value("/EditRangeOptions", 3).toInt();
 	bMidButtonModifier = m_settings.value("/MidButtonModifier", false).toBool();
+	bMidiControlSync = m_settings.value("/MidiControlSync", false).toBool();
+	bExportAddTrack = m_settings.value("/ExportAddTrack", false).toBool();
 	m_settings.endGroup();
 
 	// Session auto-save group.
@@ -493,6 +495,8 @@ void qtractorOptions::saveOptions (void)
 	m_settings.setValue("/CurveMode", iCurveMode);
 	m_settings.setValue("/EditRangeOptions", iEditRangeOptions);
 	m_settings.setValue("/MidButtonModifier", bMidButtonModifier);
+	m_settings.setValue("/MidiControlSync", bMidiControlSync);
+	m_settings.setValue("/ExportAddTrack", bExportAddTrack);
 	m_settings.endGroup();
 
 	// Session auto-save group.
@@ -525,9 +529,9 @@ void qtractorOptions::saveOptions (void)
 	QStringListIterator iter1(instrumentFiles);
     while (iter1.hasNext())
 		m_settings.setValue(sFilePrefix.arg(++iFile), iter1.next());
-    // Cleanup old entries, if any...
-    while (!m_settings.value(sFilePrefix.arg(++iFile)).isNull())
-        m_settings.remove(sFilePrefix.arg(iFile));
+	// Cleanup old entries, if any...
+	while (!m_settings.value(sFilePrefix.arg(++iFile)).isNull())
+		m_settings.remove(sFilePrefix.arg(iFile));
 	m_settings.endGroup();
 
 	// MIDI controller file list.
@@ -906,19 +910,21 @@ void qtractorOptions::loadActionShortcuts ( QObject *pObject )
 {
 	m_settings.beginGroup("/Shortcuts/" + pObject->objectName());
 
-	const QStringList& keys = m_settings.childKeys();
-	QStringListIterator iter(keys);
+	if (m_settings.childKeys().isEmpty()) {
+		m_settings.endGroup();
+		return;
+	}
+
+	QList<QAction *> actions = pObject->findChildren<QAction *> ();
+	QListIterator<QAction *> iter(actions);
 	while (iter.hasNext()) {
-		const QString& sObjectName = iter.next();
-		const QList<QAction *>& actions
-			= pObject->findChildren<QAction *> (sObjectName);
-		if (actions.isEmpty())
+		QAction *pAction = iter.next();
+		if (pAction->objectName().isEmpty())
 			continue;
-		const QString& sKey = '/' + sObjectName;
+		const QString& sKey = '/' + pAction->objectName();
 		const QString& sValue = m_settings.value(sKey).toString();
-		if (sValue.isEmpty())
+		if (sValue.isEmpty() && pAction->shortcut().isEmpty())
 			continue;
-		QAction *pAction = actions.first();
 		pAction->setShortcut(QKeySequence(sValue));
 	}
 
