@@ -1,7 +1,7 @@
 // qtractorOptions.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -186,6 +186,12 @@ public:
 	// Mouse modifier options.
 	bool    bMidButtonModifier;
 
+	// MIDI control non catch-up/hook option.
+	bool    bMidiControlSync;
+
+	// Export add new track(s) option.
+	bool    bExportAddTrack;
+
 	// Session auto-save options.
 	bool    bAutoSaveEnabled;
 	int     iAutoSavePeriod;
