@@ -1,7 +1,7 @@
 // qtractorOptionsForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -28,7 +28,8 @@
 #include "qtractorMidiTimer.h"
 #include "qtractorMidiEditor.h"
 #include "qtractorTimeScale.h"
-#include "qtractorPlugin.h"
+
+#include "qtractorPluginFactory.h"
 
 #include "qtractorAudioMeter.h"
 #include "qtractorMidiMeter.h"
@@ -46,7 +47,7 @@
 #include <QStyleFactory>
 
 
-// Needed for fabs(), logf() and powf()
+// Needed for fabsf(), logf() and powf()
 #include <math.h>
 
 static inline float log10f2 ( float x )
@@ -829,8 +830,13 @@ void qtractorOptionsForm::accept (void)
 			m_pOptions->sCustomStyleTheme.clear();
 		// Reset dirty flags.
 		if (m_iDirtyPluginPaths > 0) {
-			qtractorPluginSelectForm::clearPluginPaths();
 			m_iDirtyPluginPaths = 0;
+			qtractorPluginFactory *pPluginFactory
+				= qtractorPluginFactory::getInstance();
+			if (pPluginFactory) {
+				pPluginFactory->updatePluginPaths();
+				pPluginFactory->clear();
+			}
 		}
 		m_iDirtyCount = 0;
 	}
@@ -1405,7 +1411,7 @@ void qtractorOptionsForm::chooseMessagesLogPath (void)
 		sFilename = fileDialog.selectedFiles().first();
 #endif
 
-	if (!sFilename.isEmpty()) {
+	if (!sFilename.isEmpty() && sFilename.at(0) != '.') {
 		m_ui.MessagesLogPathComboBox->setEditText(sFilename);
 		m_ui.MessagesLogPathComboBox->setFocus();
 		changed();
