@@ -1,7 +1,7 @@
 // qtractorPlugin.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -21,15 +21,17 @@
 
 #include "qtractorAbout.h"
 #include "qtractorPlugin.h"
+#include "qtractorPluginFactory.h"
 #include "qtractorPluginListView.h"
 #include "qtractorPluginCommand.h"
 #include "qtractorPluginForm.h"
 
 #include "qtractorAudioEngine.h"
-#include "qtractorMidiBuffer.h"
+#include "qtractorMidiManager.h"
+
+#include "qtractorMainForm.h"
 
 #include "qtractorOptions.h"
-#include "qtractorMainForm.h"
 
 #include "qtractorSession.h"
 #include "qtractorDocument.h"
@@ -37,21 +39,6 @@
 
 #include "qtractorMessageList.h"
 
-#ifdef CONFIG_LADSPA
-#include "qtractorLadspaPlugin.h"
-#endif
-#ifdef CONFIG_DSSI
-#include "qtractorDssiPlugin.h"
-#endif
-#ifdef CONFIG_VST
-#include "qtractorVstPlugin.h"
-#endif
-#ifdef CONFIG_LV2
-#include "qtractorLv2Plugin.h"
-#endif
-
-#include "qtractorInsertPlugin.h"
-
 #include <QTextStream>
 #include <QFileInfo>
 #include <QDir>
@@ -68,51 +55,6 @@ const WindowFlags WindowCloseButtonHint = WindowFlags(0x08000000);
 #endif
 
 
-// A common scheme for (a default) plugin serach paths...
-//
-#if defined(__WIN32__) || defined(_WIN32) || defined(WIN32)
-#define PATH_SEP ";"
-#else
-#define PATH_SEP ":"
-#endif
-
-static QString default_paths ( const QString& suffix )
-{
-	const QString& sep  = QDir::separator();
-
-	const QString& home = QDir::homePath();
-
-	const QString& pre1 = QDir::rootPath() + "usr";
-	const QString& pre2 = pre1 + sep + "local";
-
-	const QString& lib0 = "lib";
-	const QString& lib1 = pre1 + sep + lib0;
-	const QString& lib2 = pre2 + sep + lib0;
-
-#if defined(__x86_64__)
-	const QString& x64  = "64";
-	const QString& lib3 = lib1 + x64;
-	const QString& lib4 = lib2 + x64;
-#endif
-
-	QStringList paths;
-
-	paths << home + sep + '.' + suffix;
-
-#if defined(__x86_64__)
-//	paths << home + sep + lib0 + x64 + sep + suffix;
-	paths << lib4 + sep + suffix;
-	paths << lib3 + sep + suffix;
-#endif
-
-//	paths << home + sep + lib0 + sep + suffix;
-	paths << lib2 + sep + suffix;
-	paths << lib1 + sep + suffix;
-
-	return paths.join(PATH_SEP);
-}
-
-
 // A common function for special platforms...
 //
 #if defined(__WIN32__) || defined(_WIN32) || defined(WIN32)
@@ -120,149 +62,6 @@ typedef void (*qtractorPluginFile_Function)(void);
 #endif
 
 
-//----------------------------------------------------------------------------
-// qtractorPluginPath -- Plugin path helper.
-//
-
-// Executive methods.
-bool qtractorPluginPath::open (void)
-{
-	close();
-
-	// Get paths based on hints...	
-	QString sPaths;
-	QStringList paths;
-	QStringList path_list;
-#ifdef CONFIG_LADSPA
-	// LADSPA default path...
-	if (m_typeHint == qtractorPluginType::Any ||
-		m_typeHint == qtractorPluginType::Ladspa) {
-		paths = m_paths.value(qtractorPluginType::Ladspa);
-		if (paths.isEmpty()) {
-			sPaths = ::getenv("LADSPA_PATH");
-			if (sPaths.isEmpty())
-				sPaths = default_paths("ladspa");
-			if (!sPaths.isEmpty()) {
-				paths = sPaths.split(PATH_SEP);
-				m_paths.insert(qtractorPluginType::Ladspa, paths);
-			}
-		}
-		path_list.append(paths);
-	}
-#endif
-#ifdef CONFIG_DSSI
-	// DSSI default path...
-	if (m_typeHint == qtractorPluginType::Any ||
-		m_typeHint == qtractorPluginType::Dssi) {
-		paths = m_paths.value(qtractorPluginType::Dssi);
-		if (paths.isEmpty()) {
-			sPaths = ::getenv("DSSI_PATH");
-			if (sPaths.isEmpty())
-				sPaths = default_paths("dssi");
-			if (!sPaths.isEmpty()) {
-				paths = sPaths.split(PATH_SEP);
-				m_paths.insert(qtractorPluginType::Dssi, paths);
-			}
-		}
-		path_list.append(paths);
-	}
-#endif
-#ifdef CONFIG_VST
-	// VST default path...
-	if (m_typeHint == qtractorPluginType::Any ||
-		m_typeHint == qtractorPluginType::Vst) {
-		paths = m_paths.value(qtractorPluginType::Vst);
-		if (paths.isEmpty()) {
-			sPaths = ::getenv("VST_PATH");
-			if (sPaths.isEmpty())
-				sPaths = default_paths("vst");
-			if (!sPaths.isEmpty()) {
-				paths = sPaths.split(PATH_SEP);
-				m_paths.insert(qtractorPluginType::Vst, paths);
-			}
-		}
-		path_list.append(paths);
-	}
-#endif
-#ifdef CONFIG_LV2
-	// LV2 default path...
-	if (m_typeHint == qtractorPluginType::Any ||
-		m_typeHint == qtractorPluginType::Lv2) {
-		paths = m_paths.value(qtractorPluginType::Lv2);
-		if (paths.isEmpty()) {
-			sPaths = ::getenv("LV2_PATH");
-			if (sPaths.isEmpty())
-				sPaths = default_paths("lv2");
-			if (!sPaths.isEmpty()) {
-				paths = sPaths.split(PATH_SEP);
-				m_paths.insert(qtractorPluginType::Lv2, paths);
-			}
-		}
-		path_list.append(paths);
-	#ifdef CONFIG_LV2_PRESETS
-		QString sPresetDir;
-		qtractorOptions *pOptions = qtractorOptions::getInstance();
-		if (pOptions)
-			sPresetDir = pOptions->sLv2PresetDir;
-		if (sPresetDir.isEmpty())
-			sPresetDir = QDir::homePath() + QDir::separator() + ".lv2";
-		if (!paths.contains(sPresetDir))
-			paths.append(sPresetDir);
-	#endif
-		// HACK: set special environment for LV2...
-		::setenv("LV2_PATH", paths.join(PATH_SEP).toUtf8().constData(), 1);
-	}
-#endif
-
-#ifdef CONFIG_DEBUG
-	qDebug("qtractorPluginPath[%p]::open() paths=\"%s\" typeHint=%d",
-		this, path_list.join(PATH_SEP).toUtf8().constData(), int(m_typeHint));
-#endif
-
-	QStringListIterator path_iter(path_list);
-	while (path_iter.hasNext())
-		addFiles(path_iter.next());
-
-	return (m_files.count() > 0);
-}
-
-
-void qtractorPluginPath::close (void)
-{
-	qDeleteAll(m_files);
-	m_files.clear();
-}
-
-
-// Helper methods.
-void qtractorPluginPath::setPaths (
-	qtractorPluginType::Hint typeHint, const QString& sPaths )
-{
-	m_paths.insert(typeHint, sPaths.split(PATH_SEP));
-}
-
-
-// Recursive plugin file/path inventory method.
-void qtractorPluginPath::addFiles ( const QString& sPath )
-{
-	const QDir dir(sPath);
-	QDir::Filters filters = QDir::Files;
-	if (m_typeHint == qtractorPluginType::Vst)
-		filters = filters | QDir::AllDirs | QDir::NoDotAndDotDot;
-	const QFileInfoList& info_list = dir.entryInfoList(filters);
-	QListIterator<QFileInfo> info_iter(info_list);
-	while (info_iter.hasNext()) {
-		const QFileInfo& info = info_iter.next();
-		const QString& sFilename = info.absoluteFilePath();
-		if (info.isDir() && info.isReadable())
-			addFiles(sFilename);
-		else
-		if (QLibrary::isLibrary(sFilename))
-			m_files.append(new qtractorPluginFile(sFilename));
-	}
-}
-
-
 //----------------------------------------------------------------------------
 // qtractorPluginFile -- Plugin file library instance.
 //
@@ -270,12 +69,16 @@ void qtractorPluginPath::addFiles ( const QString& sPath )
 // Executive methods.
 bool qtractorPluginFile::open (void)
 {
-	close();
+	// Check whether already open...
+	if (++m_iOpenCount > 1)
+		return true;
 
 	// ATTN: Not really needed, as it would be
 	// loaded automagically on resolve(), but ntl...
-	if (!QLibrary::load())
+	if (!QLibrary::load()) {
+		m_iOpenCount = 0;
 		return false;
+	}
 
 	// Do the openning dance...
 #if defined(__WIN32__) || defined(_WIN32) || defined(WIN32)
@@ -285,6 +88,7 @@ bool qtractorPluginFile::open (void)
 		(*pfnInit)();
 #endif
 
+	// Done alright.
 	return true;
 }
 
@@ -294,6 +98,9 @@ void qtractorPluginFile::close (void)
 	if (!QLibrary::isLoaded())
 		return;
 
+	if (--m_iOpenCount > 0)
+		return;
+
 	// Do the closing dance...
 #if defined(__WIN32__) || defined(_WIN32) || defined(WIN32)
 	qtractorPluginFile_Function pfnFini
@@ -310,192 +117,34 @@ void qtractorPluginFile::close (void)
 }
 
 
-// Plugin type listing.
-bool qtractorPluginFile::getTypes ( qtractorPluginPath& path,
-	qtractorPluginType::Hint typeHint )
+// Plugin file resgistry methods.
+qtractorPluginFile::Files qtractorPluginFile::g_files;
+
+qtractorPluginFile *qtractorPluginFile::addFile ( const QString& sFilename )
 {
-	// Try to fill the types list at this moment...
-	qtractorPluginType *pType;
-	unsigned long iIndex = 0;
+	qtractorPluginFile *pFile = g_files.value(sFilename, NULL);
 
-#ifdef CONFIG_DSSI
-	// Try DSSI plugin types first...
-	if (typeHint == qtractorPluginType::Any ||
-		typeHint == qtractorPluginType::Dssi) {
-		while (true) {
-			pType = qtractorDssiPluginType::createType(this, iIndex);
-			if (pType == NULL)
-				break;
-			if (pType->open()) {
-				path.addType(pType);
-				pType->close();
-				++iIndex;
-			} else {
-				delete pType;
-				break;
-			}
-		}
+	if (pFile == NULL && QLibrary::isLibrary(sFilename)) {
+		pFile = new qtractorPluginFile(sFilename);
+		g_files.insert(pFile->filename(), pFile);
 	}
-	// Have we found some, already?
-	if (iIndex > 0)
-		return true;
-#endif
 
-#ifdef CONFIG_LADSPA
-	// Try LADSPA plugin types...
-	if (typeHint == qtractorPluginType::Any ||
-		typeHint == qtractorPluginType::Ladspa) {
-		while (true) {
-			pType = qtractorLadspaPluginType::createType(this, iIndex);
-			if (pType == NULL)
-				break;
-			if (pType->open()) {
-				path.addType(pType);
-				pType->close();
-				++iIndex;
-			} else {
-				delete pType;
-				break;
-			}
-		}
-	}
-	// Have we found some, already?
-	if (iIndex > 0)
-		return true;
-#endif
+	if (pFile && !pFile->open())
+		pFile = NULL;
 
-#ifdef CONFIG_VST
-	// Try VST plugin types...
-	if (typeHint == qtractorPluginType::Any ||
-		typeHint == qtractorPluginType::Vst) {
-		// Need to look at the options...
-		qtractorOptions *pOptions = qtractorOptions::getInstance();
-		while (true) {
-			if (pOptions && pOptions->bDummyVstScan)
-				pType = qtractorDummyPluginType::createType(this, iIndex);
-			else
-				pType = qtractorVstPluginType::createType(this, iIndex);
-			if (pType == NULL)
-				break;
-			if (pType->open()) {
-				path.addType(pType);
-				pType->close();
-				++iIndex;
-			} else {
-				delete pType;
-				break;
-			}
-		}
-	}
-#endif
+	if (pFile)
+		pFile->addRef();
 
-	// Have we something?
-	return (iIndex > 0);
+	return pFile;
 }
 
 
-// Plugin factory method (static).
-qtractorPlugin *qtractorPluginFile::createPlugin (
-	qtractorPluginList *pList,
-	const QString& sFilename, unsigned long iIndex,
-	qtractorPluginType::Hint typeHint )
+void qtractorPluginFile::removeFile ( qtractorPluginFile *pFile )
 {
-#ifdef CONFIG_DEBUG
-	qDebug("qtractorPluginFile::createPlugin(%p, \"%s\", %lu, %d)",
-		pList, sFilename.toUtf8().constData(), iIndex, int(typeHint));
-#endif
-
-	// Attend to insert pseudo-plugin hints...
-	if (sFilename.isEmpty()) {
-		if (typeHint == qtractorPluginType::Insert) {
-			qtractorInsertPluginType *pInsertType
-				= qtractorInsertPluginType::createType(iIndex);
-			if (pInsertType) {
-				if (pInsertType->open())
-					return new qtractorInsertPlugin(pList, pInsertType);
-				delete pInsertType;
-			}
-		}
-		else
-		if (typeHint == qtractorPluginType::AuxSend) {
-			qtractorAuxSendPluginType *pAuxSendType
-				= qtractorAuxSendPluginType::createType(iIndex);
-			if (pAuxSendType) {
-				if (pAuxSendType->open())
-					return new qtractorAuxSendPlugin(pList, pAuxSendType);
-				delete pAuxSendType;
-			}
-		}
-		// Don't bother with anything else.
-		return NULL;
-	}
-
-#ifdef CONFIG_LV2
-	// Try LV2 plugins hints before anything else...
-	if (typeHint == qtractorPluginType::Any ||
-		typeHint == qtractorPluginType::Lv2) {
-		qtractorLv2PluginType *pLv2Type
-			= qtractorLv2PluginType::createType(sFilename);
-		if (pLv2Type) {
-			if (pLv2Type->open())
-				return new qtractorLv2Plugin(pList, pLv2Type);
-			delete pLv2Type;
-		}
-	}
-#endif
-
-	// Try to fill the types list at this moment...
-	qtractorPluginFile *pFile = new qtractorPluginFile(sFilename);
-	if (!pFile->open()) {
+	if (pFile && pFile->removeRef()) {
+		g_files.remove(pFile->filename());
 		delete pFile;
-		return NULL;
 	}
-
-#ifdef CONFIG_DSSI
-	// Try DSSI plugin types first...
-	if (typeHint == qtractorPluginType::Any ||
-		typeHint == qtractorPluginType::Dssi) {
-		qtractorDssiPluginType *pDssiType
-			= qtractorDssiPluginType::createType(pFile, iIndex);
-		if (pDssiType) {
-			if (pDssiType->open())
-				return new qtractorDssiPlugin(pList, pDssiType);
-			delete pDssiType;
-		}
-	}
-#endif
-
-#ifdef CONFIG_LADSPA
-	// Try LADSPA plugin types...
-	if (typeHint == qtractorPluginType::Any ||
-		typeHint == qtractorPluginType::Ladspa) {
-		qtractorLadspaPluginType *pLadspaType
-			= qtractorLadspaPluginType::createType(pFile, iIndex);
-		if (pLadspaType) {
-			if (pLadspaType->open())
-				return new qtractorLadspaPlugin(pList, pLadspaType);
-			delete pLadspaType;
-		}
-	}
-#endif
-
-#ifdef CONFIG_VST
-	// Try VST plugin types...
-	if (typeHint == qtractorPluginType::Any ||
-		typeHint == qtractorPluginType::Vst) {
-		qtractorVstPluginType *pVstType
-			= qtractorVstPluginType::createType(pFile, iIndex);
-		if (pVstType) {
-			if (pVstType->open())
-				return new qtractorVstPlugin(pList, pVstType);
-			delete pVstType;
-		}
-	}
-#endif
-
-	// Bad luck, no valid plugin found...
-	delete pFile;
-	return NULL;
 }
 
 
@@ -517,26 +166,14 @@ unsigned short qtractorPluginType::instances (
 {
 	unsigned short iInstances = 0;
 	if (iChannels > 0) {
-		if (bMidi && m_iMidiOuts > 0)
+		if (bMidi && (m_iMidiIns > 0 || m_iMidiOuts > 0))
 			iInstances = 1;
 		else
-	#if 0
-		if (iChannels >= m_iAudioIns && m_iAudioOuts > 0)
-			iInstances = (m_iAudioOuts >= iChannels ? 1 : iChannels);
-		else
-		if (m_iAudioOuts >= iChannels)
-			iInstances = (iChannels >= m_iAudioIns ? iChannels : 1);
-		else
-	#else
 		if (m_iAudioOuts >= iChannels || m_iAudioIns >= iChannels)
 			iInstances = 1;
 		else
 		if (m_iAudioOuts > 0)
 			iInstances = (iChannels / m_iAudioOuts);
-		else
-	#endif
-		if (bMidi && m_iMidiIns > 0)
-			iInstances = 1;
 	}
 	return iInstances;
 }
@@ -592,69 +229,6 @@ QString qtractorPluginType::textFromHint (
 }
 
 
-//----------------------------------------------------------------------------
-// qtractorDummyPluginType -- Dummy plugin type instance.
-//
-
-// Constructor.
-qtractorDummyPluginType::qtractorDummyPluginType (
-	qtractorPluginFile *pFile, unsigned long iIndex, Hint typeHint)
-	: qtractorPluginType(pFile, iIndex, typeHint)
-{
-}
-
-// Must be overriden methods.
-bool qtractorDummyPluginType::open (void)
-{
-	m_sName  = QFileInfo(filename()).baseName();
-	m_sLabel = m_sName.simplified().replace(QRegExp("[\\s|\\.|\\-]+"), "_");
-
-	m_iUniqueID = qHash(m_sLabel);
-
-	// Fake the rest...
-	m_iAudioIns  = 2;
-	m_iAudioOuts = 2;
-	m_iMidiIns   = 1;
-
-	return true;
-}
-
-
-void qtractorDummyPluginType::close (void)
-{
-}
-
-
-// Factory method (static)
-qtractorDummyPluginType *qtractorDummyPluginType::createType (
-	qtractorPluginFile *pFile, unsigned long iIndex, Hint typeHint )
-{
-	// Sanity check...
-	if (pFile == NULL || iIndex > 0)
-		return NULL;
-
-	// Yep, most probably its a dummy plugin effect...
-	return new qtractorDummyPluginType(pFile, iIndex, typeHint);
-}
-
-
-// Instance cached-deferred accesors.
-const QString& qtractorDummyPluginType::aboutText (void)
-{
-	if (m_sAboutText.isEmpty()) {
-		m_sAboutText  = QTRACTOR_TITLE;
-		m_sAboutText += ' ';
-		m_sAboutText += QObject::tr("Dummy plugin type.");
-		m_sAboutText += '\n';
-		m_sAboutText += QTRACTOR_WEBSITE;
-		m_sAboutText += '\n';
-		m_sAboutText += QTRACTOR_COPYRIGHT;
-	}
-
-	return m_sAboutText;
-}
-
-
 //----------------------------------------------------------------------------
 // qtractorPlugin -- Plugin instance.
 //
@@ -692,26 +266,11 @@ qtractorPlugin::~qtractorPlugin (void)
 	m_params.clear();
 
 	// Rest of stuff goes cleaned too...
-	if (m_pType) {
-		qtractorPluginFile *pFile = m_pType->file();
-		delete m_pType;
-		if (pFile)
-			delete pFile;
-	}
+	if (m_pType) delete m_pType;
 }
 
 
 // Chain helper ones.
-unsigned int qtractorPlugin::sampleRate (void) const
-{
-	return (m_pList ? m_pList->sampleRate() : 0);
-}
-
-unsigned int qtractorPlugin::bufferSize (void) const
-{
-	return (m_pList ? m_pList->bufferSize() : 0);
-}
-
 unsigned short qtractorPlugin::channels (void) const
 {
 	return (m_pList ? m_pList->channels() : 0);
@@ -769,10 +328,25 @@ void qtractorPlugin::updateActivated ( bool bActivated )
 	m_bActivated = bActivated;
 }
 
+
 void qtractorPlugin::updateActivatedEx ( bool bActivated )
 {
 	updateActivated(bActivated);
 
+	// Get extra visual feedback as well,
+	// iif. we're not exporting/freewheeling...
+	//
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return;
+
+	qtractorAudioEngine *pAudioEngine = pSession->audioEngine();
+	if (pAudioEngine == NULL)
+		return;
+
+	if (pAudioEngine->isFreewheel())
+		return;
+
 	QListIterator<qtractorPluginListItem *> iter(m_items);
 	while (iter.hasNext())
 		iter.next()->updateActivated();
@@ -890,18 +464,16 @@ void qtractorPlugin::openForm ( QWidget *pParent )
 
 	if (bCreate) {
 		// Build up the plugin form...
-		if (pParent == NULL)
-			pParent = qtractorMainForm::getInstance();
-	// What style do we create tool childs?
-		Qt::WindowFlags wflags = Qt::Window
-			| Qt::CustomizeWindowHint
-			| Qt::WindowTitleHint
-			| Qt::WindowSystemMenuHint
-			| Qt::WindowMinMaxButtonsHint
-			| Qt::WindowCloseButtonHint;
+		// What style do we create tool childs?
+		Qt::WindowFlags wflags = Qt::Window;
 		qtractorOptions *pOptions = qtractorOptions::getInstance();
 		if (pOptions && pOptions->bKeepToolsOnTop) {
 			wflags |= Qt::Tool;
+		#if 0//QTRACTOR_PLUGIN_FORM_TOOL_PARENT
+			// Make sure it has a parent...
+			if (pParent == NULL)
+				pParent = qtractorMainForm::getInstance();
+		#endif
 		}
 		// Do it...
 		m_pForm = new qtractorPluginForm(pParent, wflags);
@@ -942,17 +514,17 @@ void qtractorPlugin::toggleFormEditor ( bool bOn )
 }
 
 
-void qtractorPlugin::updateFormParamValue ( unsigned long iIndex )
+void qtractorPlugin::updateFormDirtyCount (void)
 {
 	if (m_pForm && m_pForm->isVisible())
-		m_pForm->updateParamValue(iIndex);
+		m_pForm->updateDirtyCount();
 }
 
 
-void qtractorPlugin::updateFormAudioBusName (void)
+void qtractorPlugin::updateFormAuxSendBusName (void)
 {
 	if (m_pForm && m_pForm->isVisible())
-		m_pForm->updateAudioBusName();
+		m_pForm->updateAuxSendBusName();
 }
 
 
@@ -1150,6 +722,10 @@ bool qtractorPlugin::loadPresetEx ( const QString& sPreset )
 	if (pSession == NULL)
 		return false;
 
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+	if (pMainForm == NULL)
+		return false;
+
 	if (sPreset.isEmpty() || sPreset == g_sDefPreset) {
 		// Reset to default...
 		return pSession->execute(new qtractorResetPluginCommand(this));
@@ -1158,19 +734,26 @@ bool qtractorPlugin::loadPresetEx ( const QString& sPreset )
 	bool bResult = loadPreset(sPreset);
 	if (bResult) {
 		setPreset(sPreset);
+		pMainForm->dirtyNotifySlot();
+		refreshForm();
 	} else {
 		// An existing preset is about to be loaded...
 		QSettings& settings = pOptions->settings();
 		// Should it be load from known file?...
 		if (type()->isConfigure()) {
 			settings.beginGroup(presetGroup());
-			bResult = loadPresetFile(settings.value(sPreset).toString());
+			bResult = loadPresetFileEx(settings.value(sPreset).toString());
 			settings.endGroup();
-			if (bResult) setPreset(sPreset);
+			if (bResult) {
+				setPreset(sPreset);
+				pMainForm->dirtyNotifySlot();
+				refreshForm();
+			}
 		} else {
 			//...or make it as usual (parameter list only)...
 			settings.beginGroup(presetGroup());
-			QStringList vlist = settings.value(sPreset).toStringList();
+			const QStringList& vlist
+				= settings.value(sPreset).toStringList();
 			settings.endGroup();
 			if (!vlist.isEmpty()) {
 				bResult = pSession->execute(
@@ -1183,6 +766,19 @@ bool qtractorPlugin::loadPresetEx ( const QString& sPreset )
 }
 
 
+// Load an existing preset from file.
+bool qtractorPlugin::loadPresetFileEx ( const QString& sFilename )
+{
+	const bool bActivated = isActivated();
+	setActivated(false);
+
+	const bool bResult = loadPresetFile(sFilename);
+
+	setActivated(bActivated);
+	return bResult;
+}
+
+
 // Plugin parameter lookup.
 qtractorPluginParam *qtractorPlugin::findParam ( unsigned long iIndex ) const
 {
@@ -1306,8 +902,10 @@ void qtractorPlugin::realizeValues (void)
 		qtractorPluginParam *pParam = findParam(iIndex);
 		if (pParam) {
 			const QString& sName = m_values.names.value(iIndex);
-			if (!sName.isEmpty() && sName != pParam->name())
-				pParam = findParamName(sName);
+			if (!sName.isEmpty() && sName != pParam->name()) {
+				qtractorPluginParam *pParamEx = findParamName(sName);
+				if (pParamEx) pParam = pParamEx;
+			}
 			if (pParam)
 				pParam->setValue(param.value(), true);
 		}
@@ -1636,16 +1234,16 @@ void qtractorPluginParam::setValue ( float fValue, bool bUpdate )
 		if (!isInteger()) {
 			const float fDecs
 				= ::log10f(maxValue() - minValue());
-			if (fDecs < -3.0f)
+			if (fDecs < 0.0f)
 				m_iDecimals = 6;
-			else if (fDecs < 0.0f)
+			else if (fDecs < 3.0f)
 				m_iDecimals = 3;
-			else if (fDecs < 1.0f)
-				m_iDecimals = 2;
 			else if (fDecs < 6.0f)
 				m_iDecimals = 1;
+		#if 0
 			if (isLogarithmic())
 				++m_iDecimals;
+		#endif
 		}
 		// Make this permanent...
 		m_subject.setToggled(isToggled());
@@ -1709,11 +1307,12 @@ void qtractorPluginParam::Observer::update ( bool bUpdate )
 //
 
 // Constructor.
-qtractorPluginList::qtractorPluginList ( unsigned short iChannels,
-	unsigned int iBufferSize, unsigned int iSampleRate, unsigned int iFlags )
-	: m_iChannels(0), m_iBufferSize(0), m_iSampleRate(0),
-		m_iFlags(0), m_iActivated(0), m_pMidiManager(NULL),
-		m_iMidiBank(-1), m_iMidiProg(-1), m_pMidiProgramSubject(NULL)
+qtractorPluginList::qtractorPluginList (
+	unsigned short iChannels, unsigned int iFlags )
+	: m_iChannels(iChannels), m_iFlags(iFlags),
+		m_iActivated(0), m_pMidiManager(NULL),
+		m_iMidiBank(-1), m_iMidiProg(-1),
+		m_pMidiProgramSubject(NULL)
 {
 	setAutoDelete(true);
 
@@ -1727,14 +1326,16 @@ qtractorPluginList::qtractorPluginList ( unsigned short iChannels,
 	m_bAudioOutputAutoConnect
 		= qtractorMidiManager::isDefaultAudioOutputAutoConnect();
 
-	setBuffer(iChannels, iBufferSize, iSampleRate, iFlags);
+	m_iAudioInsertActivated = 0;
+
+	setChannels(iChannels, iFlags);
 }
 
 // Destructor.
 qtractorPluginList::~qtractorPluginList (void)
 {
 	// Reset allocated channel buffers.
-	setBuffer(0, 0, 0, 0);
+	setChannels(0, 0);
 
 	// Clear out all dependables...
 	m_views.clear();
@@ -1759,19 +1360,9 @@ void qtractorPluginList::setName ( const QString& sName )
 
 
 // Main-parameters accessor.
-void qtractorPluginList::setBuffer ( unsigned short iChannels,
-	unsigned int iBufferSize, unsigned int iSampleRate, unsigned int iFlags )
+void qtractorPluginList::setChannels (
+	unsigned short iChannels, unsigned int iFlags )
 {
-	unsigned short i;
-
-	// Delete old interim buffer...
-	if (m_pppBuffers[1]) {
-		for (i = 0; i < m_iChannels; ++i)
-			delete [] m_pppBuffers[1][i];
-		delete [] m_pppBuffers[1];
-		m_pppBuffers[1] = NULL;
-	}
-
 	// Destroy any MIDI manager still there...
 	if (m_pMidiManager) {
 		m_bAudioOutputBus = m_pMidiManager->isAudioOutputBus();
@@ -1786,49 +1377,29 @@ void qtractorPluginList::setBuffer ( unsigned short iChannels,
 		m_pMidiProgramSubject = NULL;
 	}
 
-	// Set proper sample-rate and flags at once.
-	m_iSampleRate = iSampleRate;
+	// Go, go, go...
 	m_iFlags = iFlags;
 
-	// Some sanity is in order, at least for now...
-	if (iChannels == 0 || iBufferSize == 0)
+	if (iChannels == 0)
 		return;
 
-	// Go, go, go...
-	m_iChannels   = iChannels;
-	m_iBufferSize = iBufferSize;
-
-	// Allocate new interim buffer...
-	if (m_iChannels > 0 && m_iBufferSize > 0) {
-		m_pppBuffers[1] = new float * [m_iChannels];
-		for (i = 0; i < m_iChannels; ++i) {
-			m_pppBuffers[1][i] = new float [m_iBufferSize];
-			::memset(m_pppBuffers[1][i], 0, m_iBufferSize * sizeof(float));
-		}
-	}
-
 	// Allocate new MIDI manager, if applicable...
 	if (m_iFlags & Midi) {
 		m_pMidiProgramSubject = new MidiProgramSubject(m_iMidiBank, m_iMidiProg);
 		m_pMidiManager = qtractorMidiManager::createMidiManager(this);
-		// Set loaded/cached properties properly...
-		m_pMidiManager->setCurrentBank(m_iMidiBank);
-		m_pMidiManager->setCurrentProg(m_iMidiProg);
-		m_pMidiManager->setAudioOutputBusName(m_sAudioOutputBusName);
-		m_pMidiManager->setAudioOutputAutoConnect(m_bAudioOutputAutoConnect);
-		m_pMidiManager->setAudioOutputBus(m_bAudioOutputBus);
-		if (m_pMidiManager->isAudioOutputBus()) {
-			qtractorAudioBus *pAudioOutputBus
-				= m_pMidiManager->audioOutputBus();
-			if (pAudioOutputBus)
+		qtractorAudioBus *pAudioOutputBus
+			= m_pMidiManager->audioOutputBus();
+		if (pAudioOutputBus) {
+			// Override number of channels from audio output bus...
+			iChannels = pAudioOutputBus->channels();
+			// Restore it's connections if dedicated...
+			if (m_pMidiManager->isAudioOutputBus())
 				pAudioOutputBus->outputs().copy(m_audioOutputs);
 		}
 	}
 
-	// Reset all plugin chain channels...
-	for (qtractorPlugin *pPlugin = first();
-			pPlugin; pPlugin = pPlugin->next())
-		pPlugin->setChannels(m_iChannels);
+	// Allocate all new interim buffers...
+	setChannelsEx(iChannels, false);
 
 	// FIXME: This should be better managed...
 	if (m_pMidiManager)
@@ -1836,9 +1407,76 @@ void qtractorPluginList::setBuffer ( unsigned short iChannels,
 }
 
 
-// Reset and (re)activate all plugin chain.
-void qtractorPluginList::resetBuffer (void)
+void qtractorPluginList::setChannelsEx (
+	unsigned short iChannels, bool bReset )
 {
+	// Maybe we don't need to change a thing here...
+	if (iChannels == m_iChannels)
+		return;
+
+	unsigned short i;
+
+	// Delete old interim buffer...
+	if (m_pppBuffers[1]) {
+		for (i = 0; i < m_iChannels; ++i)
+			delete [] m_pppBuffers[1][i];
+		delete [] m_pppBuffers[1];
+		m_pppBuffers[1] = NULL;
+	}
+
+	// Go, go, go...
+	m_iChannels = iChannels;
+
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return;
+
+	qtractorAudioEngine *pAudioEngine = pSession->audioEngine();
+	if (pAudioEngine == NULL)
+		return;
+
+	const unsigned int iBufferSize = pAudioEngine->bufferSize();
+
+	// Allocate new interim buffer...
+	if (m_iChannels > 0) {
+		m_pppBuffers[1] = new float * [m_iChannels];
+		for (i = 0; i < m_iChannels; ++i) {
+			m_pppBuffers[1][i] = new float [iBufferSize];
+			::memset(m_pppBuffers[1][i], 0, iBufferSize * sizeof(float));
+		}
+	}
+
+	// Reset all plugin chain channels...
+	for (qtractorPlugin *pPlugin = first();
+			pPlugin; pPlugin = pPlugin->next()) {
+		if (bReset && m_iChannels > 0) {
+			pPlugin->freezeConfigs();
+			pPlugin->freezeValues();
+		}
+		pPlugin->setChannels(m_iChannels);
+		if (bReset && m_iChannels > 0) {
+			pPlugin->realizeConfigs();
+			pPlugin->realizeValues();
+			pPlugin->releaseConfigs();
+			pPlugin->releaseValues();
+		}
+	}
+}
+
+
+// Reset and (re)activate all plugin chain.
+void qtractorPluginList::resetBuffers (void)
+{
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return;
+
+	qtractorAudioEngine *pAudioEngine = pSession->audioEngine();
+	if (pAudioEngine == NULL)
+		return;
+
+	const unsigned int iBufferSize = pAudioEngine->bufferSize();
+
 #if 0
 	// Save and reset activation count...
 	int iActivated = m_iActivated;
@@ -1854,7 +1492,7 @@ void qtractorPluginList::resetBuffer (void)
 	// Reset interim buffer, if any...
 	if (m_pppBuffers[1]) {
 		for (unsigned short i = 0; i < m_iChannels; ++i)
-			::memset(m_pppBuffers[1][i], 0, m_iBufferSize * sizeof(float));
+			::memset(m_pppBuffers[1][i], 0, iBufferSize * sizeof(float));
 	}
 #if 0
 	// Restore activation of all previously deactivated plugins...
@@ -1870,23 +1508,6 @@ void qtractorPluginList::resetBuffer (void)
 }
 
 
-// Special guard activation methods.
-bool qtractorPluginList::isActivatedAll (void) const
-{
-	return (m_iActivated > 0 && m_iActivated >= (unsigned int) count());
-}
-
-
-void qtractorPluginList::updateActivated ( bool bActivated )
-{
-	if (bActivated) {
-		++m_iActivated;
-	} else  {
-		--m_iActivated;
-	}
-}
-
-
 // Add-guarded plugin method.
 void qtractorPluginList::addPlugin ( qtractorPlugin *pPlugin )
 {
@@ -1896,8 +1517,8 @@ void qtractorPluginList::addPlugin ( qtractorPlugin *pPlugin )
 
 
 // Insert-guarded plugin method.
-void qtractorPluginList::insertPlugin ( qtractorPlugin *pPlugin,
-	qtractorPlugin *pNextPlugin )
+void qtractorPluginList::insertPlugin (
+	qtractorPlugin *pPlugin, qtractorPlugin *pNextPlugin )
 {
 	// We'll get prepared before plugging it in...
 	pPlugin->setChannels(m_iChannels);
@@ -2030,8 +1651,8 @@ qtractorPlugin *qtractorPluginList::copyPlugin ( qtractorPlugin *pPlugin )
 #endif
 
 	// Filename is empty for insert pseudo-plugins.
-	QString sFilename = pType->filename();
-	qtractorPlugin *pNewPlugin = qtractorPluginFile::createPlugin(this,
+	const QString& sFilename = pType->filename();
+	qtractorPlugin *pNewPlugin = qtractorPluginFactory::createPlugin(this,
 		sFilename, pType->index(), pType->typeHint());
 	if (pNewPlugin) {
 		pNewPlugin->setPreset(pPlugin->preset());
@@ -2073,9 +1694,7 @@ void qtractorPluginList::removeView ( qtractorPluginListView *pView )
 void qtractorPluginList::process ( float **ppBuffer, unsigned int nframes )
 {
 	// Sanity checks...
-	if (ppBuffer == NULL || *ppBuffer == NULL)
-		return;
-	if (nframes > m_iBufferSize || m_pppBuffers[1] == NULL)
+	if (ppBuffer == NULL || *ppBuffer == NULL || m_pppBuffers[1] == NULL)
 		return;
 
 	// Start from first input buffer...
@@ -2222,14 +1841,18 @@ bool qtractorPluginList::loadElement (
 					posForm.setY(sxy.at(1).toInt());
 				}
 			}
-			qtractorPlugin *pPlugin
-				= qtractorPluginFile::createPlugin(this,
+			// Try to find some alternative, if it doesn't exist...
+			qtractorPlugin *pPlugin = NULL;
+			if (checkPluginFile(sFilename, typeHint)) {
+				pPlugin = qtractorPluginFactory::createPlugin(this,
 					sFilename, iIndex, typeHint);
+			}
 		#if 0
 			if (!sFilename.isEmpty() && !sLabel.isEmpty() &&
 				((pPlugin == NULL) || ((pPlugin->type())->label() != sLabel))) {
 				iIndex = 0;
 				do {
+					if (pPlugin) delete pPlugin;
 					pPlugin = qtractorPluginFile::createPlugin(this,
 						sFilename, iIndex++, typeHint);
 				} while (pPlugin && (pPlugin->type())->label() != sLabel);
@@ -2431,4 +2054,44 @@ bool qtractorPluginList::isUniqueID ( qtractorPluginType *pType ) const
 }
 
 
+// Check/sanitize plugin file-path;
+bool qtractorPluginList::checkPluginFile (
+	QString& sFilename, qtractorPluginType::Hint typeHint ) const
+{
+	// Care of internal pseudo-plugins...
+	if (sFilename.isEmpty()) {
+		return (typeHint == qtractorPluginType::Insert)
+			|| (typeHint == qtractorPluginType::AuxSend);
+	}
+
+	// LV2 plug-ins are identified by URI...
+	if (typeHint == qtractorPluginType::Lv2)
+		return true;
+
+	// Primary check for plugin pathname...
+	QFileInfo fi(sFilename);
+	if (fi.exists() && fi.isReadable())
+		return true;
+
+	// Otherwise search for an alternative
+	// under each respective search paths...
+	qtractorPluginFactory *pPluginFactory
+		= qtractorPluginFactory::getInstance();
+	if (pPluginFactory) {
+		const QString fname = fi.fileName();
+		QStringListIterator iter(pPluginFactory->pluginPaths(typeHint));
+		while (iter.hasNext()) {
+			fi.setFile(QDir(iter.next()), fname);
+			if (fi.exists() && fi.isReadable()) {
+				sFilename = fi.absoluteFilePath();
+				return true;
+			}
+		}
+	}
+
+	// No alternative has been found, sorry.
+	return false;
+}
+
+
 // end of qtractorPlugin.cpp
