@@ -1,7 +1,7 @@
 // qtractorPluginCommand.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -28,7 +28,7 @@
 #include "qtractorInsertPlugin.h"
 
 #include "qtractorSession.h"
-#include "qtractorMidiBuffer.h"
+#include "qtractorMidiManager.h"
 
 #include "qtractorMainForm.h"
 #include "qtractorTracks.h"
@@ -188,9 +188,9 @@ bool qtractorAddAuxSendPluginCommand::undo (void)
 
 // Constructor.
 qtractorAuxSendPluginCommand::qtractorAuxSendPluginCommand (
-	qtractorAuxSendPlugin *pAuxSendPlugin, const QString& sAudioBusName )
-	: qtractorPluginCommand(QObject::tr("aux-send bus"), pAuxSendPlugin),
-		m_sAudioBusName(sAudioBusName)
+	qtractorPlugin *pPlugin, const QString& sAuxSendBusName )
+	: qtractorPluginCommand(QObject::tr("aux-send bus"), pPlugin),
+		m_sAuxSendBusName(sAuxSendBusName)
 {
 }
 
@@ -198,16 +198,29 @@ qtractorAuxSendPluginCommand::qtractorAuxSendPluginCommand (
 // Plugin insertion command methods.
 bool qtractorAuxSendPluginCommand::redo (void)
 {
-	qtractorAuxSendPlugin *pAuxSendPlugin
-		= static_cast<qtractorAuxSendPlugin *> (plugins().first());
-	if (pAuxSendPlugin == NULL)
+	qtractorPlugin *pPlugin = plugins().first();
+	if (pPlugin == NULL)
 		return false;
 
-	const QString sAudioBusName = pAuxSendPlugin->audioBusName();
-	pAuxSendPlugin->setAudioBusName(m_sAudioBusName);
-	m_sAudioBusName = sAudioBusName;
-
-	pAuxSendPlugin->updateFormAudioBusName();
+	if ((pPlugin->type())->index() > 0) {
+		qtractorAudioAuxSendPlugin *pAudioAuxSendPlugin
+			= static_cast<qtractorAudioAuxSendPlugin *> (pPlugin);
+		if (pAudioAuxSendPlugin == NULL)
+			return false;
+		const QString sAudioBusName = pAudioAuxSendPlugin->audioBusName();
+		pAudioAuxSendPlugin->setAudioBusName(m_sAuxSendBusName);
+		m_sAuxSendBusName = sAudioBusName;
+		pAudioAuxSendPlugin->updateFormAuxSendBusName();
+	} else {
+		qtractorMidiAuxSendPlugin *pMidiAuxSendPlugin
+			= static_cast<qtractorMidiAuxSendPlugin *> (pPlugin);
+		if (pMidiAuxSendPlugin == NULL)
+			return false;
+		const QString sMidiBusName = pMidiAuxSendPlugin->midiBusName();
+		pMidiAuxSendPlugin->setMidiBusName(m_sAuxSendBusName);
+		m_sAuxSendBusName = sMidiBusName;
+		pMidiAuxSendPlugin->updateFormAuxSendBusName();
+	}
 
 	return true;
 }
@@ -266,7 +279,7 @@ bool qtractorInsertPluginCommand::redo (void)
 	if (pSession == NULL)
 		return false;
 
-	// Save the previous track alright...
+	// Save the previous plugin alright...
 	qtractorPluginList *pPluginList = pPlugin->list();
 	if (pPluginList == NULL)
 		return false;
@@ -422,6 +435,7 @@ bool qtractorActivatePluginCommand::undo (void)
 	return redo();
 }
 
+
 //----------------------------------------------------------------------
 // class qtractorPresetPluginCommand - implementation
 //
@@ -449,7 +463,7 @@ bool qtractorPresetPluginCommand::redo (void)
 
 	pPlugin->setPreset(m_sPreset);
 	pPlugin->setValueList(m_vlist);
-	pPlugin->realizeValues();
+	pPlugin->releaseValues();
 
 	// Swap it nice, finally.
 	m_sPreset = sPreset;
@@ -497,7 +511,7 @@ bool qtractorResetPluginCommand::redo (void)
 		pPlugin->reset();
 	} else {
 		pPlugin->setValueList(m_vlist);
-		pPlugin->realizeValues();
+		pPlugin->releaseValues();
 	}
 
 	// Swap it nice.
@@ -523,9 +537,9 @@ bool qtractorResetPluginCommand::undo (void)
 //
 
 // Constructor.
-qtractorProgramPluginCommand::qtractorProgramPluginCommand (
+qtractorPluginProgramCommand::qtractorPluginProgramCommand (
 	qtractorPlugin *pPlugin, int iBank, int iProg )
-	: qtractorPluginCommand(QObject::tr("program plugin"), pPlugin)
+	: qtractorPluginCommand(QObject::tr("plugin program"), pPlugin)
 {
 	m_iBank = iBank;
 	m_iProg = iProg;
@@ -533,7 +547,7 @@ qtractorProgramPluginCommand::qtractorProgramPluginCommand (
 
 
 // Plugin-preset command methods.
-bool qtractorProgramPluginCommand::redo (void)
+bool qtractorPluginProgramCommand::redo (void)
 {
 	qtractorPlugin *pPlugin = plugins().first();
 	if (pPlugin == NULL)
@@ -557,7 +571,74 @@ bool qtractorProgramPluginCommand::redo (void)
 	return true;
 }
 
-bool qtractorProgramPluginCommand::undo (void)
+bool qtractorPluginProgramCommand::undo (void)
+{
+	// As we swap the prev/state this is non-idempotent.
+	return redo();
+}
+
+
+//----------------------------------------------------------------------
+// class qtractorProgramPluginCommand - implementation
+//
+
+// Constructor.
+qtractorPluginPropertyCommand::qtractorPluginPropertyCommand (
+	qtractorPlugin *pPlugin, unsigned long iProperty, const QVariant& value )
+	: qtractorPluginCommand(QObject::tr("plugin property"), pPlugin)
+{
+	m_iProperty = iProperty;
+	m_value = value;
+
+	setRefresh(false);
+}
+
+
+// Plugin-property command methods.
+bool qtractorPluginPropertyCommand::redo (void)
+{
+	qtractorPlugin *pPlugin = plugins().first();
+	if (pPlugin == NULL)
+		return false;
+
+	// Save the current toggled state alright...
+	QVariant value;
+
+#ifdef CONFIG_LV2_PATCH
+	qtractorPluginType *pType = pPlugin->type();
+	qtractorLv2Plugin *pLv2Plugin = NULL;
+	qtractorLv2Plugin::Property *pLv2Prop = NULL;
+	if (pType && pType->typeHint() == qtractorPluginType::Lv2)
+		pLv2Plugin = static_cast<qtractorLv2Plugin *> (pPlugin);
+	if (pLv2Plugin) {
+		const LV2_URID key = m_iProperty;
+		const char *pszKey = qtractorLv2Plugin::lv2_urid_unmap(key);
+		if (pszKey) {
+			pLv2Prop = pLv2Plugin->lv2_properties().value(pszKey, NULL);
+			if (pLv2Prop) {
+				value = pLv2Prop->value();
+				pLv2Prop->setValue(m_value);
+				pLv2Plugin->lv2_property_update(key);
+			}
+		}
+	}
+#endif
+
+	m_value = value;
+
+	// Update the form, showing it up as necessary...
+	pPlugin->updateFormDirtyCount();
+
+	// Update any GUI editor...
+	// pPlugin->idleEditor();
+
+	// FIXME: Might no work the first time...
+	pPlugin->refreshForm();
+
+	return true;
+}
+
+bool qtractorPluginPropertyCommand::undo (void)
 {
 	// As we swap the prev/state this is non-idempotent.
 	return redo();
@@ -624,7 +705,7 @@ bool qtractorPluginParamCommand::redo (void)
 	m_bUpdate    = true;
 
 	// Update the form, showing it up as necessary...
-	pPlugin->updateFormParamValue(m_pParam->index());
+	pPlugin->updateFormDirtyCount();
 
 	// Update any GUI editor...
 	// pPlugin->idleEditor();
