@@ -1,7 +1,7 @@
 // qtractorPluginCommand.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -26,9 +26,9 @@
 
 #include "qtractorPlugin.h"
 
+#include <QVariant>
 #include <QList>
 
-
 // Forward declarations...
 class qtractorAuxSendPlugin;
 class qtractorPluginPortWidget;
@@ -131,8 +131,8 @@ class qtractorAuxSendPluginCommand : public qtractorPluginCommand
 public:
 
 	// Constructor.
-	qtractorAuxSendPluginCommand(qtractorAuxSendPlugin *pAuxSendPlugin,
-		const QString& sAudioBusName);
+	qtractorAuxSendPluginCommand(
+		qtractorPlugin *pPlugin, const QString& sAuxSendBusName);
 
 	// Plugin insertion command methods.
 	bool redo();
@@ -141,7 +141,7 @@ public:
 private:
 
 	// Instance variables.
-	QString m_sAudioBusName;
+	QString m_sAuxSendBusName;
 };
 
 
@@ -289,12 +289,12 @@ private:
 // class qtractorPluginProgramCommand - declaration.
 //
 
-class qtractorProgramPluginCommand : public qtractorPluginCommand
+class qtractorPluginProgramCommand : public qtractorPluginCommand
 {
 public:
 
 	// Constructor.
-	qtractorProgramPluginCommand(qtractorPlugin *pPlugin,
+	qtractorPluginProgramCommand(qtractorPlugin *pPlugin,
 		int iBank, int iProg);
 
 	// Plugin-change command methods.
@@ -309,6 +309,30 @@ private:
 };
 
 
+//----------------------------------------------------------------------
+// class qtractorPluginPropertyCommand - declaration.
+//
+
+class qtractorPluginPropertyCommand : public qtractorPluginCommand
+{
+public:
+
+	// Constructor.
+	qtractorPluginPropertyCommand(qtractorPlugin *pPlugin,
+		unsigned long iProperty, const QVariant& value);
+
+	// Plugin-port command methods.
+	bool redo();
+	bool undo();
+
+private:
+
+	// Instance variables.
+	unsigned long m_iProperty;
+	QVariant m_value;
+};
+
+
 //----------------------------------------------------------------------
 // class qtractorPluginParamCommand - declaration.
 //
