@@ -1,7 +1,7 @@
 // qtractorPluginForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -23,10 +23,15 @@
 #include "qtractorPluginForm.h"
 #include "qtractorPluginCommand.h"
 #include "qtractorAudioEngine.h"
+#include "qtractorMidiEngine.h"
 
 #include "qtractorPlugin.h"
 #include "qtractorPluginListView.h"
 
+#ifdef CONFIG_LV2_PATCH
+#include "qtractorLv2Plugin.h"
+#endif
+
 #include "qtractorInsertPlugin.h"
 
 #include "qtractorObserverWidget.h"
@@ -43,7 +48,7 @@
 #include <QTabWidget>
 #include <QGridLayout>
 #include <QValidator>
-#include <QLineEdit>
+#include <QTextEdit>
 #include <QLabel>
 
 #include <QFileInfo>
@@ -116,12 +121,12 @@ qtractorPluginForm::qtractorPluginForm (
 	QObject::connect(m_ui.ReturnsToolButton,
 		SIGNAL(clicked()),
 		SLOT(returnsSlot()));
-	QObject::connect(m_ui.AudioBusNameComboBox,
+	QObject::connect(m_ui.AuxSendBusNameComboBox,
 		SIGNAL(activated(const QString&)),
-		SLOT(changeAudioBusNameSlot(const QString&)));
-	QObject::connect(m_ui.AudioBusNameToolButton,
+		SLOT(changeAuxSendBusNameSlot(const QString&)));
+	QObject::connect(m_ui.AuxSendBusNameToolButton,
 		SIGNAL(clicked()),
-		SLOT(clickAudioBusNameSlot()));
+		SLOT(clickAuxSendBusNameSlot()));
 	QObject::connect(m_ui.ActivateToolButton,
 		SIGNAL(toggled(bool)),
 		SLOT(activateSlot(bool)));
@@ -161,36 +166,43 @@ void qtractorPluginForm::setPlugin ( qtractorPlugin *pPlugin )
 		m_pPlugin->activateObserver());
 
 	qtractorPluginType *pType = m_pPlugin->type();
-
 	const bool bVstPlugin = (pType->typeHint() == qtractorPluginType::Vst);
 	const int MaxRowsPerPage    = (bVstPlugin ? 12 : 8);
 	const int MaxColumnsPerPage = (bVstPlugin ?  2 : 3);
-	const int MaxParamsPerPage  = MaxRowsPerPage * MaxColumnsPerPage;
+	const int MaxItemsPerPage   = MaxRowsPerPage * MaxColumnsPerPage;
 
 	const qtractorPlugin::Params& params = m_pPlugin->params();
 	const int iParams = params.count();
 
-	int iParamsPerPage = iParams;
-	int iParamsOnLastPage = 0;
-	if (iParamsPerPage > MaxParamsPerPage) {
-		iParamsPerPage = MaxParamsPerPage;
-		iParamsOnLastPage = (iParams % iParamsPerPage);
-		while (iParamsOnLastPage > 0
-			&& iParamsOnLastPage < ((3 * iParamsPerPage) >> 2))
-			iParamsOnLastPage = (iParams % --iParamsPerPage);
+	int iItems = iParams;
+#ifdef CONFIG_LV2_PATCH
+	qtractorLv2Plugin *pLv2Plugin = NULL;
+	if (pType->typeHint() == qtractorPluginType::Lv2)
+		pLv2Plugin = static_cast<qtractorLv2Plugin *> (m_pPlugin);
+	if (pLv2Plugin)
+		iItems += pLv2Plugin->lv2_properties().count();
+#endif
+	int iItemsPerPage = iItems;
+	int iItemsOnLastPage = 0;
+	if (iItemsPerPage > MaxItemsPerPage) {
+		iItemsPerPage = MaxItemsPerPage;
+		iItemsOnLastPage = (iItems % iItemsPerPage);
+		while (iItemsOnLastPage > 0
+			&& iItemsOnLastPage < ((3 * iItemsPerPage) >> 2))
+			iItemsOnLastPage = (iItems % --iItemsPerPage);
 	}
 
 	int iPages = 1;
-	int iRowsPerPage = iParamsPerPage;
+	int iRowsPerPage = iItemsPerPage;
 	int iColumnsPerPage = 1;
 	if (iRowsPerPage > MaxRowsPerPage) {
-		iPages = (iParams / iParamsPerPage);
-		if (iParamsOnLastPage > 0)
+		iPages = (iItems / iItemsPerPage);
+		if (iItemsOnLastPage > 0)
 			++iPages;
 		while (iRowsPerPage > MaxRowsPerPage
 			&& iColumnsPerPage < MaxColumnsPerPage)
-			iRowsPerPage = (iParamsPerPage / ++iColumnsPerPage);
-		if (iParamsPerPage % iColumnsPerPage) // Adjust to balance.
+			iRowsPerPage = (iItemsPerPage / ++iColumnsPerPage);
+		if (iItemsPerPage % iColumnsPerPage) // Adjust to balance.
 			++iRowsPerPage;
 	}
 
@@ -201,7 +213,7 @@ void qtractorPluginForm::setPlugin ( qtractorPlugin *pPlugin )
 
 	int iPage = 0;
 	const QString sPage = tr("Page %1");
-	if (iParams > 0) {
+	if (iItems > 0) {
 		pTabWidget  = m_ui.TabWidget;
 		pGridLayout = new QGridLayout();
 		pGridLayout->setMargin(8);
@@ -212,10 +224,22 @@ void qtractorPluginForm::setPlugin ( qtractorPlugin *pPlugin )
 		++iPage;
 	}
 
-	// FIXME: Couldn't stand more than a hundred widgets?
-	// or do we have one dedicated editor GUI?
-	int iRow = 0;
-	int iColumn = 0;
+	QWidgetList widgets;
+
+#ifdef CONFIG_LV2_PATCH
+	if (pLv2Plugin) {
+		const qtractorLv2Plugin::Properties& props = pLv2Plugin->lv2_properties();
+		qtractorLv2Plugin::Properties::ConstIterator prop = props.constBegin();
+		const qtractorLv2Plugin::Properties::ConstIterator& prop_end = props.constEnd();
+		for ( ; prop != prop_end; ++prop) {
+			qtractorLv2Plugin::Property *pProp = prop.value();
+			qtractorPluginPropertyWidget *pPropWidget
+				= new qtractorPluginPropertyWidget(pLv2Plugin, pProp->key());
+			m_propWidgets.append(pPropWidget);
+			widgets.prepend(pPropWidget);
+		}
+	}
+#endif
 
 	qtractorPlugin::Params::ConstIterator param = params.constBegin();
 	const qtractorPlugin::Params::ConstIterator param_end = params.constEnd();
@@ -223,13 +247,21 @@ void qtractorPluginForm::setPlugin ( qtractorPlugin *pPlugin )
 		qtractorPluginParam *pParam = param.value();
 		qtractorPluginParamWidget *pParamWidget
 			= new qtractorPluginParamWidget(pParam, this);
-		m_paramWidgets.insert(pParam->index(), pParamWidget);
-		pGridLayout->addWidget(pParamWidget, iRow, iColumn);
 		qtractorMidiControlObserver *pMidiObserver = pParam->observer();
-		if (pMidiObserver) {
-		//	pMidiObserver->setCurveList(pPlugin->list()->curveList());
+		if (pMidiObserver)
 			addMidiControlAction(pParamWidget, pMidiObserver);
-		}
+		m_paramWidgets.insert(pParam->index(), pParamWidget);
+		widgets.append(pParamWidget);
+	}
+
+	// FIXME: Couldn't stand more than a hundred widgets?
+	// or do we have one dedicated editor GUI?
+	int iRow = 0;
+	int iColumn = 0;
+
+	QListIterator<QWidget *> iter(widgets);
+	while (iter.hasNext()) {
+		pGridLayout->addWidget(iter.next(), iRow, iColumn);
 		if (++iRow >= iRowsPerPage) {
 			iRow = 0;
 			if (++iColumn >= iColumnsPerPage) {
@@ -255,14 +287,23 @@ void qtractorPluginForm::setPlugin ( qtractorPlugin *pPlugin )
 
 	// Show insert tool options...
 	const bool bInsertPlugin = (pType->typeHint() == qtractorPluginType::Insert);
+	if (bInsertPlugin) {
+		if (pType->index() > 0) {
+			m_ui.SendsToolButton->setIcon(QIcon(":/images/itemAudioPortOut.png"));
+			m_ui.ReturnsToolButton->setIcon(QIcon(":/images/itemAudioPortIn.png"));
+		} else {
+			m_ui.SendsToolButton->setIcon(QIcon(":/images/itemMidiPortOut.png"));
+			m_ui.ReturnsToolButton->setIcon(QIcon(":/images/itemMidiPortIn.png"));
+		}
+	}
 	m_ui.SendsToolButton->setVisible(bInsertPlugin);
 	m_ui.ReturnsToolButton->setVisible(bInsertPlugin);
 
 	// Show aux-send tool options...
-	const bool bAuxSendPlugin	= (pType->typeHint() == qtractorPluginType::AuxSend );
-	m_ui.AudioBusNameComboBox->setVisible(bAuxSendPlugin);
-	m_ui.AudioBusNameLabel->setVisible(bAuxSendPlugin);
-	m_ui.AudioBusNameToolButton->setVisible(bAuxSendPlugin);
+	const bool bAuxSendPlugin = (pType->typeHint() == qtractorPluginType::AuxSend);
+	m_ui.AuxSendBusNameComboBox->setVisible(bAuxSendPlugin);
+	m_ui.AuxSendBusNameLabel->setVisible(bAuxSendPlugin);
+	m_ui.AuxSendBusNameToolButton->setVisible(bAuxSendPlugin);
 		
 	// Set initial plugin preset name...
 	setPreset(m_pPlugin->preset());
@@ -319,7 +360,16 @@ void qtractorPluginForm::setPreset ( const QString& sPreset )
 		sEditText = qtractorPlugin::defPreset();
 
 	++m_iUpdate;
-	m_ui.PresetComboBox->setEditText(sEditText);
+	const bool bBlockSignals
+		= m_ui.PresetComboBox->blockSignals(true);
+
+	const int iIndex = m_ui.PresetComboBox->findText(sEditText);
+	if (iIndex >= 0)
+		m_ui.PresetComboBox->setCurrentIndex(iIndex);
+	else
+		m_ui.PresetComboBox->setEditText(sEditText);
+
+	m_ui.PresetComboBox->blockSignals(bBlockSignals);
 	--m_iUpdate;
 }
 
@@ -350,17 +400,10 @@ void qtractorPluginForm::updateActivated (void)
 
 
 // Update port widget state.
-void qtractorPluginForm::updateParamValue ( unsigned long /*iIndex*/ )
+void qtractorPluginForm::updateDirtyCount (void)
 {
 #ifdef CONFIG_DEBUG_0
-	qDebug("qtractorPluginForm[%p]::updateParamValue(%lu)", this, iIndex);
-#endif
-
-#if 0
-	qtractorPluginParamWidget *pParamWidget
-		= m_paramWidgets.value(iIndex, NULL);
-	if (pParamWidget)
-		pParamWidget->refresh();
+	qDebug("qtractorPluginForm[%p]::updateDirtyCount()", this);
 #endif
 
 	// Sure is dirty...
@@ -369,46 +412,72 @@ void qtractorPluginForm::updateParamValue ( unsigned long /*iIndex*/ )
 }
 
 
-// Update specific aux-send audio bus settings.
-void qtractorPluginForm::updateAudioBusName (void)
+// Update specific aux-send bus name settings.
+void qtractorPluginForm::updateAuxSendBusName (void)
 {
-	if (m_pPlugin == NULL)
-		return;
-
-	m_ui.AudioBusNameComboBox->clear();
-
-	const QIcon icon(":/images/trackAudio.png");
-	m_ui.AudioBusNameComboBox->addItem(icon, tr("(none)"));
-
-	qtractorAuxSendPlugin *pAuxSendPlugin = NULL;
-	if ((m_pPlugin->type())->typeHint() == qtractorPluginType::AuxSend)
-		pAuxSendPlugin = static_cast<qtractorAuxSendPlugin *> (m_pPlugin);
-	if (pAuxSendPlugin == NULL)
-		return;
-
 	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return;
 
-	qtractorAudioEngine *pAudioEngine = pSession->audioEngine();
-	if (pAudioEngine == NULL)
+	if (m_pPlugin == NULL)
 		return;
 
-	for (qtractorBus *pBus = pAudioEngine->buses().first();
-			pBus; pBus = pBus->next()) {
-		if (pBus->busMode() & qtractorBus::Output) {
-			qtractorAudioBus *pAudioBus
-				= static_cast<qtractorAudioBus *> (pBus);
-			if (pAudioBus && pAudioBus->channels() == m_pPlugin->channels())
-				m_ui.AudioBusNameComboBox->addItem(icon, pAudioBus->busName());
+	m_ui.AuxSendBusNameComboBox->clear();
+
+	qtractorPluginType *pType = m_pPlugin->type();
+	if (pType->typeHint() != qtractorPluginType::AuxSend)
+		return;
+
+	QString sAuxSendBusName;
+
+	if (pType->index() > 0) {
+		qtractorAudioAuxSendPlugin *pAudioAuxSendPlugin
+			= static_cast<qtractorAudioAuxSendPlugin *> (m_pPlugin);
+		if (pAudioAuxSendPlugin == NULL)
+			return;
+		qtractorAudioEngine *pAudioEngine = pSession->audioEngine();
+		if (pAudioEngine == NULL)
+			return;
+		const QIcon iconAudio(":/images/trackAudio.png");
+		m_ui.AuxSendBusNameComboBox->addItem(iconAudio, tr("(none)"));
+		for (qtractorBus *pBus = pAudioEngine->buses().first();
+				pBus; pBus = pBus->next()) {
+			if (pBus->busMode() & qtractorBus::Output) {
+				qtractorAudioBus *pAudioBus
+					= static_cast<qtractorAudioBus *> (pBus);
+				if (pAudioBus && pAudioBus->channels() == m_pPlugin->channels())
+					m_ui.AuxSendBusNameComboBox->addItem(iconAudio,
+						pAudioBus->busName());
+			}
 		}
+		sAuxSendBusName = pAudioAuxSendPlugin->audioBusName();
+	} else {
+		qtractorMidiAuxSendPlugin *pMidiAuxSendPlugin
+			= static_cast<qtractorMidiAuxSendPlugin *> (m_pPlugin);
+		if (pMidiAuxSendPlugin == NULL)
+			return;
+		qtractorMidiEngine *pMidiEngine = pSession->midiEngine();
+		if (pMidiEngine == NULL)
+			return;
+		const QIcon iconMidi(":/images/trackMidi.png");
+		m_ui.AuxSendBusNameComboBox->addItem(iconMidi, tr("(none)"));
+		for (qtractorBus *pBus = pMidiEngine->buses().first();
+				pBus; pBus = pBus->next()) {
+			if (pBus->busMode() & qtractorBus::Output) {
+				qtractorMidiBus *pMidiBus
+					= static_cast<qtractorMidiBus *> (pBus);
+				if (pMidiBus)
+					m_ui.AuxSendBusNameComboBox->addItem(iconMidi,
+						pMidiBus->busName());
+			}
+		}
+		sAuxSendBusName = pMidiAuxSendPlugin->midiBusName();
 	}
 
-	const QString& sAudioBusName = pAuxSendPlugin->audioBusName();
-	int iIndex = m_ui.AudioBusNameComboBox->findText(sAudioBusName);
+	int iIndex = m_ui.AuxSendBusNameComboBox->findText(sAuxSendBusName);
 	if (iIndex < 0)
 		iIndex = 0;
-	m_ui.AudioBusNameComboBox->setCurrentIndex(iIndex);
+	m_ui.AuxSendBusNameComboBox->setCurrentIndex(iIndex);
 }
 
 
@@ -448,8 +517,7 @@ void qtractorPluginForm::loadPresetSlot ( const QString& sPreset )
 	if (m_iUpdate > 0 || sPreset.isEmpty())
 		return;
 
-	if (m_pPlugin->loadPresetEx(sPreset))
-		refresh();
+	m_pPlugin->loadPresetEx(sPreset);
 
 	stabilize();
 }
@@ -459,7 +527,10 @@ void qtractorPluginForm::openPresetSlot (void)
 {
 	if (m_pPlugin == NULL)
 		return;
-	if (!(m_pPlugin->type())->isConfigure())
+
+	qtractorPluginType *pType = m_pPlugin->type();
+	const bool bVstPlugin = (pType->typeHint() == qtractorPluginType::Vst);
+	if (!pType->isConfigure() && !bVstPlugin)
 		return;
 
 	if (m_iUpdate > 0)
@@ -474,9 +545,11 @@ void qtractorPluginForm::openPresetSlot (void)
 	QString sFilename;
 
 	// Prompt if file does not currently exist...
-	const QString  sExt("qtx");
+	const QString sExt("qtx");
+	QStringList filters(sExt);
+	if (bVstPlugin) { filters.append("fxp"); filters.append("fxb"); }
 	const QString& sTitle  = tr("Open Preset") + " - " QTRACTOR_TITLE;
-	const QString& sFilter = tr("Preset files (*.%1)").arg(sExt); 
+	const QString& sFilter = tr("Preset files (*.%1)").arg(filters.join(" *."));
 #if 1//QT_VERSION < 0x040400
 	// Ask for the filename to save...
 	QFileDialog::Options options = 0;
@@ -507,22 +580,21 @@ void qtractorPluginForm::openPresetSlot (void)
 	if (!sFilename.isEmpty()) {
 		if (m_pPlugin->loadPresetFile(sFilename)) {
 			// Got it loaded alright...
-			QFileInfo fi(sFilename);
-			setPreset(fi.baseName()
-				.replace((m_pPlugin->type())->label() + '-', QString()));
+			const QFileInfo fi(sFilename);
+			setPreset(fi.baseName().replace(pType->label() + '-', QString()));
 			pOptions->sPresetDir = fi.absolutePath();
 		} else {
 			// Failure (maybe wrong plugin)...
 			QMessageBox::critical(this,
 				tr("Error") + " - " QTRACTOR_TITLE,
-				tr("Preset could not be loaded\n"
-				"from \"%1\".\n\n"
+				tr("Preset could not be loaded from file:\n\n"
+				"\"%1\".\n\n"
 				"Sorry.").arg(sFilename),
 				QMessageBox::Cancel);
 		}
 	}
-	refresh();
 
+	refresh();
 	stabilize();
 }
 
@@ -543,64 +615,83 @@ void qtractorPluginForm::savePresetSlot (void)
 
 	// The current state preset is about to be saved...
 	// this is where we'll make it!
-	if (!m_pPlugin->savePreset(sPreset)) {
-		QSettings& settings = pOptions->settings();
-		settings.beginGroup(m_pPlugin->presetGroup());
-		// Which mode of preset?
-		if ((m_pPlugin->type())->isConfigure()) {
-			// Sure, we'll have something complex enough
-			// to make it save into an external file...
-			const QString sExt("qtx");
-			QFileInfo fi(QDir(pOptions->sPresetDir),
-				(m_pPlugin->type())->label() + '-' + sPreset + '.' + sExt);
-			QString sFilename = fi.absoluteFilePath();
-			// Prompt if file does not currently exist...
-			if (!fi.exists()) {
-				const QString& sTitle  = tr("Save Preset") + " - " QTRACTOR_TITLE;
-				const QString& sFilter = tr("Preset files (*.%1)").arg(sExt);
-			#if 1//QT_VERSION < 0x040400
-				// Ask for the filename to save...
-				QFileDialog::Options options = 0;
-				if (pOptions->bDontUseNativeDialogs)
-					options |= QFileDialog::DontUseNativeDialog;
-				sFilename = QFileDialog::getSaveFileName(this,
-					sTitle, sFilename, sFilter, NULL, options);
-			#else
-				// Construct save-file dialog...
-				QFileDialog fileDialog(this,
-					sTitle, sFilename, sFilter);
-				// Set proper open-file modes...
-				fileDialog.setAcceptMode(QFileDialog::AcceptSave);
-				fileDialog.setFileMode(QFileDialog::AnyFile);
-				fileDialog.setDefaultSuffix(sExt);
-				// Stuff sidebar...
-				QList<QUrl> urls(fileDialog.sidebarUrls());
-				urls.append(QUrl::fromLocalFile(pOptions->sSessionDir));
-				urls.append(QUrl::fromLocalFile(pOptions->sPresetDir));
-				fileDialog.setSidebarUrls(urls);
-				if (pOptions->bDontUseNativeDialogs)
-					fileDialog.setOptions(QFileDialog::DontUseNativeDialog);
-				// Show dialog...
-				if (fileDialog.exec())
-					sFilename = fileDialog.selectedFiles().first();
-				else
-					sFilename.clear();
-			#endif
-			}
-			// We've a filename to save the preset
-			if (!sFilename.isEmpty()) {
-				if (QFileInfo(sFilename).suffix().isEmpty())
-					sFilename += '.' + sExt;
-				if (m_pPlugin->savePresetFile(sFilename)) {
-					settings.setValue(sPreset, sFilename);
-					pOptions->sPresetDir = QFileInfo(sFilename).absolutePath();
-				}
-			}
-		}	// Just leave it to simple parameter value list...
-		else settings.setValue(sPreset, m_pPlugin->valueList());
-		settings.endGroup();
+	if (m_pPlugin->savePreset(sPreset)) {
+		refresh();
+		stabilize();
+		return;
 	}
 
+	QSettings& settings = pOptions->settings();
+	settings.beginGroup(m_pPlugin->presetGroup());
+	// Which mode of preset?
+	qtractorPluginType *pType = m_pPlugin->type();
+	const bool bVstPlugin = (pType->typeHint() == qtractorPluginType::Vst);
+	if (pType->isConfigure() || bVstPlugin) {
+		// Sure, we'll have something complex enough
+		// to make it save into an external file...
+		const QString sExt("qtx");
+		QStringList filters(sExt);
+		if (bVstPlugin) { filters.append("fxp"); filters.append("fxb"); }
+		QFileInfo fi(settings.value(sPreset).toString());
+		if (!fi.exists() || !fi.isFile())
+			fi.setFile(QDir(pOptions->sPresetDir),
+				pType->label() + '-' + sPreset + '.' + sExt);
+		QString sFilename = fi.absoluteFilePath();
+		// Prompt if file does not currently exist...
+		if (!fi.exists()) {
+			const QString& sTitle  = tr("Save Preset") + " - " QTRACTOR_TITLE;
+			const QString& sFilter = tr("Preset files (*.%1)").arg(filters.join(" *."));
+		#if 1//QT_VERSION < 0x040400
+			// Ask for the filename to save...
+			QFileDialog::Options options = 0;
+			if (pOptions->bDontUseNativeDialogs)
+				options |= QFileDialog::DontUseNativeDialog;
+			sFilename = QFileDialog::getSaveFileName(this,
+				sTitle, sFilename, sFilter, NULL, options);
+		#else
+			// Construct save-file dialog...
+			QFileDialog fileDialog(this,
+				sTitle, sFilename, sFilter);
+			// Set proper open-file modes...
+			fileDialog.setAcceptMode(QFileDialog::AcceptSave);
+			fileDialog.setFileMode(QFileDialog::AnyFile);
+			fileDialog.setDefaultSuffix(sExt);
+			// Stuff sidebar...
+			QList<QUrl> urls(fileDialog.sidebarUrls());
+			urls.append(QUrl::fromLocalFile(pOptions->sSessionDir));
+			urls.append(QUrl::fromLocalFile(pOptions->sPresetDir));
+			fileDialog.setSidebarUrls(urls);
+			if (pOptions->bDontUseNativeDialogs)
+				fileDialog.setOptions(QFileDialog::DontUseNativeDialog);
+			// Show dialog...
+			if (fileDialog.exec())
+				sFilename = fileDialog.selectedFiles().first();
+			else
+				sFilename.clear();
+		#endif
+		}
+		// We've a filename to save the preset...
+		if (!sFilename.isEmpty() && sFilename.at(0) != '.') {
+			fi.setFile(sFilename);
+			if (fi.suffix().isEmpty())
+				sFilename += '.' + sExt;
+			if (m_pPlugin->savePresetFile(sFilename)) {
+				settings.setValue(sPreset, sFilename);
+				pOptions->sPresetDir = fi.absolutePath();
+			} else {
+				// Failure (maybe wrong suffix)...
+				QMessageBox::critical(this,
+					tr("Error") + " - " QTRACTOR_TITLE,
+					tr("Preset could not be saved to file:\n\n"
+					"\"%1\".\n\n"
+					"Sorry.").arg(sFilename),
+					QMessageBox::Cancel);
+			}
+		}
+	}	// Just leave it to simple parameter value list...
+	else settings.setValue(sPreset, m_pPlugin->valueList());
+	settings.endGroup();
+
 	refresh();
 	stabilize();
 }
@@ -667,7 +758,7 @@ void qtractorPluginForm::editSlot ( bool bOn )
 	++m_iUpdate;
 
 	if (bOn)
-		m_pPlugin->openEditor(NULL);
+		m_pPlugin->openEditor();
 	else
 		m_pPlugin->closeEditor();
 
@@ -690,58 +781,59 @@ void qtractorPluginForm::returnsSlot (void)
 
 
 // Audio bus name (aux-send) select slot.
-void qtractorPluginForm::changeAudioBusNameSlot ( const QString& sAudioBusName )
+void qtractorPluginForm::changeAuxSendBusNameSlot ( const QString& sAuxSendBusName )
 {
-	if (m_pPlugin == NULL)
-		return;
-
-	qtractorAuxSendPlugin *pAuxSendPlugin = NULL;
-	if (m_pPlugin->type()->typeHint() == qtractorPluginType::AuxSend)
-		pAuxSendPlugin = static_cast<qtractorAuxSendPlugin *> (m_pPlugin);
-	if (pAuxSendPlugin == NULL)
-		return;
-
 	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return;
 
+	if (m_pPlugin == NULL)
+		return;
+
+	qtractorPluginType *pType = m_pPlugin->type();
+	if (pType->typeHint() != qtractorPluginType::AuxSend)
+		return;
+
 	pSession->execute(
-		new qtractorAuxSendPluginCommand(pAuxSendPlugin, sAudioBusName));
+		new qtractorAuxSendPluginCommand(m_pPlugin, sAuxSendBusName));
 }
 
 
 // Audio bus name (aux-send) browse slot.
-void qtractorPluginForm::clickAudioBusNameSlot (void)
+void qtractorPluginForm::clickAuxSendBusNameSlot (void)
 {
-	if (m_pPlugin == NULL)
-		return;
-
-	qtractorAuxSendPlugin *pAuxSendPlugin = NULL;
-	if (m_pPlugin->type()->typeHint() == qtractorPluginType::AuxSend)
-		pAuxSendPlugin = static_cast<qtractorAuxSendPlugin *> (m_pPlugin);
-	if (pAuxSendPlugin == NULL)
-		return;
-
 	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return;
 
-	qtractorAudioEngine *pAudioEngine = pSession->audioEngine();
-	if (pAudioEngine == NULL)
+	if (m_pPlugin == NULL)
 		return;
-	
+
+	qtractorPluginType *pType = m_pPlugin->type();
+	if (pType->typeHint() != qtractorPluginType::AuxSend)
+		return;
+
+	qtractorEngine *pEngine = NULL;
+	if (pType->index() > 0)
+		pEngine = pSession->audioEngine();
+	else
+		pEngine = pSession->midiEngine();
+	if (pEngine == NULL)
+		return;
+
 	// Call here the bus management form.
 	qtractorBusForm busForm(this);
 	// Pre-select bus...
-	const QString& sAudioBusName = m_ui.AudioBusNameComboBox->currentText();
-	if (!sAudioBusName.isEmpty())
-		busForm.setBus(pAudioEngine->findBus(sAudioBusName));
+	const QString& sAuxSendBusName
+		= m_ui.AuxSendBusNameComboBox->currentText();
+	if (!sAuxSendBusName.isEmpty())
+		busForm.setBus(pEngine->findBus(sAuxSendBusName));
 	// Go for it...
 	busForm.exec();
 
 	// Check if any buses have changed...
 	if (busForm.isDirty())
-		updateAudioBusName();
+		updateAuxSendBusName();
 }
 
 
@@ -774,7 +866,7 @@ void qtractorPluginForm::updateDirectAccessParamSlot (void)
 		tr("&None"), this, SLOT(changeDirectAccessParamSlot()));
 	pAction->setCheckable(true);
 	pAction->setChecked(iDirectAccessParamIndex < 0);
-	pAction->setData(int(-1));	
+	pAction->setData(int(-1));
 }
 
 
@@ -835,11 +927,13 @@ void qtractorPluginForm::refresh (void)
 	if (m_iUpdate > 0)
 		return;
 
-#ifdef CONFIG_DEBUG
+#ifdef CONFIG_DEBUG_0
 	qDebug("qtractorPluginForm[%p]::refresh()", this);
 #endif
 
 	++m_iUpdate;
+	const bool bBlockSignals
+		= m_ui.PresetComboBox->blockSignals(true);
 
 	qtractorSubject::flushQueue(true);
 
@@ -848,20 +942,28 @@ void qtractorPluginForm::refresh (void)
 	m_ui.PresetComboBox->insertItems(0, m_pPlugin->presetList());
 	m_ui.PresetComboBox->model()->sort(0);
 	m_ui.PresetComboBox->addItem(qtractorPlugin::defPreset());
-	m_ui.PresetComboBox->setEditText(sOldPreset);
-
-	ParamWidgets::ConstIterator iter = m_paramWidgets.constBegin();
-	const ParamWidgets::ConstIterator& iter_end = m_paramWidgets.constEnd();
-	for ( ; iter != iter_end; ++iter)
-		iter.value()->refresh();
-
-	updateAudioBusName();
+	const int iIndex = m_ui.PresetComboBox->findText(sOldPreset);
+	if (iIndex >= 0)
+		m_ui.PresetComboBox->setCurrentIndex(iIndex);
+	else
+		m_ui.PresetComboBox->setEditText(sOldPreset);
 
 	m_pPlugin->idleEditor();
 
+	QListIterator<qtractorPluginPropertyWidget *> prop_iter(m_propWidgets);
+	while (prop_iter.hasNext())
+		prop_iter.next()->refresh();
+
+	QListIterator<qtractorPluginParamWidget *> param_iter(m_paramWidgets);
+	while (param_iter.hasNext())
+		param_iter.next()->refresh();
+
+	updateAuxSendBusName();
+
 	qtractorSubject::resetQueue();
 
 	m_iDirtyCount = 0;
+	m_ui.PresetComboBox->blockSignals(bBlockSignals);
 	--m_iUpdate;
 }
 
@@ -869,18 +971,18 @@ void qtractorPluginForm::refresh (void)
 // Preset control.
 void qtractorPluginForm::stabilize (void)
 {
+	qtractorPluginType *pType = m_pPlugin->type();
+	const bool bVstPlugin = (pType->typeHint() == qtractorPluginType::Vst);
+
 	bool bExists  = false;
 	bool bEnabled = (m_pPlugin != NULL);
 	m_ui.ActivateToolButton->setEnabled(bEnabled);
-	if (bEnabled) {
-		bEnabled = (
-			(m_pPlugin->type())->controlIns() > 0 ||
-			(m_pPlugin->type())->isConfigure());
-	}
+	if (bEnabled)
+		bEnabled = (pType->controlIns() > 0	|| pType->isConfigure());
 
 	m_ui.PresetComboBox->setEnabled(bEnabled);
 	m_ui.OpenPresetToolButton->setVisible(
-		bEnabled && (m_pPlugin->type())->isConfigure());
+		bEnabled && (pType->isConfigure() || bVstPlugin));
 
 	if (bEnabled) {
 		const QString& sPreset = m_ui.PresetComboBox->currentText();
@@ -906,6 +1008,9 @@ void qtractorPluginForm::clear (void)
 	qDeleteAll(m_paramWidgets);
 	m_paramWidgets.clear();
 
+	qDeleteAll(m_propWidgets);
+	m_propWidgets.clear();
+
 	m_pDirectAccessParamMenu->clear();	
 }
 
@@ -928,6 +1033,21 @@ void qtractorPluginForm::keyPressEvent ( QKeyEvent *pKeyEvent )
 }
 
 
+// Form show event (refresh).
+void qtractorPluginForm::showEvent ( QShowEvent *pShowEvent )
+{
+#ifdef CONFIG_DEBUG_0
+	qDebug("qtractorPluginForm[%p]::showEvent()", this);
+#endif
+
+	QWidget::showEvent(pShowEvent);
+
+	// Make sure all plugin-data is up-to-date...
+	refresh();
+	stabilize();
+}
+
+
 // Form hide event (save visible position).
 void qtractorPluginForm::hideEvent ( QHideEvent *pHideEvent )
 {
@@ -1092,15 +1212,14 @@ qtractorPluginParamWidget::qtractorPluginParamWidget (
 		pGridLayout->addWidget(m_pDisplay, 0, 2);
 	} else {
 		QLabel *pLabel = new QLabel(/*this*/);
+		pLabel->setText(m_pParam->name() + ':');
 		if (m_pParam->isDisplay()) {
 			pLabel->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
 		//	pLabel->setFixedWidth(72);
 			pLabel->setMinimumWidth(64);
-			pLabel->setText(m_pParam->name());
 			pGridLayout->addWidget(pLabel, 0, 0);
 		} else {
 			pLabel->setAlignment(Qt::AlignLeft | Qt::AlignBottom);
-			pLabel->setText(m_pParam->name() + ':');
 			pGridLayout->addWidget(pLabel, 0, 0, 1, 3);
 		}
 		m_pSlider = new qtractorObserverSlider(/*this*/);
@@ -1186,4 +1305,294 @@ void qtractorPluginParamWidget::updateValue ( float fValue )
 }
 
 
+//----------------------------------------------------------------------------
+// qtractorPluginPropertyWidget -- Plugin property widget.
+//
+
+// Constructor.
+qtractorPluginPropertyWidget::qtractorPluginPropertyWidget (
+	qtractorPlugin *pPlugin, unsigned long iProperty, QWidget *pParent )
+	: QWidget(pParent), m_pPlugin(pPlugin), m_iProperty(iProperty)
+{
+	m_pCheckBox   = NULL;
+	m_pSpinBox    = NULL;
+	m_pTextEdit   = NULL;
+	m_pComboBox   = NULL;
+	m_pToolButton = NULL;
+
+	QGridLayout *pGridLayout = new QGridLayout();
+	pGridLayout->setMargin(0);
+	pGridLayout->setSpacing(4);
+
+#ifdef CONFIG_LV2_PATCH
+	qtractorPluginType *pType = m_pPlugin->type();
+	qtractorLv2Plugin *pLv2Plugin = NULL;
+	qtractorLv2Plugin::Property *pLv2Prop = NULL;
+	if (pType && pType->typeHint() == qtractorPluginType::Lv2)
+		pLv2Plugin = static_cast<qtractorLv2Plugin *> (m_pPlugin);
+	if (pLv2Plugin) {
+		const LV2_URID key = m_iProperty;
+		const char *pszKey = qtractorLv2Plugin::lv2_urid_unmap(key);
+		if (pszKey)
+			pLv2Prop = pLv2Plugin->lv2_properties().value(pszKey, NULL);
+	}
+#endif
+
+#ifdef CONFIG_LV2_PATCH
+	if (pLv2Prop) {
+	//	pGridLayout->setColumnMinimumWidth(0, 120);
+		pGridLayout->setColumnMinimumWidth(2, 32);
+		if (pLv2Prop->isToggled()) {
+			m_pCheckBox = new QCheckBox(/*this*/);
+		//	m_pCheckBox->setMinimumWidth(120);
+			m_pCheckBox->setText(pLv2Prop->name());
+		//	m_pCheckBox->setChecked(pLv2Prop->value().toBool());
+			pGridLayout->addWidget(m_pCheckBox, 0, 0, 1, 3);
+		} else {
+			QLabel *pLabel = new QLabel(/*this*/);
+			pLabel->setText(pLv2Prop->name() + ':');
+			if (pLv2Prop->isString()) {
+				pLabel->setAlignment(Qt::AlignLeft | Qt::AlignBottom);
+			//	pLabel->setMinimumWidth(120);
+				pGridLayout->addWidget(pLabel, 0, 0, 1, 3);
+				m_pTextEdit = new QTextEdit(/*this*/);
+				m_pTextEdit->setTabChangesFocus(true);
+				m_pTextEdit->setMinimumWidth(120);
+				m_pTextEdit->setMaximumHeight(60);
+				m_pTextEdit->installEventFilter(this);
+			//	m_pTextEdit->setPlainText(pLv2Prop->value().toString());
+				pGridLayout->addWidget(m_pTextEdit, 1, 0, 2, 3);
+			}
+			else
+			if (pLv2Prop->isPath()) {
+				pLabel->setAlignment(Qt::AlignLeft | Qt::AlignBottom);
+			//	pLabel->setMinimumWidth(120);
+				pGridLayout->addWidget(pLabel, 0, 0, 1, 3);
+				m_pComboBox = new QComboBox(/*this*/);
+				m_pComboBox->setEditable(false);
+				m_pComboBox->setMinimumWidth(120);
+			//	m_pComboBox->addItem(pLv2Prop->value().toString());
+				pGridLayout->addWidget(m_pComboBox, 1, 0, 1, 2);
+				m_pToolButton = new QToolButton(/*this*/);
+				m_pToolButton->setIcon(QIcon(":/images/fileOpen.png"));
+				pGridLayout->addWidget(m_pToolButton, 1, 2);
+			} else {
+				pLabel->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
+			//	pLabel->setMinimumWidth(120);
+				pGridLayout->addWidget(pLabel, 0, 0);
+				const bool bIsInteger = pLv2Prop->isInteger();
+				m_pSpinBox = new QDoubleSpinBox(/*this*/);
+				m_pSpinBox->setMaximumWidth(64);
+				m_pSpinBox->setDecimals(bIsInteger ? 0 : 3);
+				m_pSpinBox->setMinimum(pLv2Prop->minValue());
+				m_pSpinBox->setMaximum(pLv2Prop->maxValue());
+				m_pSpinBox->setAlignment(Qt::AlignHCenter);
+				m_pSpinBox->setSingleStep(bIsInteger ? 1.0f : 0.001f);
+				m_pSpinBox->setAccelerated(!bIsInteger);
+			//	m_pSpinBox->setValue(pLv2Prop->value().toDouble());
+				pGridLayout->addWidget(m_pSpinBox, 0, 1);
+			}
+		}
+	}
+#endif	// CONFIG_LV2_PATCH
+
+	if (m_pCheckBox) {
+		QObject::connect(m_pCheckBox,
+			SIGNAL(toggled(bool)),
+			SLOT(propertyChanged()));
+	}
+
+	if (m_pSpinBox) {
+		QObject::connect(m_pSpinBox,
+			SIGNAL(valueChanged(double)),
+			SLOT(propertyChanged()));
+	}
+
+	if (m_pComboBox) {
+		QObject::connect(m_pComboBox,
+			SIGNAL(activated(int)),
+			SLOT(propertyChanged()));
+	}
+
+	if (m_pToolButton) {
+		QObject::connect(m_pToolButton,
+			SIGNAL(clicked()),
+			SLOT(buttonClicked()));
+	}
+
+	QWidget::setLayout(pGridLayout);
+
+#ifdef CONFIG_LV2_PATCH
+	if (pLv2Prop) QWidget::setToolTip(pLv2Prop->name());
+#endif
+}
+
+
+// Refreshner-loader method.
+void qtractorPluginPropertyWidget::refresh (void)
+{
+#ifdef CONFIG_DEBUG_0
+	qDebug("qtractorPluginPropertyWidget[%p]::refresh()", this);
+#endif
+
+#ifdef CONFIG_LV2_PATCH
+	qtractorPluginType *pType = m_pPlugin->type();
+	qtractorLv2Plugin *pLv2Plugin = NULL;
+	if (pType && pType->typeHint() == qtractorPluginType::Lv2)
+		pLv2Plugin = static_cast<qtractorLv2Plugin *> (m_pPlugin);
+	if (pLv2Plugin) {
+		qtractorLv2Plugin::Property *pLv2Prop = NULL;
+		const LV2_URID key = m_iProperty;
+		const char *pszKey = qtractorLv2Plugin::lv2_urid_unmap(key);
+		if (pszKey)
+			pLv2Prop = pLv2Plugin->lv2_properties().value(pszKey, NULL);
+		if (pLv2Prop) {
+			if (m_pCheckBox) {
+				const bool bCheckBox = m_pCheckBox->blockSignals(true);
+				m_pCheckBox->setChecked(pLv2Prop->value().toBool());
+				m_pCheckBox->blockSignals(bCheckBox);
+			}
+			if (m_pSpinBox) {
+				const bool bSpinBox = m_pSpinBox->blockSignals(true);
+				m_pSpinBox->setValue(pLv2Prop->value().toDouble());
+				m_pSpinBox->blockSignals(bSpinBox);
+			}
+			if (m_pTextEdit) {
+				const bool bTextEdit = m_pTextEdit->blockSignals(true);
+				m_pTextEdit->setPlainText(pLv2Prop->value().toString());
+				m_pTextEdit->document()->setModified(false);
+				m_pTextEdit->blockSignals(bTextEdit);
+			}
+			if (m_pComboBox) {
+				const bool bComboBox = m_pComboBox->blockSignals(true);
+				const QFileInfo fi(pLv2Prop->value().toString());
+				const QString& sPath = fi.canonicalFilePath();
+				int iIndex = m_pComboBox->findData(sPath);
+				if (iIndex < 0) {
+					m_pComboBox->insertItem(0, fi.fileName(), sPath);
+					iIndex = 0;
+				}
+				m_pComboBox->setCurrentIndex(iIndex);
+				m_pComboBox->setToolTip(sPath);
+				m_pComboBox->blockSignals(bComboBox);
+			}
+		}
+	}
+#endif
+}
+
+
+// Property file selector.
+void qtractorPluginPropertyWidget::buttonClicked (void)
+{
+	// Sure we have this...
+	if (m_pComboBox == NULL)
+		return;
+
+	// We'll need this, sure.
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
+	if (pOptions == NULL)
+		return;
+
+	// Ask for the filename to open...
+	const int iIndex = m_pComboBox->currentIndex();
+	if (iIndex < 0)
+		return;
+
+	QString sFilename = m_pComboBox->itemData(iIndex).toString();
+
+	const QString& sTitle = tr("Open File") + " - " QTRACTOR_TITLE;
+#if 1//QT_VERSION < 0x040400
+	QFileDialog::Options options = 0;
+	if (pOptions->bDontUseNativeDialogs)
+		options |= QFileDialog::DontUseNativeDialog;
+	sFilename = QFileDialog::getOpenFileName(this,
+		sTitle, sFilename, QString(), NULL, options);
+#else
+	// Construct open-file dialog...
+	QFileDialog fileDialog(this, sTitle, sFilename);
+	// Set proper open-file modes...
+	fileDialog.setAcceptMode(QFileDialog::AcceptOpen);
+	fileDialog.setFileMode(QFileDialog::ExistingFile);
+	if (pOptions->bDontUseNativeDialogs)
+		fileDialog.setOptions(QFileDialog::DontUseNativeDialog);
+	// Show dialog...
+	if (fileDialog.exec())
+		sFilename = fileDialog.selectedFiles().first();
+#endif
+	if (!sFilename.isEmpty()) {
+		const QFileInfo fi(sFilename);
+		const QString& sPath = fi.canonicalFilePath();
+		int iIndex = m_pComboBox->findData(sPath);
+		if (iIndex < 0) {
+			m_pComboBox->insertItem(0, fi.fileName(), sPath);
+			iIndex = 0;
+		}
+		m_pComboBox->setCurrentIndex(iIndex);
+		m_pComboBox->setToolTip(sPath);
+		propertyChanged();
+	}
+}
+
+
+// Property value change slot.
+void qtractorPluginPropertyWidget::propertyChanged (void)
+{
+	if (m_pPlugin == NULL)
+		return;
+
+#ifdef CONFIG_DEBUG_0
+	qDebug("qtractorPluginPropertyWidget[%p]::propertyChanged()", this);
+#endif
+
+	QVariant value;
+
+	if (m_pCheckBox)
+		value = bool(m_pCheckBox->isChecked());
+	if (m_pSpinBox)
+		value = float(m_pSpinBox->value());
+	if (m_pTextEdit && m_pTextEdit->document()->isModified()) {
+		value = m_pTextEdit->toPlainText();
+		m_pTextEdit->document()->setModified(false);
+	}
+	if (m_pComboBox) {
+		const int iIndex = m_pComboBox->currentIndex();
+		if (iIndex >= 0)
+			value = m_pComboBox->itemData(iIndex).toString();
+	}
+
+	if (!value.isValid())
+		return;
+
+	// Make it as an undoable command...
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession)
+		pSession->execute(
+			new qtractorPluginPropertyCommand(m_pPlugin, m_iProperty, value));
+}
+
+
+// Text edit (string) event filter.
+bool qtractorPluginPropertyWidget::eventFilter (
+	QObject *pObject, QEvent *pEvent )
+{
+	if (qobject_cast<QTextEdit *> (pObject) == m_pTextEdit) {
+		if(pEvent->type() == QEvent::KeyPress) {
+			QKeyEvent *pKeyEvent = static_cast<QKeyEvent*> (pEvent);
+			if (pKeyEvent->key() == Qt::Key_Return
+				&& (pKeyEvent->modifiers() &
+					(Qt::ShiftModifier | Qt::ControlModifier)) == 0) {
+				propertyChanged();
+				return true;
+			}
+		}
+		else
+		if(pEvent->type() == QEvent::FocusOut)
+			propertyChanged();
+	}
+
+	return QWidget::eventFilter(pObject, pEvent);
+}
+
+
 // end of qtractorPluginForm.cpp
