@@ -1,7 +1,7 @@
 // qtractorSession.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -32,7 +32,7 @@
 
 #include "qtractorMidiEngine.h"
 #include "qtractorMidiClip.h"
-#include "qtractorMidiBuffer.h"
+#include "qtractorMidiManager.h"
 
 #include "qtractorPlugin.h"
 #include "qtractorCurve.h"
@@ -308,6 +308,10 @@ void qtractorSession::clear (void)
 	}
 
 	m_pAudioPeakFactory->cleanup();
+
+	qtractorMidiControl *pMidiControl = qtractorMidiControl::getInstance();
+	if (pMidiControl)
+		pMidiControl->clearControlMap();
 }
 
 
@@ -644,15 +648,15 @@ unsigned long qtractorSession::tickFromFrame ( unsigned long iFrame )
 
 // Tick/Frame range conversion (delta conversion).
 unsigned long qtractorSession::frameFromTickRange (
-    unsigned long iTickStart, unsigned long iTickEnd )
+	unsigned long iTickStart, unsigned long iTickEnd, bool bOffset )
 {
-	return m_props.timeScale.frameFromTickRange(iTickStart, iTickEnd);
+	return m_props.timeScale.frameFromTickRange(iTickStart, iTickEnd, bOffset);
 }
 
 unsigned long qtractorSession::tickFromFrameRange (
-    unsigned long iFrameStart, unsigned long iFrameEnd )
+	unsigned long iFrameStart, unsigned long iFrameEnd, bool bOffset )
 {
-	return m_props.timeScale.tickFromFrameRange(iFrameStart, iFrameEnd);
+	return m_props.timeScale.tickFromFrameRange(iFrameStart, iFrameEnd, bOffset);
 }
 
 
@@ -1078,7 +1082,7 @@ void qtractorSession::resetAllPlugins (void)
 	// All tracks...
 	for (qtractorTrack *pTrack = m_tracks.first();
 			pTrack; pTrack = pTrack->next()) {
-		(pTrack->pluginList())->resetBuffer();
+		(pTrack->pluginList())->resetBuffers();
 	}
 	
 	// All audio buses...
@@ -1088,9 +1092,9 @@ void qtractorSession::resetAllPlugins (void)
 			= static_cast<qtractorAudioBus *> (pBus);
 		if (pAudioBus) {
 			if (pAudioBus->pluginList_in())
-				pAudioBus->pluginList_in()->resetBuffer();
+				pAudioBus->pluginList_in()->resetBuffers();
 			if (pAudioBus->pluginList_out())
-				pAudioBus->pluginList_out()->resetBuffer();
+				pAudioBus->pluginList_out()->resetBuffers();
 		}
 	}
 }
@@ -1116,9 +1120,11 @@ void qtractorSession::stabilize ( int msecs )
 #ifdef CONFIG_DEBUG_0
 	qDebug("qtractorSession::stabilize(%d)", msecs);
 #endif
+
 	// Wait a litle bit before continue...
 	QTime t;
 	t.start();
+
 	while (t.elapsed() < msecs) {
 		QThread::yieldCurrentThread();
 		QApplication::processEvents(/* QEventLoop::ExcludeUserInputEvents */);
@@ -1785,6 +1791,19 @@ qtractorTrack *qtractorSession::findTrack ( qtractorCurveList *pCurveList ) cons
 }
 
 
+// Find track of specific name.
+qtractorTrack *qtractorSession::findTrack ( const QString& sTrackName ) const
+{
+	for (qtractorTrack *pTrack = m_tracks.first();
+			pTrack; pTrack = pTrack->next()) {
+		if (pTrack->trackName() == sTrackName)
+			return pTrack;
+	}
+
+	return NULL;
+}
+
+
 // Session files registry accessor.
 qtractorFileList *qtractorSession::files (void) const
 {
