@@ -1,7 +1,7 @@
 // qtractorSession.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -149,9 +149,9 @@ public:
 
 	// Tick/Frame range conversion (delta conversion).
 	unsigned long frameFromTickRange(
-	    unsigned long iTickStart, unsigned long iTickEnd);
+		unsigned long iTickStart, unsigned long iTickEnd, bool bOffset = false);
 	unsigned long tickFromFrameRange(
-	    unsigned long iFrameStart, unsigned long iFrameEnd);
+		unsigned long iFrameStart, unsigned long iFrameEnd, bool bOffset = false);
 
 	// Beat/frame snap filters.
 	unsigned long tickSnap(unsigned long iTick);
@@ -375,6 +375,9 @@ public:
 	// Find track of specific curve-list.
 	qtractorTrack *findTrack(qtractorCurveList *pCurveList) const;
 
+	// Find track of specific name.
+	qtractorTrack *findTrack(const QString& sTrackName) const;
+
 	// Session files registry accessor.
 	qtractorFileList *files() const;
 
