@@ -1,7 +1,7 @@
 // qtractorShortcutForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -74,6 +74,8 @@ void qtractorShortcutTableItemEdit::keyPressEvent ( QKeyEvent *pKeyEvent )
 		iKey |= Qt::CTRL;
 	if (modifiers & Qt::AltModifier)
 		iKey |= Qt::ALT;
+	if (modifiers & Qt::MetaModifier)
+		iKey |= Qt::META;
 
 	QLineEdit::setText(QKeySequence(iKey).toString());
 }
@@ -269,7 +271,7 @@ qtractorShortcutForm::qtractorShortcutForm (
 	m_ui.setupUi(this);
 
 	// Window modality (let plugin/tool windows rave around).
-	QDialog::setWindowModality(Qt::ApplicationModal);
+	//QDialog::setWindowModality(Qt::ApplicationModal);
 
 	m_pActionControl = NULL;
 
@@ -414,17 +416,18 @@ bool qtractorShortcutForm::commitEditor (
 		return false;
 
 	if (!sShortcutText.isEmpty()) {
-		if (m_shortcuts.contains(sShortcutText)) {
+		QTreeWidgetItem *pItem = m_shortcuts.value(sShortcutText, NULL);
+		if (pItem) {
 			QMessageBox::warning(this,
 				tr("Warning") + " - " QTRACTOR_TITLE,
-				tr("Keyboard shortcut (%1) already assigned.")
-					.arg(sShortcutText),
+				tr("Keyboard shortcut (%1) already assigned (%2).")
+					.arg(sShortcutText)
+					.arg(pItem->text(0).remove('&')),
 				QMessageBox::Cancel);
 			pItemEditor->clear();
 			return false;
 		}
-		QTreeWidgetItem *pItem
-			= m_ui.ShortcutTable->topLevelItem(index.row());
+		pItem = m_ui.ShortcutTable->topLevelItem(index.row());
 		if (pItem) m_shortcuts.insert(sShortcutText, pItem);
 	}
 
