@@ -128,13 +128,13 @@ void qtractorTimeScale::Node::update (void)
 	ticksPerBeat = ts->ticksPerBeat();
 	tickRate = tempo * ticksPerBeat;
 	beatRate = tempo;
-#if 0
+#if 1// nb. standard MIDI tempo (BPM) is beatType=2 (quarter notes) per minute.
 	if (beatDivisor > beatType) {
-		unsigned short n = (beatDivisor - beatType);
+		const unsigned short n = (beatDivisor - beatType);
 		ticksPerBeat >>= n;
 		beatRate *= float(1 << n);
 	} else if (beatDivisor < beatType) {
-		unsigned short n = (beatType - beatDivisor);
+		const unsigned short n = (beatType - beatDivisor);
 		ticksPerBeat <<= n;
 		beatRate /= float(1 << n);
 	}
@@ -157,7 +157,7 @@ void qtractorTimeScale::Node::reset ( qtractorTimeScale::Node *pNode )
 }
 
 
-// Tempo accessor/convertors.
+// Tempo accessor/convertors. (default's quarter notes per minute)
 void qtractorTimeScale::Node::setTempoEx (
 	float fTempo, unsigned short iBeatType )
 {
@@ -691,12 +691,12 @@ QStringList qtractorTimeScale::snapItems ( int iSnap )
 
 // Tick/Frame range conversion (delta conversion).
 unsigned long qtractorTimeScale::frameFromTickRange (
-	unsigned long iTickStart, unsigned long iTickEnd )
+	unsigned long iTickStart, unsigned long iTickEnd, bool bOffset )
 {
 	Node *pNode = m_cursor.seekTick(iTickStart);
 	const unsigned long iFrameStart
 		= (pNode ? pNode->frameFromTick(iTickStart) : 0);
-	pNode = m_cursor.seekTick(iTickEnd);
+	if (!bOffset) pNode = m_cursor.seekTick(iTickEnd);
 	const unsigned long iFrameEnd
 		= (pNode ? pNode->frameFromTick(iTickEnd) : 0);
 	return (iFrameEnd > iFrameStart ? iFrameEnd - iFrameStart : 0);
@@ -704,12 +704,12 @@ unsigned long qtractorTimeScale::frameFromTickRange (
 
 
 unsigned long qtractorTimeScale::tickFromFrameRange (
-	unsigned long iFrameStart, unsigned long iFrameEnd )
+	unsigned long iFrameStart, unsigned long iFrameEnd, bool bOffset )
 {
 	Node *pNode = m_cursor.seekFrame(iFrameStart);
 	const unsigned long iTickStart
 		= (pNode ? pNode->tickFromFrame(iFrameStart) : 0);
-	pNode = m_cursor.seekFrame(iFrameEnd);
+	if (!bOffset) pNode = m_cursor.seekFrame(iFrameEnd);
 	const unsigned long iTickEnd
 		= (pNode ? pNode->tickFromFrame(iFrameEnd) : 0);
 	return (iTickEnd > iTickStart ? iTickEnd - iTickStart : 0);
