@@ -1,7 +1,7 @@
 // qtractorTrack.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -33,7 +33,7 @@
 #include "qtractorAudioBuffer.h"
 #include "qtractorMidiEngine.h"
 #include "qtractorMidiMonitor.h"
-#include "qtractorMidiBuffer.h"
+#include "qtractorMidiManager.h"
 #include "qtractorInstrument.h"
 #include "qtractorPlugin.h"
 #include "qtractorMixer.h"
@@ -209,6 +209,7 @@ private:
 void qtractorTrack::Properties::clear (void)
 {
 	trackName.clear();
+	trackIcon.clear();
 	trackType   = None;
 	monitor     = false;
 	record      = false;
@@ -233,6 +234,7 @@ qtractorTrack::Properties& qtractorTrack::Properties::copy (
 {
 	if (&props != this) {
 		trackName   = props.trackName;
+		trackIcon   = props.trackIcon;
 		trackType   = props.trackType;
 		monitor     = props.monitor;
 		record      = props.record;
@@ -340,16 +342,11 @@ qtractorTrack::qtractorTrack ( qtractorSession *pSession, TrackType trackType )
 	m_pMuteObserver   = new StateObserver(this, Mute,   m_pMuteSubject);
 	m_pSoloObserver   = new StateObserver(this, Solo,   m_pSoloSubject);
 
-	unsigned int iSampleRate = 0;
-	qtractorAudioEngine *pAudioEngine = pSession->audioEngine();
-	if (pAudioEngine)
-		iSampleRate = pAudioEngine->sampleRate();
-
 	unsigned int iFlags = qtractorPluginList::Track;
 	if (trackType == qtractorTrack::Midi)
 		iFlags |= qtractorPluginList::Midi;
 
-	m_pPluginList = new qtractorPluginList(0, 0, iSampleRate, iFlags);
+	m_pPluginList = new qtractorPluginList(0, iFlags);
 
 	m_pCurveFile = new qtractorCurveFile(m_pPluginList->curveList());
 
@@ -507,8 +504,7 @@ bool qtractorTrack::open (void)
 		if (pAudioBus) {
 			m_pMonitor = new qtractorAudioMonitor(pAudioBus->channels(),
 				m_props.gain, m_props.panning);
-			m_pPluginList->setBuffer(pAudioBus->channels(),
-				pAudioEngine->bufferSize(), pAudioEngine->sampleRate(),
+			m_pPluginList->setChannels(pAudioBus->channels(),
 				qtractorPluginList::AudioTrack);
 		}
 		break;
@@ -541,8 +537,7 @@ bool qtractorTrack::open (void)
 		}
 		// Set plugin-list buffer alright...
 		if (pAudioBus) {
-			m_pPluginList->setBuffer(pAudioBus->channels(),
-				pAudioEngine->bufferSize(), pAudioEngine->sampleRate(),
+			m_pPluginList->setChannels(pAudioBus->channels(),
 				qtractorPluginList::MidiTrack);
 		}
 		// Set MIDI bank/program observer...
@@ -569,6 +564,7 @@ bool qtractorTrack::open (void)
 			}
 		}
 		// Update gain and panning curve new subjects...
+	#if 0
 		qtractorCurveList *pCurveList = m_pPluginList->curveList();
 		if (pCurveList) {
 			qtractorCurve *pCurve = pCurveList->first();
@@ -587,7 +583,21 @@ bool qtractorTrack::open (void)
 				}
 				pCurve = pCurve->next();
 			}
-		}		
+		}
+	#else
+		// Panning subject ownership transfer...
+		qtractorCurve *pCurve = pMonitor->panningSubject()->curve();
+		if (pCurve) {
+			pCurve->setSubject(m_pMonitor->panningSubject());
+			m_pMonitor->panningSubject()->setCurve(pCurve);
+		}
+		// Gain subject ownership transfer...
+		pCurve = pMonitor->gainSubject()->curve();
+		if (pCurve) {
+			pCurve->setSubject(m_pMonitor->gainSubject());
+			m_pMonitor->gainSubject()->setCurve(pCurve);
+		}
+	#endif
 		// That's it...
 		delete pMonitor;
 	}
@@ -676,6 +686,21 @@ void qtractorTrack::updateTrackName (void)
 }
 
 
+// Track icon (filename) accessors.
+const QString& qtractorTrack::trackIcon (void) const
+{
+	return m_props.trackIcon;
+}
+
+void qtractorTrack::setTrackIcon ( const QString& sTrackIcon )
+{
+	if (sTrackIcon.at(0) == ':' || QFileInfo(sTrackIcon).exists())
+		m_props.trackIcon = sTrackIcon;
+	else
+		m_props.trackIcon.clear();
+}
+
+
 // Track type accessors.
 qtractorTrack::TrackType qtractorTrack::trackType (void) const
 {
@@ -703,12 +728,7 @@ void qtractorTrack::setTrackType ( qtractorTrack::TrackType trackType )
 	}
 
 	// (Re)set plugin-list...
-	unsigned int iSampleRate = 0;
-	qtractorAudioEngine *pAudioEngine = m_pSession->audioEngine();
-	if (pAudioEngine)
-		iSampleRate = pAudioEngine->sampleRate();
-
-	m_pPluginList->setBuffer(0, 0, iSampleRate, iFlags);
+	m_pPluginList->setChannels(0, iFlags);
 }
 
 
@@ -1206,6 +1226,12 @@ QColor qtractorTrack::trackColor ( int iTrack )
 
 
 // Alternate properties accessor.
+void qtractorTrack::setProperties ( const qtractorTrack::Properties& props )
+{
+	m_props = props;
+}
+
+
 qtractorTrack::Properties& qtractorTrack::properties (void)
 {
 	return m_props;
@@ -1244,7 +1270,7 @@ void qtractorTrack::process ( qtractorClip *pClip,
 	// Audio buffers needs monitoring and commitment...
 	if (pAudioMonitor && pOutputBus) {
 		// Plugin chain post-processing...
-		if (m_pPluginList->activated() > 0)
+		if (m_pPluginList->isActivated())
 			m_pPluginList->process(pOutputBus->buffer(), nframes);
 		// Monitor passthru...
 		pAudioMonitor->process(pOutputBus->buffer(), nframes);
@@ -1287,7 +1313,7 @@ void qtractorTrack::process_export ( qtractorClip *pClip,
 	// Audio buffers needs monitoring and commitment...
 	if (pAudioMonitor && pOutputBus) {
 		// Plugin chain post-processing...
-		if (m_pPluginList->activated() > 0)
+		if (m_pPluginList->isActivated())
 			m_pPluginList->process(pOutputBus->buffer(), nframes);
 		// Monitor passthru...
 		pAudioMonitor->process(pOutputBus->buffer(), nframes);
@@ -1625,6 +1651,8 @@ bool qtractorTrack::loadElement (
 					qtractorTrack::setMidiBank(eProp.text().toInt());
 				else if (eProp.tagName() == "midi-program")
 					qtractorTrack::setMidiProg(eProp.text().toInt());
+				else if (eProp.tagName() == "icon")
+					qtractorTrack::setTrackIcon(eProp.text());
 			}
 		}
 		else
@@ -1741,6 +1769,9 @@ bool qtractorTrack::saveElement (
 
 	// Save track properties...
 	QDomElement eProps = pDocument->document()->createElement("properties");
+	const QString& sTrackIcon = qtractorTrack::trackIcon();
+	if (!sTrackIcon.isEmpty())
+		pDocument->saveTextElement("icon", sTrackIcon, &eProps);
 	pDocument->saveTextElement("input-bus",
 		qtractorTrack::inputBusName(), &eProps);
 	pDocument->saveTextElement("output-bus",
@@ -2078,7 +2109,15 @@ qtractorCurveFile *qtractorTrack::curveFile (void) const
 }
 
 
-// Track automation current curve accessor.
+// Track automation current curve accessors.
+void qtractorTrack::setCurrentCurve ( qtractorCurve *pCurrentCurve )
+{
+	qtractorCurveList *pCurveList = curveList();
+	if (pCurveList)
+		pCurveList->setCurrentCurve(pCurrentCurve);
+}
+
+
 qtractorCurve *qtractorTrack::currentCurve (void) const
 {
 	qtractorCurveList *pCurveList = curveList();
