@@ -1,7 +1,7 @@
 // qtractorTrack.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -83,6 +83,10 @@ public:
 	const QString& trackName() const;
 	void updateTrackName();
 
+	// Track icon (filename) accessors.
+	void setTrackIcon(const QString& sTrackIcon);
+	const QString& trackIcon() const;
+
 	// Track type accessors.
 	void setTrackType(TrackType trackType);
 	TrackType trackType() const;
@@ -264,6 +268,7 @@ public:
 	qtractorCurveFile *curveFile() const;
 
 	// Track automation current curve accessor.
+	void setCurrentCurve(qtractorCurve *pCurrentCurve);
 	qtractorCurve *currentCurve() const;
 
 	// Track automation curve serialization methods.
@@ -291,6 +296,7 @@ public:
 		void clear();
 		// Members.
 		QString        trackName;
+		QString        trackIcon;
 		TrackType      trackType;
 		bool           monitor;
 		bool           record;
@@ -310,6 +316,7 @@ public:
 	};
 
 	// Alternate properties accessor.
+	void setProperties(const Properties& props);
 	Properties& properties();
 
 	// Track type textual helper methods.
