@@ -1,7 +1,7 @@
 // qtractorTrackCommand.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -27,11 +27,17 @@
 
 #include "qtractorTracks.h"
 #include "qtractorTrackList.h"
+#include "qtractorAudioClip.h"
 #include "qtractorMidiEngine.h"
 #include "qtractorMidiControl.h"
+#include "qtractorMidiManager.h"
 #include "qtractorMidiClip.h"
 #include "qtractorMixer.h"
 
+#include "qtractorMonitor.h"
+#include "qtractorPlugin.h"
+#include "qtractorCurve.h"
+
 
 //----------------------------------------------------------------------
 // class qtractorTrackCommand - implementation
@@ -39,7 +45,8 @@
 
 // Constructor.
 qtractorTrackCommand::qtractorTrackCommand ( const QString& sName,
-	qtractorTrack *pTrack ) : qtractorCommand(sName), m_pTrack(pTrack)
+	qtractorTrack *pTrack, qtractorTrack *pAfterTrack )
+	: qtractorCommand(sName), m_pTrack(pTrack), m_pAfterTrack(pAfterTrack)
 {
 	setClearSelect(true);
 }
@@ -54,10 +61,10 @@ qtractorTrackCommand::~qtractorTrackCommand (void)
 
 
 // Track command methods.
-bool qtractorTrackCommand::addTrack ( qtractorTrack *pAfterTrack )
+bool qtractorTrackCommand::addTrack (void)
 {
 #ifdef CONFIG_DEBUG
-	qDebug("qtractorTrackCommand::addTrack(%p, %p)", m_pTrack, pAfterTrack);
+	qDebug("qtractorTrackCommand::addTrack(%p, %p)", m_pTrack, m_pAfterTrack);
 #endif
 
 	if (m_pTrack == NULL)
@@ -76,13 +83,16 @@ bool qtractorTrackCommand::addTrack ( qtractorTrack *pAfterTrack )
 		return false;
 
 	// Guess which item we're adding after...
-	if (pAfterTrack == NULL)
-		pAfterTrack = m_pTrack->prev();
-	if (pAfterTrack == NULL)
-		pAfterTrack = pSession->tracks().last();
-	int iTrack = pSession->tracks().find(pAfterTrack) + 1;
+#if 0
+	if (m_pAfterTrack == NULL)
+		m_pAfterTrack = m_pTrack->prev();
+#else
+	if (m_pAfterTrack == NULL)
+		m_pAfterTrack = pSession->tracks().last();
+#endif
+	int iTrack = pSession->tracks().find(m_pAfterTrack) + 1;
 	// Link the track into session...
-	pSession->insertTrack(m_pTrack, pAfterTrack);
+	pSession->insertTrack(m_pTrack, m_pAfterTrack);
 	// And the new track list view item too...
 	qtractorTrackList *pTrackList = pTracks->trackList();
 	iTrack = pTrackList->insertTrack(iTrack, m_pTrack);
@@ -118,7 +128,7 @@ bool qtractorTrackCommand::addTrack ( qtractorTrack *pAfterTrack )
 bool qtractorTrackCommand::removeTrack (void)
 {
 #ifdef CONFIG_DEBUG
-	qDebug("qtractorTrackCommand::removeTrack(%p)", m_pTrack);
+	qDebug("qtractorTrackCommand::removeTrack(%p, %p)", m_pTrack, m_pAfterTrack);
 #endif
 
 	if (m_pTrack == NULL)
@@ -136,6 +146,13 @@ bool qtractorTrackCommand::removeTrack (void)
 	if (pTracks == NULL)
 		return false;
 
+	// Save which item we're adding after...
+	if (m_pAfterTrack == NULL)
+		m_pAfterTrack = m_pTrack->prev();
+#if 0
+	if (m_pAfterTrack == NULL)
+		m_pAfterTrack = pSession->tracks().last();
+#endif
 	// Get the list view item reference of the intended track...
 	int iTrack = pSession->tracks().find(m_pTrack);
 	if (iTrack < 0)
@@ -179,8 +196,8 @@ bool qtractorTrackCommand::removeTrack (void)
 // Constructor.
 qtractorAddTrackCommand::qtractorAddTrackCommand (
 	qtractorTrack *pTrack, qtractorTrack *pAfterTrack  )
-	: qtractorTrackCommand(QObject::tr("add track"), pTrack),
-		m_pAfterTrack(pAfterTrack)
+	: qtractorTrackCommand(QObject::tr("add track"),
+		pTrack, pAfterTrack)
 {
 }
 
@@ -188,7 +205,7 @@ qtractorAddTrackCommand::qtractorAddTrackCommand (
 // Track insertion command methods.
 bool qtractorAddTrackCommand::redo (void)
 {
-	return addTrack(m_pAfterTrack);
+	return addTrack();
 }
 
 bool qtractorAddTrackCommand::undo (void)
@@ -203,7 +220,8 @@ bool qtractorAddTrackCommand::undo (void)
 
 // Constructor.
 qtractorRemoveTrackCommand::qtractorRemoveTrackCommand ( qtractorTrack *pTrack )
-	: qtractorTrackCommand(QObject::tr("remove track"), pTrack)
+	: qtractorTrackCommand(QObject::tr("remove track"),
+		pTrack, pTrack->prev())
 {
 }
 
@@ -220,6 +238,238 @@ bool qtractorRemoveTrackCommand::undo (void)
 }
 
 
+//----------------------------------------------------------------------
+// class qtractorCopyTrackCommand - implementation
+//
+
+// Constructor.
+qtractorCopyTrackCommand::qtractorCopyTrackCommand (
+	qtractorTrack *pTrack, qtractorTrack *pAfterTrack  )
+	: qtractorTrackCommand(QObject::tr("duplicate track"),
+		pTrack, pAfterTrack), m_iCopyCount(0)
+{
+}
+
+
+// Track insertion command methods.
+bool qtractorCopyTrackCommand::redo (void)
+{
+	const bool bResult = addTrack();
+
+	if (++m_iCopyCount > 1)
+		return bResult;
+
+	// One-time copy...
+	qtractorTrack *pTrack = afterTrack();
+	qtractorTrack *pNewTrack = track();
+
+	// Copy all former clips depending of track type...
+	const qtractorTrack::TrackType trackType = pTrack->trackType();
+	for (qtractorClip *pClip = pTrack->clips().first();
+			pClip; pClip = pClip->next()) {
+		switch (trackType) {
+		case qtractorTrack::Audio: {
+			qtractorAudioClip *pAudioClip
+				= static_cast<qtractorAudioClip *> (pClip);
+			if (pAudioClip) {
+				qtractorAudioClip *pNewAudioClip
+					= new qtractorAudioClip(*pAudioClip);
+				pNewAudioClip->setClipStart(pAudioClip->clipStart());
+				pNewAudioClip->setClipOffset(pAudioClip->clipOffset());
+				pNewAudioClip->setClipLength(pAudioClip->clipLength());
+				pNewAudioClip->setFadeInLength(pAudioClip->fadeInLength());
+				pNewAudioClip->setFadeOutLength(pAudioClip->fadeOutLength());
+				pNewAudioClip->setPitchShift(pAudioClip->pitchShift());
+				pNewAudioClip->setTimeStretch(pAudioClip->timeStretch());
+				pNewTrack->addClip(pNewAudioClip);
+			}
+			break;
+		}
+		case qtractorTrack::Midi: {
+			qtractorMidiClip *pMidiClip
+				= static_cast<qtractorMidiClip *> (pClip);
+			if (pMidiClip) {
+				qtractorMidiClip *pNewMidiClip
+					= new qtractorMidiClip(*pMidiClip);
+				pNewMidiClip->setClipStart(pMidiClip->clipStart());
+				pNewMidiClip->setClipOffset(pMidiClip->clipOffset());
+				pNewMidiClip->setClipLength(pMidiClip->clipLength());
+				pNewMidiClip->setFadeInLength(pMidiClip->fadeInLength());
+				pNewMidiClip->setFadeOutLength(pMidiClip->fadeOutLength());
+				pNewTrack->addClip(pNewMidiClip);
+			}
+			break;
+		}
+		default:
+			break;
+		}
+	}
+
+	// About to copy all automation/curves...
+	qtractorCurveList *pCurveList = pTrack->curveList();
+	qtractorCurveList *pNewCurveList = pNewTrack->curveList();
+
+	// About to find and set current automation/curve...
+	qtractorCurve *pCurve, *pCurrentCurve = pTrack->currentCurve();
+	qtractorCurve *pNewCurve, *pNewCurrentCurve = NULL;
+
+	// Copy all former plugins and respective automation/curves...
+	qtractorPluginList *pPluginList = pTrack->pluginList();
+	qtractorPluginList *pNewPluginList = pNewTrack->pluginList();
+	if (pPluginList && pNewPluginList) {
+		for (qtractorPlugin *pPlugin = pPluginList->first();
+				pPlugin; pPlugin = pPlugin->next()) {
+			// Copy new plugin...
+			qtractorPlugin *pNewPlugin = pNewPluginList->copyPlugin(pPlugin);
+			if (pNewPlugin == NULL)
+				continue;
+			pNewPluginList->insertPlugin(pNewPlugin, NULL);
+			// Activation automation/curves...
+			if (pCurveList == NULL || pNewCurveList == NULL)
+				continue;
+			pCurve = pPlugin->activateSubject()->curve();
+			if (pCurve && pCurve->list() == pCurveList) {
+				pNewCurve = cloneCurve(pNewCurveList,
+					pNewPlugin->activateSubject(), pCurve);
+				if (pNewCurrentCurve == NULL && pCurrentCurve == pCurve)
+					pNewCurrentCurve = pNewCurve;
+			}
+			// Copy plugin parameters automation/curves...
+			const qtractorPlugin::Params& params = pPlugin->params();
+			qtractorPlugin::Params::ConstIterator param = params.constBegin();
+			const qtractorPlugin::Params::ConstIterator& param_end
+				= params.constEnd();
+			for ( ; param != param_end; ++param) {
+				qtractorPluginParam *pParam = param.value();
+				pCurve = pParam->subject()->curve();
+				if (pCurve && pCurve->list() == pCurveList) {
+					qtractorPluginParam *pNewParam
+						= pNewPlugin->findParam(pParam->index());
+					if (pNewParam == NULL)
+						continue;
+					pNewCurve = cloneCurve(pNewCurveList,
+						pNewParam->subject(), pCurve);
+					if (pNewCurrentCurve == NULL && pCurrentCurve == pCurve)
+						pNewCurrentCurve = pNewCurve;
+				}
+			}
+		}
+		// And other MIDI specific plugins-list properties as well
+		qtractorMidiManager *pMidiManager = pPluginList->midiManager();
+		qtractorMidiManager *pNewMidiManager = pNewPluginList->midiManager();
+		if (pMidiManager && pNewMidiManager) {
+			// The basic ones...
+			pNewPluginList->setMidiBank(
+				pPluginList->midiBank());
+			pNewPluginList->setMidiProg(
+				pPluginList->midiProg());
+			pNewPluginList->setAudioOutputBusName(
+				pPluginList->audioOutputBusName());
+			pNewPluginList->setAudioOutputAutoConnect(
+				pPluginList->isAudioOutputAutoConnect());
+			pNewPluginList->setAudioOutputBus(
+				pPluginList->isAudioOutputBus());
+			// The effective ones...
+			pNewMidiManager->setAudioOutputBusName(
+				pMidiManager->audioOutputBusName());
+			pNewMidiManager->setAudioOutputAutoConnect(
+				pMidiManager->isAudioOutputAutoConnect());
+			pNewMidiManager->setAudioOutputBus(
+				pMidiManager->isAudioOutputBus());
+		}
+	}
+
+	// Copy all former plugins and respective automation...
+	if (pCurveList && pNewCurveList) {
+		pCurve = pTrack->monitorSubject()->curve();
+		if (pCurve) {
+			pNewCurve = cloneCurve(pNewCurveList,
+				pNewTrack->monitorSubject(), pCurve);
+			if (pNewCurrentCurve == NULL && pCurrentCurve == pCurve)
+				pNewCurrentCurve = pNewCurve;
+		}
+		pCurve = pTrack->monitor()->panningSubject()->curve();
+		if (pCurve) {
+			pNewCurve = cloneCurve(pNewCurveList,
+				pNewTrack->monitor()->panningSubject(), pCurve);
+			if (pNewCurrentCurve == NULL && pCurrentCurve == pCurve)
+				pNewCurrentCurve = pNewCurve;
+		}
+		pCurve = pTrack->monitor()->gainSubject()->curve();
+		if (pCurve) {
+			pNewCurve = cloneCurve(pNewCurveList,
+				pNewTrack->monitor()->gainSubject(), pCurve);
+			if (pNewCurrentCurve == NULL && pCurrentCurve == pCurve)
+				pNewCurrentCurve = pNewCurve;
+		}
+		pCurve = pTrack->recordSubject()->curve();
+		if (pCurve) {
+			pNewCurve = cloneCurve(pNewCurveList,
+				pNewTrack->recordSubject(), pCurve);
+			if (pNewCurrentCurve == NULL && pCurrentCurve == pCurve)
+				pNewCurrentCurve = pNewCurve;
+		}
+		pCurve = pTrack->muteSubject()->curve();
+		if (pCurve) {
+			pNewCurve = cloneCurve(pNewCurveList,
+				pNewTrack->muteSubject(), pCurve);
+			if (pNewCurrentCurve == NULL && pCurrentCurve == pCurve)
+				pNewCurrentCurve = pNewCurve;
+		}
+		pCurve = pTrack->soloSubject()->curve();
+		if (pCurve) {
+			pNewCurve = cloneCurve(pNewCurveList,
+				pNewTrack->soloSubject(), pCurve);
+			if (pNewCurrentCurve == NULL && pCurrentCurve == pCurve)
+				pNewCurrentCurve = pNewCurve;
+		}
+	}
+
+	// Set final new track properties...
+	if (pNewCurrentCurve)
+		pNewTrack->setCurrentCurve(pNewCurrentCurve);
+
+	// Refresh to most recent things...
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+	if (pMainForm) {
+		qtractorMixer *pMixer = pMainForm->mixer();
+		if (pMixer)
+			pMixer->updateTrackStrip(pNewTrack);
+	}
+
+	return bResult;
+}
+
+bool qtractorCopyTrackCommand::undo (void)
+{
+	return removeTrack();
+}
+
+
+// Clone an existing automation/curve.
+qtractorCurve *qtractorCopyTrackCommand::cloneCurve (
+	qtractorCurveList *pNewCurveList, qtractorSubject *pNewSubject,
+	qtractorCurve *pCurve ) const
+{
+	qtractorCurve *pNewCurve = new qtractorCurve(pNewCurveList,
+		pNewSubject, pCurve->mode(), pCurve->minFrameDist());
+
+	pNewCurve->setDefaultValue(pCurve->defaultValue());
+	pNewCurve->setLength(pCurve->length());
+
+	pNewCurve->setCapture(pCurve->isCapture());
+	pNewCurve->setProcess(pCurve->isProcess());
+	pNewCurve->setLocked(pCurve->isLocked());
+
+	pNewCurve->setLogarithmic(pCurve->isLogarithmic());
+	pNewCurve->setColor(pCurve->color());
+
+	pNewCurve->copyNodes(pCurve);
+
+	return pNewCurve;
+}
+
+
 //----------------------------------------------------------------------
 // class qtractorMoveTrackCommand - implementation
 //
@@ -314,15 +564,27 @@ qtractorResizeTrackCommand::qtractorResizeTrackCommand (
 // Track-resize command methods.
 bool qtractorResizeTrackCommand::redo (void)
 {
+	qtractorTrack *pTrack = track();
+	if (pTrack == NULL)
+		return false;
+
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+	if (pMainForm == NULL)
+		return false;
+
 	// Save the previous item height alright...
-	const int iZoomHeight = track()->zoomHeight();
+	const int iZoomHeight = pTrack->zoomHeight();
 
 	// Just set new one...
-	track()->setZoomHeight(m_iZoomHeight);
+	pTrack->setZoomHeight(m_iZoomHeight);
 
 	// Swap it nice, finally.
 	m_iZoomHeight = iZoomHeight;
 
+	// Update track list item...
+	qtractorTrackList *pTrackList = pMainForm->tracks()->trackList();
+	pTrackList->updateTrack(pTrack);
+
 	return true;
 }
 
@@ -1002,4 +1264,78 @@ bool qtractorTrackPanningCommand::redo (void)
 }
 
 
+//----------------------------------------------------------------------
+// class qtractorTrackstrumentCommand - implementation.
+//
+
+// Constructor.
+qtractorTrackInstrumentCommand::qtractorTrackInstrumentCommand (
+	qtractorTrack *pTrack, const QString& sInstrumentName, int iBank, int iProg )
+	: qtractorTrackCommand(QObject::tr("track instrument"), pTrack),
+		m_sInstrumentName(sInstrumentName), m_iBank(iBank), m_iProg(iProg)
+{
+	setRefresh(false);
+}
+
+
+// Track-instrument command method.
+bool qtractorTrackInstrumentCommand::redo (void)
+{
+	qtractorTrack *pTrack = track();
+	if (pTrack == NULL)
+		return false;
+
+	if (pTrack->trackType() != qtractorTrack::Midi)
+		return false;
+
+	// Gotta make sure we've a proper MIDI bus...
+	qtractorMidiBus *pMidiBus
+		= static_cast<qtractorMidiBus *> (pTrack->outputBus());
+	if (pMidiBus == NULL)
+		return false;
+
+	qtractorSession *pSession = pTrack->session();
+	if (pSession == NULL)
+		return false;
+
+
+	// Set undo values...
+	const unsigned short iChannel = pTrack->midiChannel();
+	const qtractorMidiBus::Patch& patch = pMidiBus->patch(iChannel);
+	QString sInstrumentName = patch.instrumentName;
+	if (sInstrumentName.isEmpty())
+		sInstrumentName = pMidiBus->instrumentName();
+	int iBankSelMethod = pTrack->midiBankSelMethod();
+	if (iBankSelMethod < 0)
+		iBankSelMethod = patch.bankSelMethod;
+
+	const int iBank = pTrack->midiBank();
+	const int iProg = pTrack->midiProg();
+
+	// Set instrument patch...
+	pMidiBus->setPatch(iChannel, m_sInstrumentName,
+		iBankSelMethod, m_iBank, m_iProg, pTrack);
+
+	// Set track instrument...
+	pTrack->setMidiBank(m_iBank);
+	pTrack->setMidiProg(m_iProg);
+
+	// Reset undo values...
+	m_sInstrumentName = sInstrumentName;
+
+	m_iBank = iBank;
+	m_iProg = iProg;
+
+	// Refresh to most recent things...
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+	if (pMainForm) {
+		qtractorMixer *pMixer = pMainForm->mixer();
+		if (pMixer)
+			pMixer->updateTrackStrip(pTrack);
+	}
+
+	return true;
+}
+
+
 // end of qtractorTrackCommand.cpp
