@@ -1,7 +1,7 @@
 // qtractorTrackList.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -242,10 +242,12 @@ private:
 		bool updateBankProgram (qtractorMidiManager *pMidiManager,
 			const QString& sInstrument,
 			QString& sBank, QString& sProgram ) const;
-		// Item updater.
+		// Item updaters.
 		void update(qtractorTrackList *pTrackList);
+		void update_icon(qtractorTrackList *pTrackList);
 		// Item members.
 		qtractorTrack *track;
+		QPixmap        icon;
 		QStringList    text;
 		unsigned int   flags;
 		// Track-list item widget.
