@@ -1,7 +1,7 @@
 // qtractorTrackView.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -214,6 +214,9 @@ public:
 	void setEditTail(unsigned long iEditTail);
 	int editTailX() const;
 
+	// Make given contents position visible in view.
+	void ensureVisible(int cx, int cy, int mx, int my);
+
 	// Make given frame position visible in view.
 	void ensureVisibleFrame(unsigned long iFrame);
 
@@ -281,7 +284,6 @@ protected:
 
 	// Drag-n-drop event stuffers (for clips).
 	qtractorTrack *dragClipMove(const QPoint& pos, bool bKeyStep = false);
-
 	qtractorTrack *dragClipDrop(const QPoint& pos, bool bKeyStep = false,
 		const QMimeData *pMimeData = NULL);
 	qtractorTrack *dragClipDropEvent(QDropEvent *pDropEvent);
@@ -343,8 +345,14 @@ protected:
 	void moveRubberBand(qtractorRubberBand **ppRubberBand,
 		const QRect& rectDrag, int thick = 1) const;
 
-	// Clip fade-in/out handle and resize detection.
-	bool dragMoveStart(const QPoint& pos);
+	// Check whether we're up to drag something on a track or one of its clips.
+	qtractorClip *dragClipStart(const QPoint& pos,
+		const Qt::KeyboardModifiers& modifiers,
+		bool bSelectTrack = false, QRect *pClipRect = NULL);
+	// Check whether we're up to drag a clip fade-in/out or resize handles.
+	bool dragClipStartEx(const QPoint& pos,
+		const Qt::KeyboardModifiers& modifiers,
+		qtractorClip *pClip, const QRect& rectClip);
 
 	// Clip fade-in/out handle drag-move methods.
 	void dragClipFadeMove(const QPoint& pos);
@@ -354,8 +362,13 @@ protected:
 	void dragClipResizeMove(const QPoint& pos);
 	void dragClipResizeDrop(const QPoint& pos, bool bTimeStretch = false);
 
+	// Clip resize/repeat handler (over to the left/right).
+	void dragClipRepeatLeft(const QPoint& pos);
+	void dragClipRepeatRight(const QPoint& pos);
+
 	// Automation curve node drag-move methods.
-	void dragCurveNode(const QPoint& pos, bool bToggle);
+	void dragCurveNode(const QPoint& pos,
+		const Qt::KeyboardModifiers& modifiers);
 
 	// Common tool-tip builder for automation nodes.
 	QString nodeToolTip(qtractorCurve *pCurve, qtractorCurve::Node *pNode) const;
@@ -367,7 +380,7 @@ protected:
 	void keyPressEvent(QKeyEvent *pKeyEvent);
 
 	// Keyboard step handler.
-	bool keyStep(int iKey);
+	bool keyStep(int iKey, const Qt::KeyboardModifiers& modifiers);
 
 	// Vertical line positioning.
 	void drawPositionX(int& iPositionX, int x, bool bSyncView = false);
@@ -388,6 +401,7 @@ protected slots:
 	void dragTimeout();
 
 	// Automatio/curve node editor slots.
+	void editCurveNodeChanged();
 	void editCurveNodeFinished();
 
 private:
@@ -438,6 +452,7 @@ private:
 		DragClipPaste, DragClipPasteDrop,
 		DragClipFadeIn, DragClipFadeOut,
 		DragClipResizeLeft, DragClipResizeRight,
+		DragClipRepeatLeft, DragClipRepeatRight,
 		DragCurveMove, DragCurveStep,
 		DragCurvePaste, DragCurveNode
 	} m_dragState, m_dragCursor;
@@ -562,6 +577,7 @@ private:
 	qtractorCurve       *m_pEditCurve;
 	qtractorCurve::Node *m_pEditCurveNode;
 	QDoubleSpinBox      *m_pEditCurveNodeSpinBox;
+	int                  m_iEditCurveNodeDirty;
 };
 
 
