@@ -1,7 +1,7 @@
 // qtractorTracks.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -418,8 +418,8 @@ void qtractorTracks::updateContents ( bool bRefresh )
 
 	// Update dependant views.
 	if (iRefresh > 0) {
-		m_pTrackView->updateContentsWidth();
 		m_pTrackList->updateContentsHeight();
+		m_pTrackView->updateContentsWidth();
 	//	m_pTrackView->setFocus();
 	}
 }
@@ -487,8 +487,13 @@ bool qtractorTracks::newClip (void)
 		if (pSession->editTail() > pSession->editHead()) {
 			pClip->setClipLength(pSession->editTail() - pSession->editHead());
 		} else {
-			pClip->setClipLength(pSession->frameFromTick(
-				pSession->ticksPerBeat() * pSession->beatsPerBar()));
+			// Deafult length is one bar...
+			const unsigned long iClipStartTime
+				= pSession->tickFromFrame(iClipStart);
+			const unsigned long iClipLengthTime
+				= pSession->ticksPerBeat() * pSession->beatsPerBar();
+			pClip->setClipLength(pSession->frameFromTickRange(
+				iClipStartTime, iClipStartTime + iClipLengthTime));
 		}
 		// Proceed to setup the MDII clip properly...
 		qtractorMidiClip *pMidiClip
@@ -507,7 +512,7 @@ bool qtractorTracks::newClip (void)
 				pClipCommand->addClip(pClip, pTrack);
 				pSession->execute(pClipCommand);
 				// Just start the MIDI editor on it...
-				return pClip->startEditor(pMainForm);
+				return pClip->startEditor();
 			}
 		}
 	}
@@ -533,12 +538,8 @@ bool qtractorTracks::editClip ( qtractorClip *pClip )
 	if (pClip == NULL)
 		return false;
 
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return false;
-
 	// All else hasn't fail.
-	return pClip->startEditor(pMainForm);
+	return pClip->startEditor();
 }
 
 
@@ -1124,11 +1125,10 @@ struct audioClipBufferItem
 	// Constructor.
 	audioClipBufferItem(qtractorClip *pClip,
 		qtractorAudioBufferThread *pSyncThread,
-		unsigned short iChannels,
-		unsigned int iSampleRate)
+		unsigned short iChannels)
 		: clip(static_cast<qtractorAudioClip *> (pClip))
 	{
-		buff = new qtractorAudioBuffer(pSyncThread, iChannels, iSampleRate);
+		buff = new qtractorAudioBuffer(pSyncThread, iChannels);
 		buff->setOffset(clip->clipOffset());
 		buff->setLength(clip->clipLength());
 		buff->setTimeStretch(clip->timeStretch());
@@ -1201,7 +1201,7 @@ bool qtractorTracks::mergeExportAudioClips ( qtractorClipCommand *pClipCommand )
 		return false;
 	QString sFilename = fileDialog.selectedFiles().first();
 #endif
-	if (sFilename.isEmpty())
+	if (sFilename.isEmpty() || sFilename.at(0) == '.')
 		return false;
 	if (QFileInfo(sFilename).suffix().isEmpty())
 		sFilename += '.' + sExt;
@@ -1238,7 +1238,7 @@ bool qtractorTracks::mergeExportAudioClips ( qtractorClipCommand *pClipCommand )
 	const qtractorClipSelect::ItemList::ConstIterator& iter_end = items.constEnd();
 
 	const unsigned short iChannels = pAudioBus->channels();
-	const unsigned int iSampleRate = pSession->sampleRate();
+	const unsigned int iBufferSize = pSession->audioEngine()->bufferSize();
 
 	// Multi-selection extents (in frames)...
 	QList<audioClipBufferItem *> list;
@@ -1255,7 +1255,7 @@ bool qtractorTracks::mergeExportAudioClips ( qtractorClipCommand *pClipCommand )
 			if (iSelectEnd < pClip->clipSelectEnd())
 				iSelectEnd = pClip->clipSelectEnd();
 			list.append(new audioClipBufferItem(
-				pClip, pSyncThread, iChannels, iSampleRate));
+				pClip, pSyncThread, iChannels));
 		}
 	}
 
@@ -1270,7 +1270,6 @@ bool qtractorTracks::mergeExportAudioClips ( qtractorClipCommand *pClipCommand )
 	}
 
 	// Allocate merge audio scratch buffer...
-	const unsigned int iBufferSize = pSession->audioEngine()->bufferSize();
 	unsigned short i;
 	float **ppFrames = new float * [iChannels];
 	for (i = 0; i < iChannels; ++i)
@@ -1467,7 +1466,7 @@ bool qtractorTracks::mergeExportMidiClips ( qtractorClipCommand *pClipCommand )
 		return false;
 	QString sFilename = fileDialog.selectedFiles().first();
 #endif
-	if (sFilename.isEmpty())
+	if (sFilename.isEmpty() || sFilename.at(0) == '.')
 		return false;
 	if (QFileInfo(sFilename).suffix().isEmpty())
 		sFilename += '.' + sExt;
@@ -1713,7 +1712,7 @@ bool qtractorTracks::tempoClip ( qtractorClip *pClip )
 	unsigned long iRangeLength = pSession->editTail() - iRangeStart;
 
 	if (pClip == NULL)
-		pClip = currentClip();
+		pClip = m_pTrackView->currentClip();
 	if (pClip) {
 		if (pClip->isClipSelected()) {
 			iRangeStart  = pClip->clipSelectStart();
@@ -1749,6 +1748,17 @@ bool qtractorTracks::tempoClip ( qtractorClip *pClip )
 
 	// Done.
 	pSession->setAutoTimeStretch(bAutoTimeStretch);	
+
+	if (pClip) {
+		if (pClip->isClipSelected()) {
+			iRangeStart  = pClip->clipSelectStart();
+			iRangeLength = pClip->clipSelectEnd() - iRangeStart;
+		} else {
+			iRangeStart  = pClip->clipStart();
+			iRangeLength = pClip->clipLength();
+		}
+	}
+
 	pSession->setEditHead(iRangeStart);
 	pSession->setEditTail(iRangeStart + iRangeLength);
 
@@ -1756,6 +1766,72 @@ bool qtractorTracks::tempoClip ( qtractorClip *pClip )
 	return true;
 }
 
+// Auto-crossfade a give clip.
+bool qtractorTracks::crossFadeClip ( qtractorClip *pClip )
+{
+	if (pClip == NULL)
+		pClip = m_pTrackView->currentClip();
+	if (pClip == NULL)
+		return false;
+
+	qtractorTrack *pTrack = pClip->track();
+	if (pTrack == NULL)
+		return false;
+
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return false;
+
+	// We'll build a command...
+	qtractorClipCommand *pClipCommand
+		= new qtractorClipCommand(tr("clip cross-fade"));
+
+	const unsigned long iClipStart = pClip->clipStart();
+	const unsigned long iClipEnd = iClipStart + pClip->clipLength();
+
+	qtractorClip *pClip2 = pTrack->clips().first();
+
+	while (pClip2 && pClip2->clipStart() < iClipEnd) {
+		// Avoid cross-fading over the very self...
+		const unsigned long iClipStart2 = pClip2->clipStart();
+		const unsigned long iClipEnd2 = iClipStart2 + pClip2->clipLength();
+		if (iClipEnd2 > iClipStart && iClipStart > iClipStart2) {
+			const unsigned long iCrossFadeLength = iClipEnd2 - iClipStart;
+			if (pClip2->fadeOutLength() != iCrossFadeLength) {
+				pClipCommand->fadeOutClip(pClip2,
+					iCrossFadeLength, pClip2->fadeOutType());
+			}
+			if (pClip->fadeInLength() != iCrossFadeLength) {
+				pClipCommand->fadeInClip(pClip,
+					iCrossFadeLength, pClip->fadeInType());
+			}
+		}
+		else
+		if (iClipStart2 < iClipEnd && iClipEnd < iClipEnd2) {
+			const unsigned long iCrossFadeLength = iClipEnd - iClipStart2;
+			if (pClip->fadeOutLength() != iCrossFadeLength) {
+				pClipCommand->fadeOutClip(pClip,
+					iCrossFadeLength, pClip->fadeOutType());
+			}
+			if (pClip2->fadeInLength() != iCrossFadeLength) {
+				pClipCommand->fadeInClip(pClip2,
+					iCrossFadeLength, pClip2->fadeInType());
+			}
+		}
+		// Move forward...
+		pClip2 = pClip2->next();
+	}
+
+	// Check if valid...
+	if (pClipCommand->isEmpty()) {
+		delete pClipCommand;
+		return false;
+	}
+
+	// Make it undoable command
+	return pSession->execute(pClipCommand);
+}
+
 
 // Whether there's anything currently selected.
 bool qtractorTracks::isSelected (void) const
@@ -2468,12 +2544,19 @@ bool qtractorTracks::addTrack (void)
 		return false;
 	}
 
+	// Might take a while...
+	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+
 	// Take care of user supplied properties...
-	pTrack->properties() = trackForm.properties();
+	pTrack->setProperties(trackForm.properties());
 
 	// Put it in the form of an undoable command...
-	return pSession->execute(
+	const bool bResult = pSession->execute(
 		new qtractorAddTrackCommand(pTrack, currentTrack()));
+
+	QApplication::restoreOverrideCursor();
+
+	return bResult;
 }
 
 
@@ -2508,9 +2591,16 @@ bool qtractorTracks::removeTrack ( qtractorTrack *pTrack )
 			return false;
 	}
 
+	// Might take a while...
+	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+
 	// Put it in the form of an undoable command...
-	return pSession->execute(
+	const bool bResult = pSession->execute(
 		new qtractorRemoveTrackCommand(pTrack));
+
+	QApplication::restoreOverrideCursor();
+
+	return bResult;
 }
 
 
@@ -2541,15 +2631,70 @@ bool qtractorTracks::editTrack ( qtractorTrack *pTrack )
 	if (!trackForm.exec())
 		return false;
 
+	// Might take a while...
+	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+
 	// Put it in the form of an undoable command...
-	return pSession->execute(
+	const bool bResult = pSession->execute(
 		new qtractorEditTrackCommand(pTrack, trackForm.properties()));
+
+	QApplication::restoreOverrideCursor();
+
+	return bResult;
+}
+
+
+// Copy/duplicate given(current) track.
+bool qtractorTracks::copyTrack ( qtractorTrack *pTrack )
+{
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return false;
+
+	// Get the list view item reference of the intended track...
+	if (pTrack == NULL)
+		pTrack = currentTrack();
+	if (pTrack == NULL)
+		return false;
+
+	// Might take a while...
+	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+
+	// Duplicate into a new track...
+	const int iTrack = pSession->tracks().count() + 1;
+	const QColor& color = qtractorTrack::trackColor(iTrack);
+	qtractorTrack *pNewTrack = new qtractorTrack(pSession, pTrack->trackType());
+	pNewTrack->setProperties(pTrack->properties());
+	// Find an incremental/next track name...
+	QString sTrackName = pTrack->trackName();
+	QString sNewTrackName;
+	const QRegExp rxTrackNo("([0-9]+)$");
+	int iTrackNo = 0;
+	if (rxTrackNo.indexIn(sTrackName) >= 0) {
+		iTrackNo = rxTrackNo.cap(1).toInt();
+		sTrackName.remove(rxTrackNo);
+	}
+	else sTrackName += ' ';
+	do { sNewTrackName = sTrackName + QString::number(++iTrackNo); }
+	while (pSession->findTrack(sNewTrackName));
+	pNewTrack->setTrackName(sNewTrackName);
+	pNewTrack->setBackground(color);
+	pNewTrack->setForeground(color.darker());
+	pNewTrack->setZoomHeight(pTrack->zoomHeight());
+
+	// Put it in the form of an undoable command...
+	const bool bResult = pSession->execute(
+		new qtractorCopyTrackCommand(pNewTrack, pTrack));
+
+	QApplication::restoreOverrideCursor();
+
+	return bResult;
 }
 
 
 // Import Audio files into new tracks...
-bool qtractorTracks::addAudioTracks (
-	QStringList files, unsigned long iClipStart )
+bool qtractorTracks::addAudioTracks ( const QStringList& files,
+	unsigned long iClipStart, qtractorTrack *pAfterTrack )
 {
 	// Have we some?
 	if (files.isEmpty())
@@ -2566,7 +2711,7 @@ bool qtractorTracks::addAudioTracks (
 
 	// We'll build a composite command...
 	qtractorImportTrackCommand *pImportTrackCommand
-		= new qtractorImportTrackCommand(currentTrack());
+		= new qtractorImportTrackCommand(pAfterTrack);
 
 	// Increment this for suggestive track coloring...
 	int iTrack = pSession->tracks().count();
@@ -2642,8 +2787,8 @@ bool qtractorTracks::addAudioTracks (
 
 
 // Import MIDI files into new tracks...
-bool qtractorTracks::addMidiTracks (
-	QStringList files, unsigned long iClipStart )
+bool qtractorTracks::addMidiTracks ( const QStringList& files,
+	unsigned long iClipStart, qtractorTrack *pAfterTrack )
 {
 	// Have we some?
 	if (files.isEmpty())
@@ -2660,7 +2805,7 @@ bool qtractorTracks::addMidiTracks (
 
 	// We'll build a composite command...
 	qtractorImportTrackCommand *pImportTrackCommand
-		= new qtractorImportTrackCommand(currentTrack());
+		= new qtractorImportTrackCommand(pAfterTrack);
 
 	// Increment this for suggestive track coloring...
 	int iTrack = pSession->tracks().count();
@@ -2762,7 +2907,7 @@ bool qtractorTracks::addMidiTracks (
 
 // Import MIDI file track-channel into new track...
 bool qtractorTracks::addMidiTrackChannel ( const QString& sPath,
-	int iTrackChannel, unsigned long iClipStart )
+	int iTrackChannel, unsigned long iClipStart, qtractorTrack *pAfterTrack )
 {
 	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
@@ -2775,7 +2920,7 @@ bool qtractorTracks::addMidiTrackChannel ( const QString& sPath,
 
 	// We'll build a composite command...
 	qtractorImportTrackCommand *pImportTrackCommand
-		= new qtractorImportTrackCommand(currentTrack());
+		= new qtractorImportTrackCommand(pAfterTrack);
 
 	// Increment this for suggestive track coloring...
 	int iTrack = pSession->tracks().count();
