@@ -1,7 +1,7 @@
 // qtractorTracks.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -25,17 +25,15 @@
 #include <QSplitter>
 
 // Forward declarations.
-class qtractorInstrumentList;
-class qtractorTrackItemWidget;
 class qtractorTrackList;
 class qtractorTrackTime;
 class qtractorTrackView;
 class qtractorTrack;
-class qtractorSession;
 class qtractorClip;
 class qtractorClipCommand;
 class qtractorClipRangeCommand;
 class qtractorClipToolCommand;
+
 class qtractorMidiToolsForm;
 
 
@@ -67,12 +65,15 @@ public:
 	bool addTrack();
 	bool removeTrack(qtractorTrack *pTrack = NULL);
 	bool editTrack(qtractorTrack *pTrack = NULL);
+	bool copyTrack(qtractorTrack *pTrack = NULL);
 
 	// Import Audio/MIDI files into new tracks...
-	bool addAudioTracks(QStringList files, unsigned long iClipStart = 0);
-	bool addMidiTracks(QStringList files, unsigned long iClipStart = 0);
+	bool addAudioTracks(const QStringList& files,
+		unsigned long iClipStart, qtractorTrack *pAfterTrack = NULL);
+	bool addMidiTracks(const QStringList& files,
+		unsigned long iClipStart, qtractorTrack *pAfterTrack = NULL);
 	bool addMidiTrackChannel(const QString& sPath, int iTrackChannel,
-		unsigned long iClipStart = 0);
+		unsigned long iClipStart, qtractorTrack *pAfterTrack = NULL);
 
 	// Track-list active maintenance update.
 	void updateTrack(qtractorTrack *pTrack = NULL);
@@ -91,6 +92,7 @@ public:
 	bool rangeClip(qtractorClip *pClip = NULL);
 	bool loopClip(qtractorClip *pClip = NULL);
 	bool tempoClip(qtractorClip *pClip = NULL);
+	bool crossFadeClip(qtractorClip *pClip = NULL);
 	bool executeClipTool(int iTool, qtractorClip *pClip = NULL);
 	bool importClips(QStringList files, unsigned long iClipStart = 0);
 	bool exportClips();
